<?php
/*******************************************************************************
 *	BLogin.php		Login and permittions manager
 *	Copyright		Terry Barnaby,	BeamWeb,	2006-05-08
 *******************************************************************************
 */

function loginFunction1(){
	echo "<form method='post' action='index.php'>";
	echo "<input type='text' username='username'>";
	echo "<input type='password' name='password'>";
	echo "<input type='submit'>";
	echo "</form>";
}

function loginFunction2(){
	echo "<H1>Please Login to the system</H1>";
}

class BLogin {
	function __construct($sys, $loginFunction, $showLogin = TRUE){
		$this->sys = $sys;
		$this->showLogin = $showLogin;
		$this->loginFunction = $loginFunction;
		$this->userName = "";
		$this->id = 0;
		$this->enabled = 0;
		$this->fixed = 0;
		$this->roles = "";
		$this->sessionVariable = "{$this->sys->siteName}_userId";
	}
	
	// Check user/password using form
	function performLogin($roles = ""){
		//print("BLogin::performLogin: '" . $_POST["username"] . "'<br>");
		//$session = $_SESSION[$this->sessionVariable]; debug_log("BLogin::performLogin: {$this->sessionVariable} {$session}");

		// Check if already logged in
		if(isset($_SESSION[$this->sessionVariable])){
			$user = $this->sys->db->getObject("User", filterToInt($_SESSION[$this->sessionVariable]));
		}
		else if(isset($_POST["username"]) && isset($_POST["password"])){
			// Check password. This allows the old md5 style passwords as well as the newe password_hash() generated ones
			$user = $this->getUser(filterString($_POST["username"]));
			$password = filterString($_POST["password"]);
			
			//print("Check password: '" . $password . "' to '" . $user->userPassword . "'</br>");
			if(($user == Null) || !password_verify($password, $user->userPassword) && ($user->userPassword != md5($password))){
				sleep(1);

				if($this->showLogin && ($this->loginFunction != "")){
					call_user_func($this->loginFunction, "Your username and/or password is incorrect");
					exit();
				}
				return False;
			}
			$_SESSION[$this->sessionVariable] = $user->id;
		}
		else if(0 && isset($_SERVER["PHP_AUTH_USER"]) && isset($_SERVER["PHP_AUTH_PW"])){
			// Check password. This allows the old md5 style passwords as well as the new password_hash() generated ones
			$user = $this->getUser(filterString($_SERVER["PHP_AUTH_USER"]));
			$password = filterString($_SERVER["PHP_AUTH_PW"]);
			
			//print("Check password: '" . $password . "' to '" . $user->userPassword . "'</br>");
			if(!password_verify($password, $user->userPassword) && ($user->userPassword != md5($password))){
				header("WWW-Authenticate: Basic realm='{$this->sys->siteName}'");
				header("HTTP/1.0 401 Unauthorized");
				echo "Error 401 - Access denied";
				exit();
			}
			$_SESSION[$this->sessionVariable] = $user->id;
		}
		else {
			//header("WWW-Authenticate: Basic realm='{$this->sys->siteName}'");

			if($this->showLogin && ($this->loginFunction != "")){
				http_response_code(401);
				call_user_func($this->loginFunction);
				exit();
			}
			return False;
		}
		
		if($user){
			$this->id = $user->id;
			$this->userName = $user->userName;
			$this->roles = $user->roles;
			$this->enabled = $user->enabled;
			$this->fixed = $user->fixed;
			$this->group = $user->group;
			$this->groups = $user->groups;
			$this->membershipId = $user->membershipId;
			if($this->membershipId){
				if($this->roles != "")
					$this->roles .= ",member";
				else
					$this->roles .= "member";
			}

			if($this->enabled == 0){
				$this->logout();
				if($this->showLogin && ($this->loginFunction != "")){
					call_user_func($this->loginFunction, "Your account is disabled. Please contact the sites administrators");
					exit();
				}
				return False;
			}

			if($roles == "any"){
				if($this->roles == "")
					return False;
			}
			else if($roles != ""){
				$r1 = explode(",", $roles);
				$r2 = explode(",", $this->roles);
				$r = array_intersect($r1, $r2);
				if(count($r) == 0)
					return False;
			}
			
			return True;
		}
		else {
			return False;
		}
	}

	// Check user/password using form
	function checkLogin($roles = ""){
		//print("BLogin::checkLogin: '" . $_POST["username"] . "'<br>");

		// Check if already logged in
		if(isset($_SESSION[$this->sessionVariable])){
			$user = $this->sys->db->getObject("User", filterToInt($_SESSION[$this->sessionVariable]));
			if($user){
				$this->id = $user->id;
				$this->userName = $user->userName;
				$this->roles = $user->roles;
				$this->enabled = $user->enabled;
				$this->fixed = $user->fixed;
				$this->group = $user->group;
				$this->groups = $user->groups;
				$this->membershipId = $user->membershipId;
				if($this->membershipId){
					if($this->roles != "")
						$this->roles .= ",member";
					else
						$this->roles .= "member";
				}

				if($this->enabled == 0){
					$this->logout();
					if($this->showLogin && ($this->loginFunction != "")){
						call_user_func($this->loginFunction, "Your account is disabled. Please contact the sites administrators");
						exit();
					}
					return False;
				}

				if($roles == "any"){
					if($this->roles == "")
						return False;
				}
				else if($roles != ""){
					$r1 = explode(",", $roles);
					$r2 = explode(",", $this->roles);
					$r = array_intersect($r1, $r2);
					if(count($r) == 0)
						return False;
				}

				return True;
			}
		}
		return False;
	}

	// Check user/password using HTTP
	function performLoginHttp($roles = ""){
		// Check if already logged in
		if(isset($_SESSION[$this->sessionVariable])){
			$user = $this->sys->db->getObject("User", filterToInt($_SESSION[$this->sessionVariable]));
		}
		else if(isset($_SERVER["PHP_AUTH_USER"]) && isset($_SERVER["PHP_AUTH_PW"])){
			// Check password. This allows the old md5 style passwords as well as the new password_hash() generated ones
			$user = $this->getUser(filterString($_SERVER["PHP_AUTH_USER"]));
			$password = filterString($_SERVER["PHP_AUTH_PW"]);
			
			//print("Check password: '" . $password . "' to '" . $user->userPassword . "'</br>");
			if(!password_verify($password, $user->userPassword) && ($user->userPassword != md5($password))){
				header("WWW-Authenticate: Basic realm='{$this->sys->siteName}'");
				header("HTTP/1.0 401 Unauthorized");
				echo "Error 401 - Access denied";
				exit();
			}
			$_SESSION[$this->sessionVariable] = $user->id;
		}
		else {
			header("WWW-Authenticate: Basic realm='{$this->sys->siteName}'");
			header("HTTP/1.0 401 Unauthorized");
			echo "Error 401 - Access denied";
			exit();
		}
		
		if($user){
			$this->id = $user->id;
			$this->userName = $user->userName;
			$this->roles = $user->roles;
			$this->enabled = $user->enabled;
			$this->fixed = $user->fixed;
			$this->group = $user->group;
			$this->groups = $user->groups;
			$this->membershipId = $user->membershipId;
			if($this->membershipId){
				if($this->roles != "")
					$this->roles .= ",member";
				else
					$this->roles .= "member";
			}

			if($this->enabled == 0){
				$this->logout();
				return False;
			}

			if($roles == "any"){
				if($this->roles == "")
					return False;
			}
			else if($roles != ""){
				$r1 = explode(",", $roles);
				$r2 = explode(",", $this->roles);
				$r = array_intersect($r1, $r2);
				if(count($r) == 0)
					return False;
			}
			
			return True;
		}
		else {
			return False;
		}
	}

	function getUser($username){
		$users = $this->sys->db->getObjectList("User", "", "`userName` = '{$username}'", "");
		if(count($users)){
			return $users[0];
		}
		else {
			return Null;
		}
	}

	function logout(){
		unset($_SESSION[$this->sessionVariable]);
	}
};

?>
