<?php
/*******************************************************************************
 *	BObjPicture.php		Picture handling
 *	Copyright:		Terry Barnaby,	BeamWeb,	2017-05-15
 *******************************************************************************
 */

global	$sys;
$sys->db->addObjectClasses(array("Picture" => "BObjPicture"));

class BObjPicture extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("source", "String"),
			new BMember("sourceId", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("good", "Int"),
			new BMember("userId", "Int", "Number"),
			new BMember("date", "Date"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

function pictureSave($id, $fileName){
	global $sys;

//	debug_log("pictureSave: {$id} {$fileName}\n");

	$maxsize = $sys->pictureWidth;
	$thumbSize = $sys->pictureThumbWidth;
	
	$image = imagecreatefromjpeg($fileName);
	
	// Rotate image if needed
	$exif = exif_read_data($fileName);
	if(!empty($exif['Orientation'])) {
		switch($exif['Orientation']){
		case 8:
			$image = imagerotate($image, 90,0);
			break;
		case 3:
			$image = imagerotate($image,180,0);
			break;
		case 6:
			$image = imagerotate($image,-90,0);
			break;
		}
	}

	$x = imagesx($image);
	$y = imagesy($image);
	$aspect = (float)$x / (float)$y;
	
//	debug_log("pictureSave: original: {$x} x {$y}\n");
	if($aspect > 1.0){
		$w = $x;
		if($w > $maxsize)
			$w = $maxsize;
		$h = (int)((float)$w/$aspect);
		$tw = $thumbSize;
		$th = (int)((float)$tw/$aspect);
	}
	else {
		$h = $y;
		if($h > $maxsize)
			$h = $maxsize;
		$w = (int)((float)$h * $aspect);
		$th = $thumbSize;
		$tw = (int)((float)$th * $aspect);
	}
//	debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
	$pic = imagecreatetruecolor($w, $h);
	$thumb = imagecreatetruecolor($tw, $th);
	imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $x, $y);
	imagecopyresized($thumb, $image, 0, 0, 0, 0, $tw, $th, $x, $y);
	imagejpeg($pic, $sys->baseDir . "/files/pictures/picture-{$id}.jpg");
	imagejpeg($thumb, $sys->baseDir . "/files/pictures/picture-{$id}-thumb.jpg");
}

function pictureSaveDir($filename, $type, $dirname, $id, $thumbWidth = 128, $savePng = 0){
	global $sys;

	//debug_log("pictureSaveDir: {$fileName} {$type} {$dirname} {$id}");

	$maxsize = $sys->pictureWidth;
	$s = getimagesize($filename);
	$aspect = (float)$s[0] / (float)$s[1];
	
	//debug_log("pictureSave: original: {$s[0]} x {$s[1]}\n");
	if($aspect > 1.0){
		$w = $s[0];
		if($w > $maxsize)
			$w = $maxsize;
		$h = (int)((float)$w/$aspect);
		$tw = $thumbWidth;
		$th = (int)round((float)$tw/$aspect);
	}
	else {
		$h = $s[1];
		if($h > $maxsize)
			$h = $maxsize;
		$w = (int)((float)$h * $aspect);
		$tw = $thumbWidth;
		$th = (int)round((float)$tw/$aspect);
	}

	//debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
	//print "Stype: '{$type}'<br>";

	if(($type == "jpg") || ($type == "jpeg"))
		$image = imagecreatefromjpeg($filename);
	else if($type == "png")
		$image = imagecreatefrompng($filename);
	else if($type == "gif")
		$image = imagecreatefromgif($filename);
		
	$pic = imagecreatetruecolor($w, $h);
	$thumb = imagecreatetruecolor($tw, $th);

	imagealphablending($pic, false);
	imagesavealpha($pic, true);
	imagealphablending($thumb, false);
	imagesavealpha($thumb, true);

	imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
	imagecopyresized($thumb, $image, 0, 0, 0, 0, $tw, $th, $s[0], $s[1]);
	
	if($savePng){
		imagepng($pic, $sys->baseDir . "/" . $dirname . "/picture-{$id}.png");
		imagepng($thumb, $sys->baseDir . "/" . $dirname . "/picture-{$id}-thumb.png");
	}
	else {
		imagejpeg($pic, $sys->baseDir . "/" . $dirname . "/picture-{$id}.jpg");
		imagejpeg($thumb, $sys->baseDir . "/" . $dirname . "/picture-{$id}-thumb.jpg");
	}
}

function pictureUrl($id = 0, $source = "", $sourceId = 0, $thumb = 0){
	global $sys;

	$select = "";

	if($id){
		$obj = $sys->db->getObject("Picture", $id);
	}
	else {
		if($source){
			$source = dbEscapeString($source);
			$select = "`source` = '{$source}'";
		}
		if($sourceId){
			$sourceId = dbEscapeString(strval($sourceId));
			if($select != "")
				$select .= " and ";
			$select = "`sourceId` = '{$sourceId}'";
		}
		$list = $sys->db->getObjectList("Picture", "", $select, "id", "1");
		if(count($list)){
			$obj = $list[0];
		}
		else {
			$obj = new ObjLocal;
			$obj->id = 0;
			$obj->title = "No image";
		}
	}
	
	if($thumb)
		$html = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}-thumb.jpg";
	else
		$html = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}.jpg";

	return $html;
}

function pictureHtml($id = 0, $source = "", $sourceId = 0, $thumb = 0, $noLink = 0, $args = ""){
	global $sys;
	
	//print "pictureHtml: {$source} - {$sourceId}<br>";

	$select = "";

	if($id){
		$obj = $sys->db->getObject("Picture", $id);
	}
	else {
		if($source){
			$source = dbEscapeString($source);
			$select = "`source` = '{$source}'";
		}
		if($sourceId){
			$sourceId = dbEscapeString(strval($sourceId));
			if($select != "")
				$select .= " and ";
			$select = "`sourceId` = '{$sourceId}'";
		}
		$list = $sys->db->getObjectList("Picture", "", $select, "id", "1");
		if(count($list)){
			$obj = $list[0];
		}
		else {
			$obj = new ObjLocal;
			$obj->id = 0;
			$obj->title = "No image";
		}
	}
	
	if($thumb)
		$url = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}-thumb.jpg";
	else
		$url = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}.jpg";
	
	if($noLink){
		$html = "<img alt='{$obj->title}' title='{$obj->title}' src='{$url}' {$args} />";
	}
	else {
		$html = "<a href='{$sys->baseUrl}/files/pictures/picture-{$obj->id}.jpg'><img alt='{$obj->title}' title='{$obj->title}' src='{$url}' {$args} /></a>";
	}
	
	return $html;
}


?>
