<?php
/*******************************************************************************
 *	BPage.php		Main page output
 *				Generates a HTML page for display
 *	Copyright		Terry Barnaby,	BeamWeb,	2008-02-08
 *******************************************************************************
 */
require_once "BMenu.php";
require_once "BMenuRender.php";


class BPage {
	function __construct($sys){
		$this->sys = $sys;
		$this->leftObjects = array();
		$this->setLogo($sys->logo);
		$this->setTitle($sys->siteTitle);
		$this->template = Null;
		$this->print = 0;
		$this->popup = 0;
		$this->aditemsShow = 0;
		$this->isHomepage = 0;
		$this->styleSheets = "";
	}
	
	// Create standard layout
	function init(){
	}

	function setTemplate($template){
		$this->template = $template;
	}

	function setLogo($logo){
		$this->logo = $logo;

		if($logo[0] == '/')
			$this->logoHtml = "{$this->sys->baseUrl}$logo";
		else if(file_exists("{$this->sys->baseDir}/files/themes/{$this->sys->style}/$logo"))
			$this->logoHtml = "{$this->sys->baseUrl}/files/themes/{$this->sys->style}/$logo";
		else if(file_exists("{$this->sys->baseDir}/themes/{$this->sys->style}/$logo"))
			$this->logoHtml = "{$this->sys->baseUrl}/themes/{$this->sys->style}/$logo";
		else if(file_exists("{$this->sys->baseDir}/icons/$logo"))
			$this->logoHtml = "{$this->sys->baseUrl}/icons/$logo";
		else
			$this->logoHtml = $logo;
	}
	
	function setTitle($title){
		$this->title = $title;
	}

	function setMenuBar($menu){
		$this->menuMain = $menu;
		$this->menuMain->rebaseMenu();
		$this->menuBarHtml = menuBarRender($this->menuMain);
	}

	function setStatus($status){
		$this->status = $status;
	}

	function leftObjectInsert($obj){
		if($obj instanceof BMenu)
			$obj->rebaseMenu();
		array_splice($this->leftObjects, 0, 0, array($obj));
	}

	function setCentre($center){
		$this->center = $center;
	}
	
	function addStyleSheet($styleSheet){
		$this->styleSheets .= "<link href=\"{$this->templateUrl}/{$styleSheet}\" rel=\"stylesheet\" type=\"text/css\" />\n";
	}
	
	// Render the page
	function render(){
	}

	
	// Override standard layout bits
	function setTop($top){
		$this->top = $top;
	}
	
	function setTopMenu($topMenu){
		$this->topMenu = $topMenu;
	}
	
	function setLeftSide($leftSide){
		$this->leftSide = $leftSide;
	}
	
	function setBottom($bottom){
		$this->bottom = $bottom;
	}
	

	// Page settings
	var	$sys;								// The overal state
	var	$logo;								// The Logo to use
	var	$title;								// The page title
	var	$mainMenu;							// The main menu
	var	$location;
	var	$leftObjects;							// The list of objects on the LHS
	var	$styleSheets;							// Additional style sheets

	// Special functions
	var	$template;			// The page template to use. Overrides default.
	var	$print;				// Sets page for printing
	var	$popup;				// Sets page for popup

	// The page elements
	var	$top;
	var	$topMenu;
	var	$status;
	var	$leftSide;
	var	$center;
	var	$bottom;
	var	$rightSide;
	
	// Internal bits
	var	$logoHtml;
	var	$menuBarHtml;							// The Rendered menubar
};

class BPageTemplate extends BPage {
	function __construct($sys){
		parent::__construct($sys);

		if($this->sys->style == Null){
			$this->templateDir = "{$this->sys->baseDir}/themes/default";
			$this->templateUrl = "{$this->sys->baseUrl}/themes/default";
		}
		else if(file_exists("{$this->sys->baseDir}/files/themes/{$this->sys->style}")){
			$this->templateDir = "{$this->sys->baseDir}/files/themes/{$this->sys->style}";
			$this->templateUrl = "{$this->sys->baseUrl}/files/themes/{$this->sys->style}";
		}
		else {
			$this->templateDir = "{$this->sys->baseDir}/themes/{$this->sys->style}";
			$this->templateUrl = "{$this->sys->baseUrl}/themes/{$this->sys->style}";
		}
	}
	
	function doLeftSide(){
		$str = "<div>\n";
		$str .= "<button class='LefthandHideButton' onClick='leftside_toggle();'><img src='/icons/general/Close.svg'/>Hide menu</button>";
		foreach($this->leftObjects as $obj){
			$str .= "<div class='MenuLeftBlock'>\n";
			if($obj->title != "")
				$str .= "<h2>{$obj->title}</h2>\n";
			if(get_class($obj) == "BMenu"){
				foreach($obj as $m){
					if($m->url != "")
						$str .= "<div class='LeftMenuEntry'><a href='{$m->url}' {$m->args}>{$m->title}</a></div>\n";
					else
						$str .= "<div class='LeftMenuEntry'><a {$m->args}>{$m->title}</a></div>\n";
				}
			}
			else {
				$str .= "<div class='LeftMenuEntries'>{$obj->render()}</div>\n";
			}
			$str .= "</div>\n";
		}
		$str .= "</div>\n";
		$this->setLeftSide($str);
	}

	function render(){
		$template = "";
		
		$this->doLeftSide();
		
		// Create aditems display if wanted and available
		if($this->aditemsShow && array_key_exists("aditem", $this->sys->modules)){
			$this->rightSide = aditemListVertical();
		}

		if($this->template && file_exists("{$this->templateDir}/{$this->template}")){
			$template = file_get_contents("{$this->templateDir}/{$this->template}");
		}
		else if($this->print){
			print $this->center;
		}
		else if($this->popup){
			if($this->sys->isMobile && file_exists("{$this->templateDir}/popup-mobile.template"))
				$template = file_get_contents("{$this->templateDir}/popup-modile.template");
			else
				$template = file_get_contents("{$this->templateDir}/popup.template");
		}
		else {
			if($this->sys->isMobile && file_exists("{$this->templateDir}/page-mobile.template")){
				$template = file_get_contents("{$this->templateDir}/page-mobile.template");
			}
			else {
				$template = file_get_contents("{$this->templateDir}/page.template");
			}
		}
		
		if($template != ""){		
			$text = "";
			$pos = 0;
			while(1){
				if(!($start = strpos($template, "{", $pos))){
					$text .= substr($template, $pos);
					break;
				}
				if(!($end = strpos($template, "}", $start))){
					$text .= substr($template, $pos);
					break;
				}
				
				if($template[$start + 1] == "{"){
					$text .= substr($template, $pos, $start - $pos);
					$text .= substr($template, $start + 1, $end - $start);
					$pos = $end + 2;
				}
				else {
					$text .= substr($template, $pos, $start - $pos);
					$str = substr($template, $start + 1, $end - $start - 1);
					// print "Parse: '$str'\n";
					if(1){
						if($str[0] == "$")
							$str = "print({$str});";

						ob_start();
						eval($str);
						$text .= ob_get_contents();
						ob_end_clean();
					}
					else {
						eval('$s = ' . $str . ';');
						$text .= $s;
					}
					$pos = $end + 1;
				}
			}
			print "$text";
		}
	}
};

?>
