<?php
/*******************************************************************************
 *	BSys.php		The main system class of WebSys
 *	Copyright:		Terry Barnaby,	BeamWeb,	2009-03-18
 *******************************************************************************
 */
require_once("BConfig.php");
require_once("BCommon.php");
require_once("BDatabase.php");
require_once("BPage.php");
require_once("BLogin.php");
require_once("BDirectoryLister.php");
require_once("BHtml.php");
require_once("Mobile_Detect/Mobile_Detect.php");

function debug_error($message){
	global	$sys;

	$message = "Error: " . $message;
	$f = fopen($sys->baseDir . "/error_log.txt", "a");
	fputs($f, date(DATE_ISO8601) . ": " . $message . "\n"); 
	fclose($f);
	error_log($message);
}

function debug_log($message){
	global	$sys;
	
	$message = "Debug: " . $message;
	$f = fopen($sys->baseDir . "/error_log.txt", "a");
	fputs($f, date(DATE_ISO8601) . ": " . $message . "\n"); 
	fclose($f);
	error_log($message);
}

function debug_log_backtrace($message){
	debug_log($message . print_r(debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS), 1));
}

function error_handler($level, $message, $file, $line, $context){
	global	$sys;

	$message = "Error: " . $message;
	$f = fopen($sys->baseDir . "/error_log.txt", "a");
	fputs($f, date(DATE_ISO8601) . ": " . $message . "\n"); 
	fclose($f);

	if(!$sys->debug)	
		$message = "Error: an error occured that has been logged";

	$sys->pageDisplayError(new BError(500, $message));
	die();
}

function baseUrl($url){
	global $sys;
	
	return $sys->baseUrl . "/" . $url;
}

function baseDir($url){
	global $sys;
	
	return $sys->baseDir . "/" . $url;
}

function fileErrorString($e){
	switch($e){
	case UPLOAD_ERR_OK:
		$message = "Ok";
		break;
	case UPLOAD_ERR_INI_SIZE:
		$message = "The file exceeds the upload_max_filesize parameter";
		break;
	case UPLOAD_ERR_FORM_SIZE:
		$message = "The file exceeds the MAX_FILE_SIZE parameter";
		break;
	case UPLOAD_ERR_PARTIAL:
		$message = "The file was only partially uploaded";
		break;
	case UPLOAD_ERR_NO_FILE:
		$message = "No file was uploaded";
		break;
	case UPLOAD_ERR_NO_TMP_DIR:
		$message = "Missing a temporary folder";
		break;
	case UPLOAD_ERR_CANT_WRITE:
		$message = "Failed to write file to disk";
		break;
	case UPLOAD_ERR_EXTENSION:
		$message = "File upload stopped by extension";
		break;
	default:
		$message = "Unknown file upload error";
		break;
	}
	return $message;
}

// Filter user input
function canonicalizePath($path){
	$path = explode('/', $path);
	$stack = array();
	foreach($path as $seg){
		if($seg == '..'){
			// Ignore this segment, remove last segment from stack
			array_pop($stack);
			continue;
		}

		if($seg == '.'){
			// Ignore this segment
			continue;
		}

		$stack[] = $seg;
	}

	return implode('/', $stack);
}

// Returns the local filepath given a local URL path
function filePathLocal($url){
	global	$sys;
	
	$url = trim($url, "/ ");
	//if(strncmp($url, "files/", 6) && strncmp($url, "modules/", 8))
	if(strncmp($url, "files/", 6))
		$url = "files/" . $url;

	if(0){
		if($sys->basePath)
			return $sys->basePath . "/" . $url;
		else
			return $url;
	}
	else {
		return $url;
	}
}

// Returns the absolute filepath given a local URL path
function filePathAbs($url){
	global	$sys;
	
	$url = trim($url, "/ ");
	//if(strncmp($url, "files/", 6) && strncmp($url, "modules/", 8))
	if(strncmp($url, "files/", 6))
		$url = "files/" . $url;

	return $sys->baseDir . "/" . $url;
}

// Returns the URL path given a local URL path
function urlPath($url){
	global	$sys;
	
	$url = trim($url, "/ ");

	if($sys->basePath)
		return $sys->basePath . "/". $url;
	else
		return "/" . $url;
}

function stringFindCharacterPos($str, $set){
	for($i = 0; $i < strlen($str); $i++){
		$c = $str[$i];
		if(strpos($set, $str[$i]) !== False){
			return $i;
		}
	}
	return Null;
}


// Validation functions
function validateString($v, $args = Null){
	return (True);
}

function validateStringNoSpaces($v, $args = Null){
	return (false !== filter_var($v, FILTER_VALIDATE_REGEXP, array("options"=>array("regexp"=>"|^[A-Za-z0-9_]*$|"))));
}

function validateInt($v, $args = Null){
	return (false !== filter_var($v, FILTER_VALIDATE_INT));
}

function validateFloat($v, $args = Null){
	return (false !== filter_var($v, FILTER_VALIDATE_FLOAT));
}

function validateDate($v, $args = Null){
	$format = "Y-m-d";
	$d = DateTime::createFromFormat($format, $v);
	return $d && $d->format($format) === $v;
}

function validateDateTime($v, $args = Null){
	if(($v == "0000-00-00T00:00:00") || ($v == "0000-00-00 00:00:00") || ($v == "")){
		return True;
	}

	$format1 = "Y-m-d H:i:s";
	$format2 = "Y-m-d\TH:i:s";
	$d = DateTime::createFromFormat($format1, $v);
	if($d && $d->format($format1) === $v)
		return True;

	$d = DateTime::createFromFormat($format2, $v);
	return ($d && $d->format($format2) === $v);
}

function validateEmail($v, $args = Null){
	return (false !== filter_var($v, FILTER_VALIDATE_EMAIL));
}

// Filter value functions
// Should these return the correct type ??
function filterInt($str){
	return strval(intval($str));
}

function filterToInt($str){
	return intval($str);
}

function filterFloat($str){
	return strval(filter_var($str, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_SCIENTIFIC));
}

function filterToFloat($str){
	return filter_var($str, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION | FILTER_FLAG_ALLOW_SCIENTIFIC);
}

function filterString($str){
	return filter_var($str, FILTER_SANITIZE_STRING);
}

function filterStringRemoveSpaces($str){
	return str_replace(" ", "_", filter_var($str, FILTER_SANITIZE_STRING));
}

function filterStringDate($str){
	return preg_replace('/[^0-9\-\:\/Tt\s]/', '', $str);
}

function filterStringHtml($str){
	return filter_var($str, FILTER_SANITIZE_FULL_SPECIAL_CHARS);
}

function filterStringParam($str){
	return preg_replace('/[^A-Za-z0-9_\/\.\!]/', '', $str);
}

function filterStringParamList($str){
	return preg_replace('/[^A-Za-z0-9_\/\.\!,]/', '', $str);
}

function filterArray($a){
	$anew = array();
	
	foreach($a as $v)
		$anew[] = filter_var($v, FILTER_SANITIZE_STRING);

	return $anew;
}


function filterEmail($str){
	return filter_var($str, FILTER_SANITIZE_EMAIL);
}

function filterUrl($str){
	return filter_var($str, FILTER_SANITIZE_URL);
}

function filterPath($str, $baseDir = ""){
	$pos = stringFindCharacterPos($str, "?&");
	if($pos != Null)
		$str = substr($str, 0, $pos);
	$str = preg_replace('/[^A-Za-z0-9_\-\/\.() ]/', '', $str);
	$str = canonicalizePath($str);
	
	return $str;
}

function filterPathRelative($str, $baseDir = ""){
	global	$sys;

	// Remove basePath if needed
	$basePath = $sys->basePath;
	if(!strncmp($str,$basePath, strlen($basePath)))
		$str = substr($str, strlen($basePath));

	$pos = stringFindCharacterPos($str, "?&");
	if($pos != Null)
		$str = substr($str, 0, $pos);

	$str = urldecode($str);
//	$str = preg_replace('/[^A-Za-z0-9_\-\/\.() ]/', '', $str);
	$str = preg_replace('/^\/+/', '', canonicalizePath($str));
	
	return $str;
}


function addItem($icon, $title, $description, $url, $subItems){
	$h = new BHtmlNode("div", "class='adminItem'");
	$t = &$h->appendNode("table");
	$r = &$t->appendNode("tr");
	$c = &$r->appendNode("td");
	$c->appendNode("a", "href='{$url}'")->appendNode("img", "src='{$icon}'");
	$c = &$r->appendNode("td");
	$c->appendNode("a", "href='{$url}'", $title);
	$c->appendNode("br");
	$c->appendText($description);
	return $h->render();
}

abstract class AccessMode {
	const Anyone	= 0;
	const LoggedIn	= 1;
	const Member	= 2;
}

class BSys extends BConfig {
	function __construct($config){
		set_error_handler("error_handler", E_USER_ERROR);
		
		foreach($config as $key => $value){
			$this->$key = $value;
		}
		$this->config = array();
		
		//BDatabase::$debug = 1;
	}
	
	function init(){
		date_default_timezone_set('Europe/London');
	
		// Start off session
		if($this->forceHttps)
			ini_set("session.cookie_secure", "On");

		ini_set("session.cookie_httponly", "On");
		ini_set("session.cookie_samesite", "Strict");

		//ini_set("session.gc_maxlifetime", 30);
		//session_cache_limiter("private, must-revalidate");		// Set to nocache by default. How to handle pictures/files caching ?
		//session_cache_expire(30);
		//$cache_expire = session_cache_expire();
		session_start();

		if(isset($_SESSION["style"])){
			$this->style = $_SESSION["style"];
		}
		if(isset($_SESSION["device"])){
			$this->device = $_SESSION["device"];
		}

		// Create main page
		$this->page = new BPageTemplate($this);
		
		// Create database connection
		$this->db = new BDataBase($this);
		if($err = $this->db->open())
			$this->pageDisplayError($err);


		// Load core modules
		require_once "management/management.php";
		$this->modules["management"] = new Management($this);
		require_once "config/config.php";
		$this->modules["config"] = new Config($this);
		$this->modules["config"]->read();			// Load configuration
		require_once "category/category.php";
		$this->modules["category"] = new Category($this);
		require_once "user/user.php";
		$this->modules["user"] = new User($this);
		require_once "content/content.php";
		$this->modules["content"] = new Content($this);
		require_once "contentDir/contentDir.php";
		$this->modules["contentDir"] = new ContentDir($this);
		require_once "statistics/statistics.php";
		$this->modules["statistics"] = new Statistics($this);

		// Check if main database tables are present, if not create them
		if(!$this->db->tableExists("Statistics")){
			$page = $this->modules["management"]->install();
			$this->pageDisplay("Initial Database Install", $page);
			exit(0);
		}

		// Set initial user roles
		if($this->userRoles != "")
			$this->modules["user"]->setRoles($this->userRoles);

		// Load the required modules
		$this->modulesLoad();

		// Update Statistics
		//statisticsClean();
		//statisticsAdd();
	}

	function process($uri){
		// Clean up URI removing .. and absolute path
		$uriParsed = parse_url($uri);
		$this->uri = $uri = filterPathRelative($uriParsed["path"], $this->baseDir);
		
		// Remove old "files/" prefix
		if(0 && !strncmp($uri, "files/", 6)){
			$uri = substr($uri, 6);
			header ("Location: /{$uri}");
		}

//		print "BSys::process: uri: {$uri}<br>";
//		print "baseUri: {$this->baseUrl} basePath: {$this->basePath} baseDir: {$this->baseDir}<br>";
		
		// Findout device type
		if($this->device){
			if($this->device == "mobile")
				$this->isMobile = 1;
		}
		else {
			$detect = new Mobile_Detect;
			$this->isMobile = $detect->isMobile();
			if($this->isMobile)
				$this->device = "mobile";
			else
				$this->device = "pc";
		}

		$userAgent = $_SERVER["HTTP_USER_AGENT"];
		if(stripos($userAgent, "Firefox") !== false){
			$this->browser = "Firefox";
		}
		else if(stripos($userAgent, "Chrome") !== false){
			$this->browser = "Chrome";
		}
		elseif (stripos($userAgent, "Safari") !== false){
			$this->browser = "Safari";
		}
		else {
			$this->browser = "";
		}
		
		//debug_log("Browser: '{$browser}': {$userAgent}");
		
		// Get Login info
		$this->login = new BLogin($this, "pageLogin", false);
		$this->login->performLogin();

		$this->createMenuBar();

		if(array_key_exists("picturesRandom", $this->modules))
			$this->page->leftObjectInsert($this->modules["picturesRandom"]->getPageBlock($uri));

		if($this->loginLeftSide){
			// Add user login info
			$this->page->leftObjectInsert($this->modules["user"]->getLoginBlock($uri));
		}
		
		//debug_log("BSys: Access: '{$uri}' cmd: '{$cmd}'\n");
		//debug_log("BSys: Access: '{$uri}'" . print_r($_SERVER, 1) . "\n");

		// If a login is required then present the login page
		if($this->loginRequired){
			$this->login = new BLogin($this, "pageLogin", true);
			
			if(($uri == "user") && (($_GET["cmd"] == "forgottenPassword") || ($_GET["cmd"] == "forgottenPasswordEmail") || ($_GET["cmd"] == "forgottenPasswordChange"))){
			}
			else if(substr($uri, 0, 9) == "git/clone"){
				if(!$this->login->performLoginHttp())
					exit();
			}
			else {
				if(!$this->login->performLogin())
					exit();
			}

			if(isset($_GET['cmd']) && ($_GET['cmd'] == "logout")){
				$this->login->logout();
				header ("Location: index.php");
			}
		}

		// Setup user information
		if($this->login->id){
			$this->user = $this->db->getObject("User", $this->login->id);
			$this->modules["user"]->updateAccessTime($this->user->id);
		}

		// Setup work directory
		$this->workDir = "/" . $uri;
		if(substr($this->workDir, 0, 6) == "/files")
			$this->workDir = "/" . substr($this->workDir, 7);
		if(!is_dir(filePathAbs($this->workDir))){
			$this->workDir = pathinfo($this->workDir,  PATHINFO_DIRNAME);
		}
		$this->workDir = filePathNormalise($this->workDir);

//		error_log("Uri: {$uri} WorkDir: {$this->workDir}");

		// Team (logged in) only access
		if(in_array("team", preg_split("/[\/?]+/", $uri))){
			$this->login = new BLogin($this, "pageLogin", true);

			if(!$this->login->performLogin()){
				exit();
			}

			if(!$this->login->checkLogin()){
				$this->pageDisplay($this->page->title, "<div class='error'><h3>Only members can access this area of the website.</h3></div>");
				exit();
			}
		}

		// Members only access
		if(in_array("members", preg_split("/[\/?]+/", $uri)) || (substr($uri, 6) == "/rides")){
			$this->login = new BLogin($this, "pageLogin", true);

			if(!$this->login->performLogin()){
				exit();
			}

			if(!$this->login->checkLogin() || !$this->login->membershipId){
				$this->pageDisplay($this->page->title, "<div class='error'><h3>Only members can access this area of the website.</h3></div>");
				exit();
			}
		}

		// Process the URI
		//error_log("Process: '{$uri}'");
		if($this->moduleFind($uri)){
			$this->processModule($uri);
		}
		else if(($uri == "") || ($uri == "{$this->basePath}/") || ($uri == "files/index.html") || ($uri == "files/index-new.html") || ($uri == ($this->basePath . "/index.php"))){
			if($this->browser == "Safari"){
				$this->status = "The Safari browser is not well supported due to its non standard issues. Please install/use a Firefox or Google Chrome WEB browser";
				debug_log("Safari web browser in use: " . $_SERVER["REMOTE_ADDR"] . ", " . $_SERVER["HTTP_USER_AGENT"]);
			}

			$this->pageDisplayHome();
		}
		else if($uri == "admin"){
			$this->pageDisplayAdmin();
		}
		else if($this->debug && ($uri == "phpinfo")){
			phpinfo();
		}
		else if(0 && preg_match("/lib\/.*\.php/", $uri)){
			// For file browser etc. Done in .htaccess at the moment ...
			include($uri);
		}
		else if(preg_match("/.*\.php/", $uri)){
			$this->pageDisplayError(new BError(404, "Uri: not accessible"));
		}
		else if(!strncmp($uri, "icons/", 6)){
			// Return any files in icons
			fileOutput($uri);
		}
		else if(!strncmp($uri, "themes/", 7)){
			// Return any files in themes
			fileOutput($uri);
		}
		else if(preg_match("/lib\/.*\.gif/", $uri) || preg_match("/lib\/.*\.png/", $uri)){
			// Return any icons in lib
			fileOutput($uri);
		}
		else{
			// Only allow access if within /files
			$path = filePathAbs($uri);
			//print "BSys: process file: '{$uri}' path: {$path}<br>";
			
			if(preg_match("/.*\.html/", $uri) || preg_match("/.*\.htm/", $uri)){
				if(!$this->login->checkLogin("admin") && file_exists(dirname($path) . "/index-noaccess.html"))
					$this->processContent(dirname($uri) . "/index-noaccess.html");
				else
					$this->processContent($uri);
			}
			else if(is_dir($path)){
				// Redirect if directory path does not have trailing /
				if(substr($uri, -1) != "/"){
					$url = $this->baseUrl . "/" . $uri . "/";
					//print("Would redirect to {$url}/</br>");
					header("Location: {$url}", 1, 301);
					die();
				}
				
				if(!$this->login->checkLogin("admin") && file_exists($path . "/index-noaccess.html")){
					$this->processContent($uri . "/index-noaccess.html");
				}
				else if(file_exists($path . "/index.html")){
					$this->processContent($uri . "/index.html");
				}
				else {
					$this->processDir($uri);
				}
			}
			else if(file_exists($path)){
				// Should limit file types to just a few ...
				$ext = pathinfo($path, PATHINFO_EXTENSION);
				
				//debug_log("Check if file is accessible: '{$uri}' '{$path}'");
				
				// Check if accessible
				if(isset($this->noaccessFiles)){
					$list = explode(",", $this->noaccessFiles);
					foreach($list as $pattern){
						//print("Test '{$pattern}' against '{$uri}'<br>");
						if(fnmatch($pattern, "/{$uri}", FNM_PERIOD)){
							$this->pageDisplayError(new BError(401, "<div class='error'><h3>These files are not accessible.</h3></div>"));
							exit();
						}
					}
				}
				if(isset($this->loginOnlyFiles)){
					$list = explode(",", $this->loginOnlyFiles);
					foreach($list as $pattern){
						//print("Test '{$pattern}' against '{$path}'<br>");
						if(fnmatch($pattern, $path, FNM_PERIOD)){
							if(!$this->login->checkLogin()){
								$this->pageDisplayError(new BError(401, "<div class='error'><h3>Only users can access these files. Please login to access.</h3></div>"));
								exit();
							}
						}
					}
				}
				if(isset($this->membersOnlyFiles)){
					$list = explode(",", $this->membersOnlyFiles);
					foreach($list as $pattern){
						//print("Test '{$pattern}' against '{$path}'<br>");
						if(fnmatch($pattern, $path, FNM_PERIOD)){
							if(!$this->login->checkLogin() || !$this->login->membershipId){
								$this->pageDisplayError(new BError(401, "<div class='error'><h3>Only members can access these files. Please login to access.</h3></div>"));
								exit();
							}
						}
					}
				}
				
				if($ext == "sch"){
					$mime = "application/octet-stream";
				}
				else if($ext == "gpx"){
					$mime = "application/gpx+xml";
				}
				else {
					$fi = new finfo(FILEINFO_MIME);
					$mime = $fi->file($path);
					if($mime == "")
						$mime = "text/plain; charset=utf-8";
					if($mime == "text/plain; charset=us-ascii")
						$mime = "text/plain; charset=utf-8";
					
				}
				
//				print "File: $path Ext: $ext Content: $mime";
				header("Content-type: $mime");
				$expires = 60*60*24*14;
				header("Cache-Control: max-age=$expires");
				ob_end_flush();
				readfile($path);
			}
			else {
				$errorUrl = htmlToText($uri);
				$this->pageDisplayError(new BError(404, "Uri: {$errorUrl} not found"));
			}
		}
	}

	function processModule($uri) {
		$module = $this->moduleFind($uri);

		$content = $module->getPage($uri);

		if($content != null){
			if($module->getPageBlock($uri))
				$this->page->leftObjectInsert($module->getPageBlock($uri));
			$this->pageDisplay($module->title, $content);
		}
	}

	function processContent($uri) {
		$module = $this->modules["content"];

		$page = $module->getPage($uri);

		if($page != null){
			if($module->getPageBlock($uri))
				$this->page->leftObjectInsert($module->getPageBlock($uri));
			
			if(!$this->page->aditemsShow)
				$this->page->aditemsShow = $this->aditemsShowAll;
			$this->pageDisplay($this->page->title, $page);
		}
	}

	function processDir($uri){
		if(1){
			$module = $this->modules["contentDir"];

			$page = $module->getPage($uri);

			if($page != null){
				if($module->getPageBlock($uri))
					$this->page->leftObjectInsert($module->getPageBlock($uri));
				$this->pageDisplay($this->page->title, $page);
			}
		}
		else {
			$dirl = new BDirectoryLister;

			$this->pageDisplay($uri, $dirl->createList($uri));
		}
	}

	function pageDisplayHome(){
		$this->page->isHomepage = 1;
		$this->page->aditemsShow = $this->aditemsShowHome;

		if(($this->style == "tandemClubNew") || ($this->style == "tandemClubNew1")){
			if($this->isMobile && file_exists(filePathAbs("/") . "/index-mobile-new.html")){
				$this->processContent("/index-mobile-new.html");
			}
			else {
				$this->processContent("/index-new.html");
			}
		}
		else {
			if($this->isMobile && file_exists(filePathAbs("/") . "/index-mobile.html")){
				$this->processContent("/index-mobile.html");
			}
			else {
				$this->processContent("/index.html");
			}
		}
	}

	function pageDisplayAdmin(){
		$col = 0;

		$h = new BHtmlNode("div");
		$h->appendNode("h1", "", "Admin Top");


		$t = &$h->appendNode("table");
		foreach($this->modules as $module){
			if($col == 0)
				$r = &$t->appendNode("tr");

			$c = &$r->appendNode("td");
			if($module->checkAccess())
				$c->appendText(addItem("{$this->baseUrl}/icons/topfiles/pages.gif", $module->title, $module->info, "{$this->baseUrl}/{$module->name}", Null));
			if($col++ == 1)
				$col = 0;
		}
		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td");
		$c->appendText(addItem("{$this->baseUrl}/icons/topfiles/pages.gif", "Help", "Help information on using this website", "{$this->baseUrl}/help/index.html", Null));

		$this->pageDisplay("Administration Top", $h->render());
	}

	function createMenuBar(){
		// Create Menu bar
		$m = new BMenu("MenuTop");

		$m->createFromFile($this->baseDir . "/files/menu.conf");

		if(0){
			foreach($this->modules as $module){
				if($module->checkAccess()){
					if($mt = $module->getMenu()){
						$m->add($mt);
					}
				}
			}
		}
		
//		if($this->login->checkLogin("admin,adminMembership,adminPayments")){
		if($this->login->checkLogin("any")){
			$ms = new BMenu("");
			$ms->add(new BMenu("Admin", "/admin", "", $ms));
			foreach($this->modules as $module){
				if($module->checkAccess()){
					$ms->add(new BMenu($module->title, "/{$module->name}"));
				}
			}
			$ms->add(new BMenu("Help", "/help/index.html", "target='_blank'"));
			$m->add(new BMenu("Admin", "/admin", "", $ms));
		}

		$this->page->setMenuBar($m);
	}

	function modulesLoad(){
		foreach($this->moduleNames as $key => $name){
			//print "LoadModule: {$this->baseDir}/modules/{$name}/{$name}.php<br>\n";
			if(file_exists("{$this->baseDir}/modules/{$name}/{$name}.php"))
				require_once "{$name}/{$name}.php";
			else
				require_once "{$name}/{$name}.module";
			$this->modules[$key] = new $name($this);
			$this->modules[$key]->setName($key);
		}
	}

	function moduleFind($uri){
		foreach($this->modules as $key => $value){
			if(($len = strpos($uri, '/')) === false)
				$len = strlen($uri);

			if(strlen($key) > $len)
				$len = strlen($key);

			if(!strncmp($key, $uri, $len))
				return $value;
		}
		return null;
	}

	function moduleFindType($type){
		foreach($this->modules as $key => $value){
			if(get_class($value) == $type)
				return $value;
		}
		return null;
	}

	function module($name){
		if(isset($this->modules[$name]))
			return $this->modules[$name];
		else
			return null;
	}

	function moduleFind1($uri){
		$first = $uri;
		if($pos = strpos($first, "/"))
			$first = substr($first, 0, $pos);
			
		foreach($this->modules as $key => $value){
			if($first == $key)
				return $value;
		}
		return null;
	}
	
	
	function logAdd($type, $module, $description){
		$this->modules["management"]->logAdd($type, $module, $description);
	}

	function getUserInfo($id){
		return $this->modules["user"]->getUserInfo($id);
	}
	
	function checkAccessAdmin($roles){
		return $this->login->checkLogin($roles);
	}
	
	function checkAccessView($accessMode, $group){
		if($accessMode == AccessMode::LoggedIn){
			if($this->login->checkLogin() == 0)
				return 0;
			
			if(($group != "") && (groupType($group) == "private")){
				if(($this->login->group != $group) && (strpos($this->login->groups, $group) !== false))
					return 0;
			}
			return 1;
		}
		elseif($accessMode == AccessMode::Member){
			if(($this->login->checkLogin() == 0) || ($this->sys->login->membershipId == 0))
				return 0;
			
			if(($group != "") && (groupType($group) == "private")){
				if(($this->login->group != $group) && (strpos($this->login->groups, $group) !== false))
					return 0;
			}
			return 1;
		}

		if(function_exists("groupType") && ($group != "") && (groupType($group) == "private")){
			if(($this->login->group != $group) && (strpos($this->login->groups, $group) === false))
				return 0;
		}
		return 1;
	}


	function postValidateSave($id = 0){
		$si = new PostSecureInfo();
		$si->password = makeRandomPassword();
		$si->objectId = $id;
		$_SESSION["postValidate"] = serialize($si);
		return $si->password;
	}
	
	function postValidateCheck($id = 0){
		if(!isset($_SESSION["postValidate"])){
			//debug_log("BSys::postValidateCheck: No session");
			return 0;
		}
			
		if(!isset($_POST["postValidate"])){
			//debug_log("BSys::postValidateCheck: No post");
			return 0;
		}

		$si = unserialize($_SESSION["postValidate"]);
		$password = $_POST["postValidate"];
		
		//debug_log("BSys::postValidateCheck: '{$si->password}' == '{$password}' '{$si->objectId}' == '{$id}'");

		// Validate access
		if($si->password != $password){
			//debug_log("BSys::postValidateCheck: Invalid password");
			return 0;
		}
		if($si->objectId != $id){
			//debug_log("BSys::postValidateCheck: Invalid objectId");
			return 0;
		}
		return 1;
	}

	function jsonReturnError($httpError, $err){
		http_response_code($httpError);
		$ret = array("error" => $err);

		header("Content-type: application/json");
		print(json_encode($ret));
		exit(1);
	}

	function displayError($httpError, BError $err){
		http_response_code($httpError);
		print("Error: {$err->getString()}\n");
		exit(1);
	}

	//***************************************************************************************************
	// Page managment
	//***************************************************************************************************
	function pageDisplay($title, $content){
		$this->page->setTitle($title);
		$this->page->setCentre($content);
		$this->page->render();
	}

	function pageDisplayError(BError $err){
		$html = "<div class='error'>\n";
		$html .= "<h1>Error</h1>\n";
		$html .= "{$err->getString()}\n";
		$html .= "</div>\n";

		if($err->oerrNo < 100)
			$err->oerrNo = 500;
		http_response_code($err->oerrNo);
		$this->page->setStatus("<span class='error'>An error occured</span>");
		$this->pageDisplay("{$this->siteName} - Error", $html);
		exit(1);
	}

	function pageDisplayPopup($title, $content){
		$this->page->popup = 1;
		$this->page->setTitle($title);
		$this->page->setCentre($content);
		$this->page->render();
	}

	// Internal variables
	var	$version = "1.2.8";			// The system version
	var	$modules;				// The list of modules
	var	$login;					// Login information
	var	$page;					// The output page
	var	$user;					// User information
	var	$groups = array("");			// The list of all groups
	var	$htmlFuncs = array();			// List of functions available for HTML pages
	var	$workDir = "/";				// Initial working directory
	var	$device;				// The users device type
	var	$isMobile = 0;				// Is a mobile phone

	var	$uri;					// The requests URL path
	var	$test = 0;
};

function fileOutput($uri){
	$fi = new finfo(FILEINFO_MIME);
	$mime = $fi->file($uri);
	if($mime == "")
		$mime = "text/plain";

	header("Content-type: $mime");
	$expires = 60*60*24*14;
	header("Cache-Control: private, max-age=$expires");
	readfile($uri);
	exit(0);
}

?>
