<?php
/*******************************************************************************
 *	bds.module		Bdsdata Module
 *				T.Barnaby,	BEAM Ltd,	2017-01-09
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "phplot/phplot.php";

function bdsDateIntervalHalf($di){
	if($di->y)
		$di->y /= 2;
	if($di->m)
		$di->m /= 2;
	if($di->d)
		$di->d /= 2;
	if($di->h)
		$di->h /= 2;
	if($di->i)
		$di->i /= 2;
	if($di->s)
		$di->s /= 2;

	return $di;
}

class BdsValues {
	function __construct(){
		$this->startDate = "";
		$this->startTime = "";
		$this->duration = "";
		$this->network = "";
		$this->station_wild = "";
		$this->station = array();
		$this->channel_wild = "";
		$this->channel = array();
		$this->source = "";
		$this->dataFormat = "";
		$this->metaDataFormat = "";
	}
	
	function set($values){
		foreach($values as $n => $v){
			$n = filterString($n);
			if(is_array($v)){
				$a = array();
				foreach($v as $n1 => $v1){
					$a[filterString($n1)] = filterString($v1);
				}
				$this->{$n} = $a;
			}
			else {
				$this->{$n} = filterString($v);
			}
		}
	}
}

// Use bdsDataAccess program for some functionality
class BdsDataAccessExec {
	function __construct($host, $user, $password){
		$this->host = $host;
		$this->user = $user;
		$this->password = $password;
	}
	
	function selectionString($sel){
		$str = "";
		
		if($sel->startTime != "")
			$str .= " -startTime " . $sel->startTime->format("Y-m-d\TH:i:s");
		if($sel->endTime != "")
			$str .= " -endTime " . $sel->endTime->format("Y-m-d\TH:i:s");
		
		foreach($sel->channels as $c){
			$str .= " -select " . $c->network . ":" . $c->station . ":" . $c->channel . ":" .$c->source;
		}
		
		return $str;
	}
	
	function dataGetFormatted($sel, $dataFormat, $fileName){
		$l = array();
		
		$cmd = "-command dataGetFormatted";
		$cmd .= " -format {$dataFormat}";
		$cmd .= " -force -o {$fileName}";
		$cmd .= $this->selectionString($sel);
		
		return $this->execute($cmd, $stdout, $stderr);
	}

	function dataGetResponses($sel, $dataFormat, $fileName){
		$l = array();
		
		$cmd = "-command metadataGetFormatted";
		$cmd .= " -format {$dataFormat}";
		$cmd .= " -force -o {$fileName}";
		$cmd .= $this->selectionString($sel);
		
		return $this->execute($cmd, $stdout, $stderr);
	}

	function dataPlot($sel, $fileName){
		$l = array();
		
		$cmd = "-command dataPlot";
		$cmd .= $this->selectionString($sel);
		$cmd .= " --o " . $fileName;
		
		return $this->execute($cmd, $stdout, $stderr);
	}
	
	function execute($cmd, &$stdout, &$stderr){
		$err = new BBError();
		$files = array(
			1 => array("pipe", "w"),  // stdout
			2 => array("pipe", "w"),  // stderr
			);

		$cmd = "/usr/bds/bin/bdsDataAccess -host {$this->host} -user {$this->user}:{$this->password} {$cmd}";
		//$cmd = "/src/blacknest/bds/bds/bdsDataAccess/bdsDataAccess -host {$this->host} -user {$this->user}:{$this->password} {$cmd}";
		//debug_log("ExecuteCmd: {$cmd}");

		$proc = proc_open($cmd, $files, $pipes);
		$stdout = stream_get_contents($pipes[1]);
		$stderr = stream_get_contents($pipes[2]);
		fclose($pipes[1]);
		fclose($pipes[2]);
		$err->num = proc_close($proc);
		
		if($err->num)
			$err->str = $stderr;

//		print "Execute result: '{$err->num}' '{$err->str}'<br>";
		return $err;
	}
};


function editAdd($row, $name, $type, $value = "", $title = "", $args = Null){
	if($title != "")
		$row->appendNode("th", "align = 'left'", $title);
	$c = $row->appendNode("td");
	switch($type){
	case 'select':
		$sel = &$c->appendNode("select", "name='$name'");
		foreach($args as $a){
			if($a == $value)
				$sel->appendNode("option", "selected='selected'", $a);
			else
				$sel->appendNode("option", "", $a);
		}
		break;
	case 'date':
		$c->appendNode("script", "", "editorDate('', '$name', '$value', '');");
		break;
	default:
		$c->appendNode("input", "type='text' name='$name' value='$value'");
		break;
	}
}

class Bds extends BModule {
	// Basic Interface routines
	function  __construct(&$sys){
		parent::__construct($sys);
		$this->name = "bds";
		$this->title = "Bds";
		$this->info = "Bds Module";
		$this->bdsAccess = new BdsDataAccess();
		$this->test = $this->sys->test;
		
		$err = $this->bdsAccess->connectService("//{$this->sys->bdsHost}/bdsDataAccess");
//		print "Err: "; print_r($err);

		$err = $this->bdsAccess->connect($this->sys->bdsUser, $this->sys->bdsPassword);
//		print "Err: "; print_r($err);

		$this->bdsAccessExec = new BdsDataAccessExec($this->sys->bdsHost, $this->sys->bdsUser, $this->sys->bdsPassword);

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminBds"));
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("adminBds");
	}

	// Standard interface
	function getMenu(){
		return new BMenu("Bds", "/{$this->name}", "", array(
			"BDS" => "/bds",
			));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		case "viewGraph":		return $this->viewGraph($uri);		break;
		case "viewInfo":		return $this->viewInfo($uri);		break;
		case "fetchData":		return $this->fetchData($uri);		break;
		case "getData":			return $this->getData($uri);		break;
		case "Search":			return $this->dataSelect($uri);		break;
		case "dataSelect":		return $this->dataSelect($uri);		break;
		case "viewNote":		return $this->viewNote($uri);		break;
		case "getSelection":		return $this->getSelection($uri);	break;
		case "":			return $this->dataSelect($uri);		break;
		default:			return "Error: unknown command: {$this->cmd}";	break;
		}
	}

	function getPageBlock($uri){
		$o = new BMenu("Bds", "", "", array(
			new BMenu("Data Select", "", "href='?cmd=dataSelect'"),
			new BMenu("", "", ""),
			new BMenu("View Graph", "",	"href='?cmd=viewGraph'"),
			new BMenu("Fetch Data", "",	"href='?cmd=fetchData'"),
			new BMenu("Help",	"",	"href='/help/bds.html'"),
			new BMenu("", "", ""),
			));

		return $o;
	}
	
	function drawAvailability(){
		$h = new BHtmlNode("div", "id='bdsAvail' style='width: 100%;  height: 28px; border: 1px solid;'");
		$h->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: #0a0; width: 10px; height: 28px; text-align: center;'");
		$h->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: blue; width: 10px; height: 28px; text-align: center;'");
		$h->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: red; width: 10px; height: 28px; text-align: center;'");

		return $h->render();
	}

	function dataAvailTitles($dataAvailChans){
		$titles = array();
		
		if(count($dataAvailChans)){
			$segments = $dataAvailChans[0]->segments;
			$startTime = $segments[0]->startTime;
			$endTime = end($segments)->endTime;
			
			$period = date_diff($startTime, $endTime);
			
//			print "Period:"; print_r($period); print "<br>";
			if($period->y > 1){
				foreach($segments as $dac){
					$p = bdsDateIntervalHalf(date_diff($dac->startTime, $dac->endTime));
					$mid = $dac->startTime->add($p);
					$titles[] = $mid->format("Y");
				}
			}
			else if($period->y || ($period->m > 1)){
				foreach($segments as $dac){
					$p = bdsDateIntervalHalf(date_diff($dac->startTime, $dac->endTime));
					$mid = $dac->startTime->add($p);
					$titles[] = $mid->format("M");
				}
			}
			else if($period->m || ($period->d > 1)){
				foreach($segments as $dac){
					$p = bdsDateIntervalHalf(date_diff($dac->startTime, $dac->endTime));
					$mid = $dac->startTime->add($p);
					$titles[] = $mid->format("d");
				}
			}
			else {
				foreach($segments as $dac){
					$p = bdsDateIntervalHalf(date_diff($dac->startTime, $dac->endTime));
					$mid = $dac->startTime->add($p);
					$titles[] = $mid->format("H:i:s");
				}
			}
		}
		
		return $titles;
	}

	function getSelection($uri){
		$selIn = new BdsSelection;
		
		$obj = json_decode($_GET["json"]);
		foreach($obj->networks as $n){
			foreach($obj->stations as $s){
				$sc = new BdsSelectionChannel;
				$sc->network = $n;
				$sc->station = $s;
				$selIn->channels[] = $sc;
			}
		}
		
		$err = $this->bdsAccess->getSelectionInfo(0, $selIn, $selInfo);
		if(0){
			$selInfo->arrays = array("A1", "A2");
			$selInfo->arraysAndStations = array("AS1", "AS2");
			$selInfo->stations = array("STA1", "STA2");
			$selInfo->stations = array("STA1", "STA2");
			$selInfo->channels = array("CH1", "CH2");
		}
		print json_encode($selInfo);
		return Null;
	}
	
	function appendInput($parent, $name, $title, $type, $args, $tip){
		if($title == "")
			$title = ucfirst($name);
		
		$wild = "";
		$wildname = $name . "_wild";
		if(isset($this->values->{$wildname}))
			$wild = $this->values->{$wildname};
		$value = $this->values->{$name};

		$jargs = json_encode($args);
		$jvalue = json_encode($value);

		$parent->appendNode("th", "align='left'", $title);
		
		//debug_log("appendInput: {$name} {$value}" . print_r($value, 1));

		switch($type){
		case "select":
			$sel = $parent->appendNode("td", "valign='top'")->appendNode("select", "name='{$name}' title='{$tip}'");
			foreach($args as $item){
				if($item == $value)
					$sel->appendNode("option", "value='{$item}' selected='selected'", $item);
				else
					$sel->appendNode("option", "value='{$item}'", $item);
			}
			break;

		case "selectWild":
			$parent->appendNode("td", "valign='top'")->appendNode("script", "", "new editorSelectWild(null, '{$name}', {$jargs}, '{$wild}', {$jvalue}, 0, '{$tip}');");
			break;

		case "selectWild8":
			$parent->appendNode("td", "valign='top'")->appendNode("script", "", "new editorSelectWild(null, '{$name}', {$jargs}, '{$wild}', {$jvalue}, 8, '{$tip}');");
			break;

		default:
			$attrs = "type='{$type}' name='{$name}' value={$value} title='{$tip}'";
			if(!is_null($args)) {
				foreach($args as $item) {
					$attrs .= " {$item}";
				}
			}
			$parent->appendNode("td")->appendNode("input", $attrs);
			break;
		}
		$parent->appendText("\n");
	}
	
	function dataSelect($uri){
		$page = "";
		
		//debug_log("Bds::dataSelect: Get: " . print_r($_GET, 1));
		
		if(isset($_GET["select_cmd"]) && ($_GET["select_cmd"] == "Search"))
			$view = "availability";
		else if(isset($_GET["viewMetaData"]))
			$view = "viewMetaData";
		else if(isset($_GET["viewDataNotes"]))
			$view = "viewDataNotes";
		else if(isset($_GET["viewData"]))
			$view = "viewData";
		else if(isset($_GET["downloadData"]))
			$view = "downloadData";
		else if(isset($_GET["downloadMetadata"]))
			$view = "downloadMetadata";
		else
			$view = "dataSelect";

		// Setup values
		$this->values = new BdsValues();

		if($this->test){
			if(1){
				$tstart = new BDate("2008-04-01");
				$tstart->setTime(0, 0, 0);
				$tend = clone $tstart;
				$tend->addSeconds(60);
			}
			else {
				$tstart = new BDate("2008-01-01");
				$tstart->setTime(0, 0, 0);
				$tend = clone $tstart;
				$tend->addSeconds(10);
			}
			$this->values->station = array("TSB01");
		}
		else {
			$tstart = new BDate("now");
			$tstart->setTime(0, 0, 0);
			$tend = clone $tstart;
			$tend->addSeconds(600);
		}

		$this->values->startDate = $tstart->getDateTime("%Y-%m-%d");
		$this->values->startTime = $tstart->getDateTime("%H:%M:%S");
		$this->values->duration = "00:01:00";
		$this->values->dataFormat = "SEED";
		$this->values->metaDataFormat = "RESPONSE-SEED";
		
		$this->values->set($_GET);
		//debug_log("Values: " . print_r($this->values, 1));

		$selIn = new BdsSelection;

		if(0 && count($this->values->station)){
			foreach($this->values->station as $station){
				$selChannel = new BdsSelectionChannel;
				$selChannel->network = $this->values->network;
				$selChannel->station = $station;
				$selChannel->channel = "";
				$selChannel->source = "";
				$selIn->channels[] = $selChannel;
			}
		}
		else {
			$selChannel = new BdsSelectionChannel;
			$selChannel->network = $this->values->network;
			$selChannel->station = "";
			$selChannel->channel = "";
			$selChannel->source = "";
			$selIn->channels[] = $selChannel;
		}
		
		if(0){
			debug_log("SelectionInfo:");
			foreach($selIn->channels as $c){
				debug_log("SC: {$c->network}:{$c->station}:{$c->channel}:{$c->source}");
			}
		}
		
		$err = $this->bdsAccess->getSelectionInfo(0, $selIn, $selInfo);
		if($err->num){
			return $this->sys->pageDisplayError(new BError($err->num, "Cannot access BdsServer: {$err->str}"));
		}
		
		$this->bdsAccess->dataFormatGetList($formats);

		$dataFormats = array();
		$metaDataFormats = array();
		foreach($formats as $f){
			if($f->dataWrite)
				$dataFormats[] = $f->names[0];
			if($f->metadataWrite)
				$metaDataFormats[] = $f->names[0];
		}

		$arraysAndStations = array();
		
		foreach($selInfo->arraysAndStations as $s){
			if(in_array($s, $selInfo->arrays)){
				$arraysAndStations[] = array($s, $s, "ColourHighlight");
			}
			else {
				$arraysAndStations[] = array($s, $s, "");
			}
		}

		$page .= "This page gives access to the BDS system. It allows a user to select a ";
		$page .= "set of data over a time period and see what is available and download the ";
		$page .= "data or its metadata (calibration factors, responses etc.).";
		$page .= "<p>To use select the date/time range and the set of stations or arrays required.";
		$page .= "You can refine the selection further using the network, channel and source parameters.";
		$page .= "Click on the <b>Search</b> button to perform the search.";
		$page .= "</p>";
		

		$h = new BHtmlNode("form");
		$t = $h->appendNode("table", "class='BEdit'");
		$r = $t->appendNode("tr");
		$this->appendInput($r, "startDate", "", "date", Null, "Set the start date for data.");
		$this->appendInput($r, "startTime", "", "time", ["step='1'"], "Set the start time for data.");
		$this->appendInput($r, "duration", "", "time", ["step='1'"], "Set the duration of the data.");

		$r = $t->appendNode("tr");
		$this->appendInput($r, "network", "", "select", array_merge(array(""), $selInfo->networks), "Set the Seismic Network. A blank entry means all networks.");
		$this->appendInput($r, "station", "", "selectWild8", $arraysAndStations, "Set the Seismic Station or Array. A blank entry means all stations/arrays.");
		$this->appendInput($r, "channel", "", "selectWild8", array_merge(array(""), $selInfo->channels), "Set the Seismic Channel. A blank entry means all channels.");
		$this->appendInput($r, "source", "", "select", array_merge(array(""), $selInfo->sources), "Set the Source for the data if there are multiple sources. A blank entry means all sources.");
		
		$t = $h->appendNode("table", "class='BEdit'");
		$r = $t->appendNode("tr");
		$this->appendInput($r, "dataFormat", "", "select", $dataFormats, "Set the sensor data format when exporting.");
		$this->appendInput($r, "metaDataFormat", "", "select", $metaDataFormats, "Set the metadata format when exporting.");

		$h->appendNode("input", "type='submit' class='ButtonMain' name='select_cmd' value='Search' title='Search data store and show availability of the selected data'");
		$h->appendNode("input", "type='submit' class='ButtonMain' name='viewMetaData' value='View metadata' title='Show summary of metadata for the available selected data channels.'");
		$h->appendNode("input", "type='submit' class='ButtonMain' name='viewDataNotes' value='View data notes' title='Show notes and warnings about the available selected data.'");
		$h->appendNode("input", "type='submit' class='ButtonMain' name='viewData' value='View data' title='Crude seismogram of the first 256 points of each channel of data.'");
		$h->appendNode("input", "type='submit' class='ButtonMain' name='downloadData' value='Download data' title='Download data in the format given by DataFormat.'");
		$h->appendNode("input", "type='submit' class='ButtonMain' name='downloadMetadata' value='Download metadata' title='Download full metadata in the format given by MetaDataFormat'");
		
		$page .= $h->render();
		
		if(count($this->values->station)){
			// Setup BDS Selection
			$sel = new BdsSelection();
			$sel->startTime = new DateTime($this->values->startDate . "T" . $this->values->startTime);
			$sel->endTime = new DateTime($this->values->startDate . "T" . $this->values->startTime);
			list($hours, $minutes, $seconds) = sscanf($this->values->duration, "%d:%d:%d");
			$duration = new DateInterval(sprintf('PT%dH%dM%dS', $hours, $minutes, $seconds));
			$sel->endTime->add($duration);
			
			foreach($this->values->station as $station){
				if(count($this->values->channel)){
					foreach($this->values->channel as $channel){
						$selChannel = new BdsSelectionChannel;
						$selChannel->network = $this->values->network;
						$selChannel->station = $station;
						$selChannel->channel = $channel;
						$selChannel->source = $this->values->source;
						$sel->channels[] = $selChannel;
					}
				}
				else {
					$selChannel = new BdsSelectionChannel;
					$selChannel->network = $this->values->network;
					$selChannel->station = $station;
					$selChannel->channel = "";
					$selChannel->source = $this->values->source;
					$sel->channels[] = $selChannel;
				}
			}

			$dataFormat = $this->values->dataFormat;
			$dataFormatExt = ".txt";
			foreach($formats as $f){
				if($f->names[0] == $dataFormat)
					$dataFormatExt = $f->extension;
			}
			$metaDataFormat = $this->values->metaDataFormat;

			$_SESSION["sel"] = serialize($sel);
			$_SESSION["view"] = $view;
			$_SESSION["dataFormat"] = $dataFormat;
			$_SESSION["dataFormatExt"] = $dataFormatExt;

			$page .= "<p></p>";
			
			//debug_log("Select: " . print_r($sel, 1));
			
			if($view == "dataSelect"){
			}
			else if($view == "availability"){
				$page .= $this->dataAvailability($sel);
			}
			else if($view == "viewMetaData"){
				$page .= $this->dataListMetaData($sel);
			}
			else if($view == "viewDataNotes"){
				$page .= $this->dataListNotes($sel);
			}
			else if($view == "viewData"){
				$page .= $this->viewData($sel);
			}
			else if($view == "downloadData"){
				$res = $this->getData($sel, $dataFormat);
				if($res == null)
					return $res;
				else
					$page .= $res;
			}
			else if($view == "downloadMetadata"){
				$res = $this->getResponses($sel, $metaDataFormat);
				if($res == null)
					return $res;
				else
					$page .= $res;
			}
		}

		return $page;
	}
	
	function dataListChannels($sel){
		// Selection List
		$page = "<h2>Data Selection List</h2>";

		$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align = 'left'", "StartTime");
		$r->appendNode("th", "align = 'left'", "EndTime");
		$r->appendNode("th", "align = 'left'", "Network");
		$r->appendNode("th", "align = 'left'", "Station");
		$r->appendNode("th", "align = 'left'", "Channel");
		$r->appendNode("th", "align = 'left'", "Source");

		//debug_log("Select: " . print_r($sel, 1));

		if(count($sel->channels)){
			$err = $this->bdsAccess->dataSearch($sel, $dataInfo);
			if($err->num){
				$page .= "<b>Error: unable to access BDS system: " . $err->str . "</b><br>";
			}
			else {
				//debug_log("DataInfo: " . print_r($dataInfo, 1));
				$n = count($dataInfo->channels);

				foreach($dataInfo->channels as $c){
					$c = $c[0];
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "", $c->startTime->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $c->endTime->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $c->network);
					$r->appendNode("td", "", $c->station);
					$r->appendNode("td", "", $c->channel);
					$r->appendNode("td", "", $c->source);
				}
			}
		}
		$page .= $t->render();

		return $page;
	}
	
	function dataListMetaData($sel){
		// Selection List
		$page = "";

		$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		//$r->appendNode("th", "align = 'left'", "StartTime");
		//$r->appendNode("th", "align = 'left'", "EndTime");
		$r->appendNode("th", "align = 'left'", "Network");
		$r->appendNode("th", "align = 'left'", "Station");
		$r->appendNode("th", "align = 'left'", "Channel");
		$r->appendNode("th", "align = 'left'", "Source");
		$r->appendNode("th", "align = 'left'", "Name");
		$r->appendNode("th", "align = 'left'", "CalibrationFactor");
		$r->appendNode("th", "align = 'left'", "CalibrationUnits");
		$r->appendNode("th", "align = 'left'", "CalibrationFreq");
		$r->appendNode("th", "align = 'left'", "SampleRate");
		$r->appendNode("th", "align = 'left'", "depth");
		$r->appendNode("th", "align = 'left'", "HorizontalAngle");
		$r->appendNode("th", "align = 'left'", "VerticalAngle");

		//debug_log("Select: " . print_r($sel, 1));

		if(count($sel->channels)){
			$err = $this->bdsAccess->calibrationGetList($sel, $calbrations);
			if($err->num){
				$page .= "<b>Error: unable to access BDS system: " . $err->str . "</b><br>";
			}
			else {
				foreach($calbrations as $c){
					$r = &$t->appendNode("tr");
					//$r->appendNode("td", "", $c->startTime->format('Y-m-d H:i:s.u'));
					//$r->appendNode("td", "", $c->endTime->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $c->network);
					$r->appendNode("td", "", $c->station);
					$r->appendNode("td", "", $c->channel);
					$r->appendNode("td", "", $c->source);
					$r->appendNode("td", "", $c->name);
					$r->appendNode("td", "", strval($c->calibrationFactor));
					$r->appendNode("td", "", $c->calibrationUnits);
					$r->appendNode("td", "", strval($c->calibrationFrequency));
					$r->appendNode("td", "", strval($c->samplingFrequency));
					$r->appendNode("td", "", strval($c->depth));
					$r->appendNode("td", "", strval($c->horizontalAngle));
					$r->appendNode("td", "", strval($c->verticalAngle));
				}
			}
		}
		$page .= $t->render();

		return $page;
	}
	
	function dataListNotes($sel){
		// Selection List
		$page = "";

		$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align = 'left'", "StartTime");
		$r->appendNode("th", "align = 'left'", "EndTime");
		$r->appendNode("th", "align = 'left'", "Network");
		$r->appendNode("th", "align = 'left'", "Station");
		$r->appendNode("th", "align = 'left'", "Channel");
		$r->appendNode("th", "align = 'left'", "Source");
		$r->appendNode("th", "align = 'left'", "Type");
		$r->appendNode("th", "align = 'left'", "User");
		$r->appendNode("th", "align = 'left'", "TimeAdded");
		$r->appendNode("th", "align = 'left'", "ErrorNumber");
		$r->appendNode("th", "align = 'left'", "Title");

		//debug_log("Select: " . print_r($sel, 1));

		if(count($sel->channels)){
			$err = $this->bdsAccess->noteGetList($sel, $notes);
			if($err->num){
				$page .= "<b>Error: unable to access BDS system: " . $err->str . "</b><br>";
			}
			else {
				foreach($notes as $i){
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "", $i->startTime->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $i->endTime->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $i->network);
					$r->appendNode("td", "", $i->station);
					$r->appendNode("td", "", $i->channel);
					$r->appendNode("td", "", $i->source);
					$r->appendNode("td", "", $i->type);
					$r->appendNode("td", "", $i->user);
					$r->appendNode("td", "", $i->timeAdded->format('Y-m-d H:i:s.u'));
					$r->appendNode("td", "", $i->errorNumber);
					$r->appendNode("td", "", "")->appendNode("a", "href='?cmd=viewNote&id={$i->id}'", $i->title);
				}
			}
		}
		$page .= $t->render();

		return $page;
	}
	
	function viewNote($url){
		$id = filterToInt($_GET["id"]);
		
		$page = "";

		if($id){
			$sel = new BdsSelection;
			$sel->id = $id;
			$err = $this->bdsAccess->noteGetList($sel, $notes);
			if($err->num){
				$page .= "<b>Error: unable to access BDS system: " . $err->str . "</b><br>";
			}
			else if(!count($notes)){
				$page .= "<b>Error: unable to access Note {$id}</b><br>";
			}
			else {
				$members = get_object_vars($notes[0]);

				$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
				foreach($members as $name => $value){
					if(is_object($value) && (get_class($value) == "DateTime"))
						$value = $value->format('Y-m-d H:i:s.u');
					$t->appendTableNameValue($name, strval($value));
				}
				$page .= $t->render();
			}
		}
		else {
			$page .= "<b>Error: no note ID given.</b><br>";
		}

		return $page;
	}

	function dataAvailability($sel){
		$page = "";
		
		$period = date_diff($sel->startTime, $sel->endTime);
		if($period->y > 1){
			if($period->y > 30)
				$num = 30;
			else
				$num = $period->y;

			if($period->m)
				$num += 1;
		}
		else if($period->y || ($period->m > 1)){
			if($period->y)
				$num = 12 + $period->m;
			else
				$num = $period->m;

			if($period->d)
				$num += 1;
		}
		else if($period->m || ($period->d > 1)){
			if($period->m)
				$num = 31 + $period->d;
			else
				$num = $period->d;
		}
		else {
			$num = 24;
		}

		$err = $this->bdsAccess->dataAvailability($sel, $num, $dataAvailChans);

		if($err->num){
			$page .= "<b>Error: unable to access BDS system: " . $err->str . "</b><br>";
		}
		else {
			$titles = $this->dataAvailTitles($dataAvailChans);

			$t = new BHtmlNode("table", "border=1px width=100%");
			$r = &$t->appendNode("tr");
			$r ->appendNode("th", "width=100px", "Channel");

			foreach($titles as $title){
				$r ->appendNode("td", "width=20px style='writing-mode: tb-rl;'", $title);
			}

			foreach($dataAvailChans as $dac){
				$r = &$t->appendNode("tr");
				$r ->appendNode("th", "", "{$dac->network}:{$dac->station}:{$dac->channel}:{$dac->source}");
				for($i = 0; $i < count($dac->segments); $i += 1){
					if($dac->segments[$i]->availType == 0)
						$r ->appendNode("td", "")->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: #C00; width: 100%; height: 28px;'");
					else if($dac->segments[$i]->availType == 1)
						$r ->appendNode("td", "")->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: #CC0; width: 100%; height: 28px;'");
					else if($dac->segments[$i]->availType == 2)
						$r ->appendNode("td", "")->appendNode("div", "id='bdsAvailItem' style='float: left; background-color: #0C0; width: 100%; height: 28px;'");
				}
			}

			$page .= "<div style='float: left; padding-right: 10px;'>Availability Key: Fully available:</div><div style='float: left; background-color: #0C0; width: 50px; height: 28px;'></div>";
			$page .= "<div style='float: left; padding-left: 10px; padding-right: 10px;'>Partially available:</div> <div style='float: left; background-color: #CC0; width: 50px; height: 28px;'></div>";
			$page .= "<div style='float: left; padding-left: 10px; padding-right: 10px;'>Not available:</div> <div style='float: left; background-color: #C00; width: 50px; height: 28px;'></div>";
			$page .= $t->render();
		}
		return $page;
	}
	
	function getData($sel, $dataFormat){
		$page = "";

		//debug_log("Select: " . print_r($sel, 1));

		$err = $this->bdsAccess->dataFormatGetList($dataFormats);
		$dataFormatExt = "txt";
		foreach($dataFormats as $f){
			if(in_array($dataFormat, $f->names))
				$dataFormatExt = $f->extension;
		}

		$filename = $this->dataFilename($sel, $dataFormatExt);
		
		$filePath = $this->sys->baseDir . "/files/tmp/" . $filename;
		$fileUrl = $this->sys->baseUrl . "/files/tmp/" . $filename;

		$err = $this->bdsAccessExec->dataGetFormatted($sel, $dataFormat, $filePath);
		if($err->num){
			return "<b>Error fetching data: {$err->str}</b>";
		}
		else {
			header("Content-type: {$dataFormatExt}");
			//header("Content-type: application/octet-stream");
			header("Content-Disposition: attachment; filename={$filename}");
			readfile($filePath);
			return null;
		}
	}
	
	function getResponses($sel, $dataFormat){
		$page = "";

		//debug_log("Select: " . print_r($sel, 1));

		$err = $this->bdsAccess->dataFormatGetList($dataFormats);
		$dataFormatExt = "txt";
		foreach($dataFormats as $f){
			if(in_array($dataFormat, $f->names))
				$dataFormatExt = $f->extension;
		}

		$filename = $this->dataFilename($sel, $dataFormatExt);
		
		$filePath = $this->sys->baseDir . "/files/tmp/" . $filename;
		$fileUrl = $this->sys->baseUrl . "/files/tmp/" . $filename;

		$err = $this->bdsAccessExec->dataGetResponses($sel, $dataFormat, $filePath);
		if($err->num){
			return "<b>Error getting responses: {$err->str}</b>";
		}
		else {
			header("Content-type: {$dataFormatExt}");
			//header("Content-type: application/octet-stream");
			header("Content-Disposition: attachment; filename={$filename}");
			readfile($filePath);
			return null;
		}
	}
	
	function viewData_old($sel){
		$filename = basename(tempnam($this->sys->baseDir . "/files/tmp", "graph_")) . ".png";

		$err = $this->bdsAccessExec->dataPlot($sel, $this->sys->baseDir . "/files/tmp/{$filename}");
		if($err->num)
			$page = "<b>Error fetching data: {$err->str}</b>";
		else
			$page = "<img src='" . $this->sys->baseUrl . "/files/tmp/{$filename}" . "'>";

		return $page;
	}

	function viewData($sel){
		$ErrorEndOfData = 20;
		
		$h = new BHtmlNode("div", "class='BdsGraphs'");
		$filename = basename(tempnam($this->sys->baseDir . "/files/tmp", "graph_")) . ".png";

		$err = $this->bdsAccess->dataSearch($sel, $dataInfo);
		if($err->num){
			return "<b style='color:red;'>Error: BDS system: " . $err->str . "</b><br>";
		}
		
		$err = $this->bdsAccess->dataOpen($dataInfo, "r", "API", BdsDataFlags::DataFlagClipDataToTime, $dataHandle);
		if($err->num){
			return "<b style='color:red;'>Error: BDS system: " . $err->str . "</b><br>";
		}
		
		$err = $this->bdsAccess->dataGetInfo($dataHandle, 0, $dataInfo);
		if($err->num){
			return "<b style='color:red;'>Error: BDS system: " . $err->str . "</b><br>";
		}

		$n = count($dataInfo->channels);
		//debug_log("NumChannels: {$n}");
		
		if(count($dataInfo->channels)){
			$startTime = $dataInfo->startTime;

			for($c = 0; $c < count($dataInfo->channels); $c++){
				$chanName = $dataInfo->channels[$c][0]->network . ":" . $dataInfo->channels[$c][0]->station . ":" . $dataInfo->channels[$c][0]->channel;
				$sampleRate = $dataInfo->channels[$c][0]->sampleRate;
				$tstep = 1.0 / $sampleRate;

				//debug_log("Do channel: {$c} SampleRate: {$sampleRate}");
				$d = $h->appendNode("div", "class='BdsGraph'");
				$d->appendNode("span", "", $chanName);

				$datax = array();
				$datay = array();
				$x = 0;

				for($s = 0; $s < count($dataInfo->channels[$c]); $s++){
					//debug_log("Do segment: {$s}");
					$b = 0;

					while(1){
						$err = $this->bdsAccess->dataGetBlock($dataHandle, $c+1, $s+1, $b, $dataBlock);
						if($err->num == $ErrorEndOfData)
							break;
						if($err->num){
							return "<b>Error: BDS system: " . $err->str . "</b><br>";
							break;
						}

						$td = $dataBlock->startTime->diff($startTime);
						$x = (86400 * $td->days) + (3600 * $td->h) + (60 * $td->i) + $td->s + $td->f;
						//debug_log("Time: {$x} {$tstep} " . print_r($td, 1));

						for($i = 0; $i < count($dataBlock->channelData[0]); $i++){
							$datax[] = floatval($x);
							$datay[] = floatval($dataBlock->channelData[0][$i]);
							$x += $tstep;
						}

						$b += 1;
					}
				}

				$img = $this->plot($datax, $datay);
				$d->appendNode("img", "src='{$img}'");
			}
		}
		
		return $h->render();
	}
	
	function plot($datax, $datay){
		$plot = new PHPlot(800, 200);
		$plot->SetImageBorderType("plain");

		$data = array();
		for($x = 0; $x < count($datax); $x += 1){
			$data[] = array("", $datax[$x], $datay[$x]);
		}
		
		//debug_log("Data: " . print_r($data, 1));
		
		$plot->SetPlotType("lines");
		$plot->SetDataType("data-data");
		$plot->SetDataValues($data);

		//$plot->SetTitle("Seismic amplitude");

		if(0){
			$plot->SetXLabelType('time', '%H:%M:%S');
			$plot->SetXTimeFormat('%H:%M:%S');
		}

		# Select a plot area and force ticks to nice values:
		//$plot->SetPlotAreaWorld($startDate->getSeconds(), 0, $endDate->getSeconds(), 2600);

		# Draw both grids:
		$plot->SetDrawXGrid(True);
		$plot->SetDrawYGrid(True);

		$plot->SetPrintImage(False);
		$plot->SetIsInline(1);
		$plot->DrawGraph();
		return $plot->EncodeImage();
	}

	function viewGraph($uri){
		$page = "<h1>Graph</h1>";
		
		if(isset($_SESSION["sel"])){
			$sel = unserialize($_SESSION["sel"]);
				
			$data = array();
			$err = $this->bdsAccessExec->dataPlot($sel, $this->sys->baseDir . "/files/tmp/graph.png");
			
			if($err->num)
				$page .= "<b>Error fetching data: {$err->str}</b>";
			else
				$page .= "<img src='" . $this->sys->baseUrl . "/files/tmp/graph.png" . "'>";
		}
		else {
			$page .= "No Data has been selected";
		}

		return $page;
	}

	function viewInfo($uri){
		$page = "<h1>Info</h1>";
		
		if(isset($_SESSION["sel"])){
			$sel = unserialize($_SESSION["sel"]);

			$this->bdsAccess->dataSearch($sel, $dataInfo);

			$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align = 'left'", "Item");
			$r->appendNode("th", "align = 'left'", "Value");
			
			foreach($dataInfo->warnings as $value){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", "Warning");
				$r->appendNode("td", "", $value);
			}
			
			foreach($dataInfo->info as $key => $value){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $key);
				$r->appendNode("td", "", $value);
			}
			
			foreach($dataInfo->infoExtra as $key => $value){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $key);
				$r->appendNode("td", "", $value);
			}
			
			$page .= $t->render();
		}
		else {
			$page .= "No Data has been selected";
		}

		return $page;
	}

	function fetchData($uri){
		$page = "";

		if(isset($_SESSION["sel"])){
			$sel = unserialize($_SESSION["sel"]);
			$dataFormat = $_SESSION["dataFormat"];

			$filename = $this->dataFilename($sel, $_SESSION["dataFormatExt"]);
			$filePath = $this->sys->baseDir . "/files/tmp/" . $filename;
			$fileUrl = $this->sys->baseUrl . "/files/tmp/" . $filename;

			$data = array();
			$err = $this->bdsAccessExec->dataGetFormatted($sel, $dataFormat, $filePath);
			if($err->num)
				$page .= "<b>Error fetching data: {$err->str}</b>";
			else
				$page .= "Data is in <a href='{$fileUrl}'>{$fileUrl}</a>";
		}
		else {
			$page .= "No Data has been selected";
		}
		
		return $page;
	}
	
	function dataFilename($sel, $ext){
		$date = $sel->startTime->format("Ymd-His");
		if(count($sel->channels) > 1){
			$stations = $sel->channels[0]->station . "-" . $sel->channels[count($sel->channels) - 1]->station;
		}
		else {
			$stations = $sel->channels[0]->station;
		}
		
		
		return "data-" . $stations . "-" . $date . "." . $ext;
	}
	
	function selToArgs($sel){
		$str = "";
		
		$str .= "startTime={$sel->startTime->format("Y-m-d\TH:i:s")}";
		$str .= "&endTime={$sel->endTime->format("Y-m-d\TH:i:s")}";
		$str .= "&channel={$sel->channels[0]->network}:{$sel->channels[0]->station}:{$sel->channels[0]->channel}:{$sel->channels[0]->source}";

		return $str;
	}

	function selFromArgs($selStr){
		$sel = new BdsSelection();
		$selChannel = new BdsSelectionChannel();
		parse_str($selStr, $list);
		
		$sel->startTime = new DateTime($list["startTime"]);
		$sel->endTime = new DateTime($list["endTime"]);
		
		$channel = explode(":", $list["channel"]);
		$selChannel->network = $channel[0];
		$selChannel->station = $channel[1];
		$selChannel->channel = $channel[2];
		$selChannel->source = $channel[3];
		$sel->channels[] = $selChannel;

		return $sel;
	}

	function selFromArray($list){
		$sel = new BdsSelection();
		$selChannel = new BdsSelectionChannel();
		
		$sel->startTime = new DateTime($list["startTime"]);
		$sel->endTime = new DateTime($list["endTime"]);
		
		$channel = explode(":", $list["channel"]);
		$selChannel->network = $channel[0];
		$selChannel->station = $channel[1];
		$selChannel->channel = $channel[2];
		$selChannel->source = $channel[3];
		$sel->channels[] = $selChannel;

		return $sel;
	}
};

?>
