<?php
/*******************************************************************************
 *	Example.php	Example Module
 *	Copyright	Terry Barnaby,	BeamWeb,	2021-01-05
 *******************************************************************************
 * Testing
 *	Set the "test" variable to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for testing.
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";


class BObjExampleEntry extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("enabled", "Bool"),
			new BMember("title", "String"),
			new BMember("group", "String"),
			new BMember("category", "String"),
			new BMember("location", "String"),
			new BMember("description", "StringHtml"),
			new BMember("route", "String"),
			new BMember("picture0", "String"),
			new BMember("picture1", "String"),
			new BMember("picture2", "String"),
			new BMember("picture3", "String"),
			new BMember("userId", "Int", "Number"),
			new BMember("date", "DateTime"),
			new BMember("dateAdded", "DateTime"),
			));
	}
};

class Example extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "example";
		$this->title = "Example";
		$this->info = "Example module. This module allows the management of Example items.";
		$this->test = $this->sys->test;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("ExampleEntrys" => "BObjExampleEntry"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminExample"));

		// Add HTML functions
		//$this->sys->htmlFuncs[] = "ExampleList";
		//$this->sys->htmlFuncs[] = "ExampleRandom";
		
		// Create the directory			
		$dir = $this->sys->baseDir . "/files/example";
		if(!file_exists($dir)){
			mkdir($dir);
		}
		
		// Add help
		$helpfileSource = $this->sys->baseDir . "/modules/{$this->name}/help.html";	
		$helpfile = $this->sys->baseDir . "/files/help/{$this->name}.html";
		if(!file_exists($helpfile)){
			copy($helpfileSource, $helpfile);
			chmod($helpfile, 0664);
		}

		//if(!$this->test)
		//	$this->sys->page->leftObjectInsert($this->getPageBlockRandom());
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("adminEmailList", "StringList", "", "Administration email addresses", "List of administration email addresses");
		return $configs;
	}

	function checkAccess($admin = 0){
		if($admin)
			return $this->sys->login->checkLogin("admin,adminExample");
		else
			return $this->sys->login->checkLogin();
	}
		
	function checkAccessAdmin(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminExample");
	}

	function checkAccessEntry($obj){
		return $this->checkAccess() || ($this->sys->login->checkLogin() && ($obj->userId == $this->sys->login->id));
	}
	
	function getMenu(){
		if($this->checkAccessAdmin()){
			return new BMenu("Example", "/{$this->name}", "", array("Example List" => "/{$this->name}", "Example Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->test){
			$this->sys->status = "<h1><font color='red'>System is in test mode: {$this->test}.</font></h1>\n";
		}

		if($this->checkAccessAdmin()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "editList":		return $this->editList($uri);		break;
			case "configEdit":		return $this->configEdit($uri);		break;
			case "statistics":		return $this->statistics($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
	}

	function getPageBlock($uri){
		$this->parseUri($uri);

		$help = $this->sys->baseUrl . "/files/help/example.html";

		$m = array();
		$m[] = new BMenu("List All Example", "", "href='?cmd=viewList'");
		if($this->checkAccess())
			$m[] = new BMenu("Add Ride", "", "href='?cmd=add'");
		$m[] = new BMenu("Help", "",		"href='{$help}'");
		if($this->checkAccessAdmin()){
			$m[] = new BMenu("<b>Admin</b>", "", "");
			$m[] = new BMenu("Edit Example", "", "href='?cmd=editList'");
			$m[] = new BMenu("Configuration", "",	"href='?cmd=configEdit'");
			$m[] = new BMenu("Statistics", "",	"href='?cmd=statistics'");
		}
		$o = new BMenu("For Sale", "", "", $m);

		return $o;
	}

	function viewList($uri){
		$this->sys->page->aditemsShow = 1;

		$dateNow = new BDate("now");
		$page = "<h2>Tandem Club Example list</h2>\n";
		$page .= "<p>You can add example using the 'Add Ride' link. You must have a website login account to do this.";

		$where = "(`enabled` = '1')";
		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
			
		$pager = new BHtmlPager(32, $this->sys->db->getObjectNumber("ExampleEntrys", ""));

		$h = new BHtmlNode("table", "class=Example");
		$list = $this->sys->db->getObjectList("ExampleEntrys", "", $where, "!dateAdded", $pager->getRange());
		foreach($list as $obj){
			$r = &$h->appendNode("tr");
			$c = &$r->appendNode("td");
			if(0){
				if(($obj->picturePresent) && file_exists("{$this->sys->baseDir}/files/Example/picture-{$obj->id}-thumb.jpg")){
					$c->appendNode("a", "href='/Example?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/files/Example/picture-{$obj->id}-thumb.jpg'");
				}
				else {
					$c->appendNode("a", "href='/Example?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/modules/Example/TandemExample-thumb.png'");
				}
			}
			else {
				$c->appendNode("a", "href='/example?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/modules/forSale/TandemForSale-thumb.png'");
			}
			$c = &$r->appendNode("td");
			$c->appendNode("div", "class=ExampleEntryHeader")->appendNode("a", "href='/example?cmd=view&id={$obj->id}'", $obj->title);

			if($obj->category != "")
				$c->appendNode("span", "class=ExampleEntryLine", "Category: {$obj->category}");
			if($obj->location != "")
				$c->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");

			$c->appendNode("span", "class=ExampleEntryLine", $obj->description);
		}
		$page .= $h->render();

		return $page;
	}

	function editList($uri){
		$page = "<h2>Example List</h2>\n";
		$dateNow = new BDate("now");

		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
		
		$where = "";	

		$sel = new BSelect("select", "Select", "?cmd=editList", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("period", "dateAdded", "Period", "period", "select", array("", "Last Year", "Last 6 Months", "Last 2 Months", "Last Month", "Last Week")),
			new BViewTypeSelect("startDate", "dateAdded", "Startdate", "greaterThan", "date"),
			new BViewTypeSelect("endDate", "dateAdded", "EndDate", "lessThanEqual", "date"),
			new BViewTypeSelect("enabled", "", "", "equal", "tristate"),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			));

		$sel->getObj()->period = "Last Month";
		$sel->getObj()->enabled = 1;
		$sel->pagerSet(0, 50, 0);
		$sel->update();

		$where = $sel->getSelectString();

		$view = new BViewList("list", "selectAction");
		$view->setView(array(
			new BViewType("id", "ID"),
			new BViewType("enabled", "Enabled"),
			new BViewType("title", "Title"),
			new BViewType("group", "Group"),
			new BViewType("category", "Category"),
			new BViewType("location", "Location"),
			new BViewType("userId", "UserId"),
			new BViewType("date", "Date", "datetime"),
			new BViewType("dateAdded", "Date Published", "datetime"),
			//new BViewType("description", "Description"),
			));
			
		$list = $this->sys->db->getObjectList("ExampleEntrys", "", $where, "!dateAdded", $sel->pagerGetLimitString());
		$totalNumber = $this->sys->db->getObjectNumber("ExampleEntrys", $where);
		$sel->pagerSet(0, 50, $totalNumber);
		$view->setList($list, 1);

		$page = $page . $sel->getHtml();
		$page = $page . $view->getHtml();

		return $page;
	}

	function view($uri){
		// Information
		$page = new BHtmlNode("div");
		$div = &$page->appendNode("div", "class='Ride'");

		if(isset($_GET["id"]) && $obj = $this->sys->db->getObject("ExampleEntrys", filterToInt($_GET["id"]))){
			if($obj->enabled){
				$div->appendNode("h2", "", "Ride: $obj->title");
				if($obj->group != "")
					$div->appendNode("b", "", "Group: ")->appendNode("a", "href='/group/cmd=view&group={$obj->group}'", "{$obj->group}")->appendNode("/br");
				if($obj->category != "")
					$div->appendNode("b", "", "Category: {$obj->category}")->appendNode("/br");
				if($obj->location != "")
					$div->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");

				$div->appendNode("p", "", $obj->description);
				
				$pics = &$div->appendNode("div");
				if($obj->picture0 != ""){
					$pic = "{$this->sys->baseUrl}/files/example/{$obj->id}/{$obj->picture0}";
					$pics->appendNode("a", "href='{$pic}'")->appendNode("img", "src='{$pic}' style='width: 30%; padding:0.2em;'");
				}
				if($obj->picture1 != ""){
					$pic = "{$this->sys->baseUrl}/files/example/{$obj->id}/{$obj->picture1}";
					$pics->appendNode("a", "href='{$pic}'")->appendNode("img", "src='{$pic}' style='width: 30%; padding:0.2em;'");
				}
				if($obj->picture2 != ""){
					$pic = "{$this->sys->baseUrl}/files/example/{$obj->id}/{$obj->picture2}";
					$pics->appendNode("a", "href='{$pic}'")->appendNode("img", "src='{$pic}' style='width: 30%; padding:0.2em;'");
				}
				if($obj->picture3 != ""){
					$pic = "{$this->sys->baseUrl}/files/example/{$obj->id}/{$obj->picture3}";
					$pics->appendNode("a", "href='{$pic}'")->appendNode("img", "src='{$pic}' style='width: 30%; padding:0.2em;'");
				}
				
				if(0){
					// Add items post section
					if($items = $this->sys->module("news")){
						$div->appendText($items->itemListAdd(200, "Example", $obj->id, ""));
					}
				}

				if($this->checkAccessEntry($obj)){
					$buttons = $div->appendNode("div");
					$buttons->appendNode("button", "type='button' name='but2' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=edit&id={$obj->id}\";'", "Edit");
					$buttons->appendNode("button", "type='button' name='but4' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=delete&id={$obj->id}\";'", "Delete");
					//$buttons->appendNode("button", "type='button' name='but5' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=sendToFacebook&id={$obj->id}\";'", "Post to Facebook");
				}
			}
			else {
				$div->appendNode("h2", "", "Ride no longer available");
			}
		}

		$page->appendNode("a", "href='/example' style='font-size: 150%;'", "Show all example");

		return $page->render();
	}
	
	function edit($uri){
		$page = "";
		$edit = 0;	

		if(!$this->checkAccess()){
			return "You need to be logged in to add Example items. <a href='/user?cmd=login&url=/Example'>Login</a>";
		}

		if(is_object($uri)){
			$page = "<h2>Edit Ride</h2>";
			$obj = $uri;
			$edit = 1;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Ride</h2>";
			$obj = $this->sys->db->getObject("ExampleEntrys", filterToInt($_GET["id"]));
			$edit = 1;
		}
		else {
			$dateNow = new BDate("now");
			$obj = $this->sys->db->getNewObject("ExampleEntrys");
			$obj->date = $dateNow->get();
			$obj->dateAdded = $dateNow->get();

			if($this->sys->user){
				$obj->userId = $this->sys->user->id;
				$obj->enabled = 1;
				$obj->location = $this->sys->user->city;
				$obj->emailAddress = $this->sys->user->emailAddress;
			}

			$page = "<h2>Add a new Ride</h2>\n";
			$page .= "<p>";
			$page .= "</p>";
		}

		$view = new BEditObj("Example", "Example Information", "?cmd=save", "");
		if($this->checkAccessAdmin()){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("enabled", "", "checkbox"),
				new BViewType("group"),
				new BViewType("category", "Category", "select", categoryGetList("example")),
				new BViewType("location"),
				new BViewType("route", "", "fileUploaderRoute"),
				new BViewType("picture0", "", "pictureUpload1"),
				new BViewType("picture1", "", "pictureUpload1"),
				new BViewType("picture2", "", "pictureUpload1"),
				new BViewType("picture3", "", "pictureUpload1"),
				new BViewType("description", "None", "textareaHtml"),
				new BViewType("date"),
				new BViewType("dateAdded"),
				));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("title", "Item"),
				new BViewType("enabled", "", "hidden"),
				new BViewType("group"),
				new BViewType("category", "Category", "select", categoryGetList("example")),
				new BViewType("location"),
				new BViewType("route", "", "fileUploaderRoute"),
				new BViewType("picture0", "", "pictureUpload1"),
				new BViewType("picture1", "", "pictureUpload1"),
				new BViewType("picture2", "", "pictureUpload1"),
				new BViewType("picture3", "", "pictureUpload1"),
				new BViewType("description", "None", "textareaHtml"),
				new BViewType("date"),
				new BViewType("dateAdded", "", "hidden"),
				));
		}
		
		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save($uri){
		$page = "";
		$editing = 0;

		//print_r($_GET); print_r($_POST); print("<br>Files:"); print_r($_FILES);
		//BDatabase::$debug = 1;
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjExampleEntry();
			$obj->set($_POST);
			
			unset($obj->picture0);
			unset($obj->picture1);
			unset($obj->picture2);
			unset($obj->picture3);

			// Validate the save
			if(!$this->sys->postValidateCheck($obj->id)){
				$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
				exit();
			}

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if($obj->id){
				$this->sys->db->putObject("ExampleEntrys", $obj);
				$editing = 1;
			}
			else {
				$this->sys->db->appendObject("ExampleEntrys", $obj);
				$this->statisticsAdd("total", 1);
			}
			
			// Create the directory			
			$dir = $this->sys->baseDir . "/files/example/{$obj->id}";
			if(!file_exists($dir)){
				mkdir($dir);
			}

			// Upload any pictures present
			foreach($_FILES as $name => $file){
				if($file["name"] != ""){
					$picId = "";

					//$name = $file["name"]; printf("DoFile: {$name}<br>"); print_r($file);

					if($name == "picture0"){
						$picId = "{$obj->id}";
						$obj->picture0 = "picture-{$picId}.jpg";
					}
					else if($name == "picture1"){
						$picId = "{$obj->id}-1";
						$obj->picture1 = "picture-{$picId}.jpg";
					}
					else if($name == "picture2"){
						$picId = "{$obj->id}-2";
						$obj->picture2 = "picture-{$picId}.jpg";
					}
					else if($name == "picture3"){
						$picId = "{$obj->id}-3";
						$obj->picture3 = "picture-{$picId}.jpg";
					}
					else {
						//print("Copyfile: {$file['tmp_name']} to {$dir}</br>");
						if(copy($file["tmp_name"], $dir . "/route.gpx") == FALSE)
							print("Copy error</br>");
					}

					debug_log("uploadPictures: Do file: {$file["tmp_name"]} to {$picId}\n");

					if($picId != ""){
						// Add picture
						$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
						pictureSaveDir($file["tmp_name"], $ext, "/files/example/{$obj->id}", $picId, 192);
					}
				}
			}
			$this->sys->db->putObject("ExampleEntrys", $obj);

			$page .= "<h3>Example details saved</h3>";

			if(!$editing){
				// Send emails to all who are interested
				//$this->sendEmails($obj);
				
				// Send Facebook post
				if(0){
					$this->facebookPost($obj);
				}

				$subject = "{$this->sys->siteTitle}: Ride Added";
				$message = $subject . "\n";
				$message .= "Your ride has been added. ";
				$message .= "<p>You can edit your ride: <a href='{$this->sys->baseUrl}/example?cmd=edit&id={$obj->id}'>here</a>.</p>";
				//emailList($this->sys->user->emailAddress, $subject, $message, 1);
				
				$this->statisticsAdd("added", 1);
			}

			if($editing)
				$subject = "{$this->sys->siteTitle}: Admin: Ride Edited";
			else
				$subject = "{$this->sys->siteTitle}: Admin: Ride Added";

			$message = $subject . "\n";
			$message .= "Title:        {$obj->title}\n";
			$message .= "View: {$this->sys->baseUrl}/example?cmd=view&id={$obj->id}\n";
			$message .= "Edit: {$this->sys->baseUrl}/example?cmd=edit&id={$obj->id}\n";

			if(0){
				if($this->test)
					emailList($this->sys->testEmailAddress, $subject, $message);
				else
					emailList(configGet("example.adminEmailList"), $subject, $message);
			}
		}

		return $page;
	}

	function delete($uri){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("ExampleEntrys", $id);

		// Check role
		if(!$this->checkAccessEntry($obj)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
			exit();
		}

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("ExampleEntrys", $id);

			$filePaths = array();
			$filePaths[] = $this->sys->baseDir . "/files/Example/picture-{$id}-0";
			$filePaths[] = $this->sys->baseDir . "/files/Example/picture-{$id}-1";
			$filePaths[] = $this->sys->baseDir . "/files/Example/picture-{$id}-2";
			$filePaths[] = $this->sys->baseDir . "/files/Example/picture-{$id}-3";
			foreach($filePaths as $filePath){
				if(file_exists($filePath . ".jpg"))
					unlink($filePath . ".jpg");
				if(file_exists($filePath . "-thumb.jpg"))
					unlink($filePath . "-thumb.jpg");
			}

			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"/example\"'>Back to list</button>";
		}
		else {
			$page = "Not implemented";
		}
			
		return $page;
	}
	
	function sendEmails($obj){
		if($obj){
			// Send an email to everyone interested in receiving emails
			$subject = "{$this->sys->siteTitle}: Example post: {$obj->title}";
			
			$message = $obj->title . "</br>\n";
			if($obj->location != "")
				$message .= "Location: {$obj->location}</br>\n";
			$message .= "{$obj->description}</br>\n\n";
			$message .= "To the Example item: <a href='{$this->sys->baseUrl}/example?cmd=view&id={$obj->id}'>{$this->sys->baseUrl}/example?cmd=view&id={$obj->id}</a></br>\n\n";
			//if($obj->emailAddress != "")
			//	$message .= "Contact email: <a href='mailto:{$obj->emailAddress}'>{$obj->emailAddress}</a>\n\n";
			
			if(0){
				$objList = $this->sys->db->getObjectList("User", "emailAddress", "`ExampleEmailSend` = '1'", "");

				$emailAddressList = array();
				foreach($objList as $o){
					$emailAddressList[] = $o->emailAddress;
				}

				$noreply = preg_replace("/^.*@/", "noreply@", $this->sys->siteEmailAddress);
				if($this->test)
					return emailListFromReply($this->sys->siteEmailAddress, $noreply, $this->sys->testEmailAddress, $subject, $message, 1);
				else
					return emailListFromReply($this->sys->siteEmailAddress, $noreply, $emailAddressList, $subject, $message, 1);
			}
		}
	}


	function parseUri($uri){
		parent::parseUri($uri);
	}
	
	function statisticsAdd($item, $value){
		$this->sys->modules["statistics"]->counterAddNow($this->name, $item, $value);
	}

	function statistics(){
		$f = new BHtmlNode("div");
		
		if(0){
			$f->appendNode("h3", "", "Example Statistics");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Item");

			$r->appendNode("th", "align='left'", "Value");

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Current Example");
			$dateNow = new BDate("now");
			$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmount` = `paymentAmountPaid` and `deleted` != '1' and `sold` != '1')";
			$value = $this->sys->db->getObjectNumber("ExampleEntrys", $where);
			$r->appendNode("td", "", $value);

			// Statistics counters
			$f->appendNode("h3", "", "Example Statistics Counters");

			$date = new BDate("now");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Year");
			$r->appendNode("th", "align='left'", "Month");
			$r->appendNode("th", "align='left'", "Item");
			$r->appendNode("th", "align='left'", "Number");

	//		$list = $this->sys->modules["statistics"]->counterGetAll($date->getYear(), "%", $this->name, "%");
			$list = $this->sys->modules["statistics"]->counterGetAll("%", "%", $this->name, "%");
			foreach($list as $obj){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $obj->year);
				$r->appendNode("td", "", $obj->month);
				$r->appendNode("td", "", $obj->item);
				$r->appendNode("td", "", $obj->number);
			}
		}

		return $f->render();
	}
};

?>
