<?php
/*******************************************************************************
 *	FileBrowser.php	FileBrowser Module
 *	Copyright	Terry Barnaby,	BeamWeb,	2021-01-05
 *******************************************************************************
 * Testing
 *	Set the "test" variable to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for testing.
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

function arrayToString($a){
	$r = "";
	foreach($a as $k => $v)
	        $r .= $k . '=' . $v . ", ";
	return $r;
}

class FileBrowser extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "fileBrowser";
		$this->title = "FileBrowser";
		$this->info = "FileBrowser module. This module allows file access and upload.";
		$this->test = $this->sys->test;
		
		// Add roles
		//$this->sys->modules["user"]->appendRoles(array("adminFileBrowser"));

		// Add HTML functions
		//$this->sys->htmlFuncs[] = "FileBrowserList";
		//$this->sys->htmlFuncs[] = "FileBrowserRandom";
		
		// Add help
		$helpfileSource = $this->sys->baseDir . "/modules/{$this->name}/help.html";
		$helpfile = $this->sys->baseDir . "/files/help/{$this->name}.html";
		if(!file_exists($helpfile)){
			copy($helpfileSource, $helpfile);
			chmod($helpfile, 0664);
		}
		
		$this->images = array("jpg", "jpeg", "png", "gif");

	        $this->get = &$_GET;
        	$this->post = &$_POST;
	        $this->cookie = &$_COOKIE;
		$this->files = &$_FILES;
		$this->filesUploaded = array();

		$this->fileOverwrite = 1;
		$this->thumbSize = 200;

		$this->baseDir = $this->normalize($this->sys->baseDir);
		$this->topDir = $this->normalize($this->sys->baseDir . "/files");
		$this->topUrl = $this->normalize($this->sys->baseUrl . "/files");
		$this->workDir = $this->normalize($this->sys->workDir);
		if(($this->workDir == "") || ($this->workDir[0] != '/'))
			$this->workDir = "/" . $this->workDir;

		//$this->workDir = "/files/t1";

		//debug_log("BaseDir: " . $this->baseDir);
		//debug_log("TopDir: " . $this->topDir);
		//debug_log("TopUrl: " . $this->topUrl);
		//debug_log("WorkDir: " . $this->sys->workDir . " - " . $this->workDir);
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("adminEmailList", "StringList", "", "Administration email addresses", "List of administration email addresses");
		return $configs;
	}

	function checkAccess($admin = 0){
		if($admin)
			return $this->sys->login->checkLogin("admin,adminContent");
		else
			return $this->sys->login->checkLogin();
	}
		
	function checkAccessAdmin(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminFileBrowser");
	}

	function checkAccessEntry($obj){
		return $this->checkAccess() || ($this->sys->login->checkLogin() && ($obj->userId == $this->sys->login->id));
	}
	
	function getMenu(){
		if($this->checkAccessAdmin()){
			return new BMenu("FileBrowser", "/{$this->name}", "", array("FileBrowser List" => "/{$this->name}", "FileBrowser Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->test){
			$this->sys->status = "<h1><font color='red'>System is in test mode: {$this->test}.</font></h1>\n";
		}

		if($this->checkAccess(1)){
			$this->process("");
		}
		else {
			$this->sys->pageDisplayError(new BError(403, "Access denied. You need the adminContent role to browse and upload files"));
			die();
		}

		if(0){
		if($this->checkAccessAdmin()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "editList":		return $this->editList($uri);		break;
			case "configEdit":		return $this->configEdit($uri);		break;
			case "statistics":		return $this->statistics($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		}
	}

	function getPageBlock($uri){
		$this->parseUri($uri);

		$help = $this->sys->baseUrl . "/files/help/fileBrowser.html";

		$m = array();
		$m[] = new BMenu("List All FileBrowser", "", "href='?cmd=browse'");
		$m[] = new BMenu("Help", "",		"href='{$help}'");
		if($this->checkAccessAdmin()){
			$m[] = new BMenu("<b>Admin</b>", "", "");
			$m[] = new BMenu("Configuration", "",	"href='?cmd=configEdit'");
		}
		$o = new BMenu("FileBrowser", "", "", $m);

		return $o;
	}

	function parseUri($uri){
		parent::parseUri($uri);
	}
	
	
	function process($workDir){
		if($workDir != "")
			$this->workDir = $workDir;

		//debug_log("Cmd: " . arrayToString($this->get));
		//debug_log("Post: " .arrayToString($this->post));
		//debug_log("Files: " .arrayToString($this->files));

		$this->cmd = isset($this->get['cmd']) ? $this->get['cmd'] : "browser";
		$this->args = isset($this->get['args']) ? json_decode(stripslashes($this->get['args'])) : null;
		$this->workDir = isset($this->get['workDir']) ? $this->get['workDir'] : $this->workDir;
		
		//debug_log("WorkDir: {$this->workDir}");

		switch($this->cmd){
		case "getDir":		return $this->getDir();	break;
		case "upload":		return $this->upload();	break;
		case "delete":		return $this->delete();	break;
		case "browse":
		default:		return $this->browse();
		}
	}
	
	function browse(){
		global $sys;

		$p = "<!DOCTYPE html>\n";
//		$p = "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN' 'http://www.w3.org/TR/html4/loose.dtd'>\n";
//		$p = "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01//EN' 'http://www.w3.org/TR/html4/strict.dtd'>\n"; 
//		$p = "";
		$h = new BHtmlNode("html", "style='width:100%; height:100%;'");
		$e = &$h->appendNode("head");
		$e->appendNode("meta", "http-equiv='content-type' content='text/html; charset=UTF-8'");
		$b = &$h->appendNode("body", "style='margin: 0px; width:100%; height:100%; overflow:hidden;'");
		$b->appendNode("script", "type='text/javascript'", "var ws_baseUrl = \"{$this->sys->baseUrl}\";");
		$b->appendNode("script", "type='text/javascript'", "var ws_isMobile = \"{$this->sys->isMobile}\";");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/plupload/plupload.full.min.js'");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/javascript/main.js?ver=50'");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/modules/fileBrowser/fileBrowser.js?ver=1'");
		$b->appendNode("link", "type='text/css' rel='stylesheet' href='{$sys->baseUrl}/modules/fileBrowser/fileBrowser.css?ver=1'");

		$b->appendNode("script", "type='text/javascript'", "fileBrowser = new FileBrowser(document.body, '/files', '{$this->workDir}');");
		if(isset($this->get['CKEditor'])){
//			debug_log("FileBrowser::browse: CKEditor: " . $this->get['CKEditor']);
			$ckEditor = $this->get['CKEditor'];
			$ckKEditorFuncNum = $this->get['CKEditorFuncNum'];
			$b->appendNode("script", "type='text/javascript'", "fileBrowser.setCkEditor('{$ckEditor}', {$ckKEditorFuncNum});");
		}
		
		print $p . $h->render();
	}
	
	function dirsHasDirs($dir){
		if(filetype($dir) == "dir"){
			$files = scandir($dir);
			foreach($files as $f){
				if($f[0] != "."){
					$path = $dir . "/" . $f;
					if(filetype($path) == "dir")
						return 1;
				}
			}
		}
		return 0;
	}
	
	function getDir(){
		$dir = $this->topDir . "/" . $this->args->dir;
		$files = scandir($dir);
		$res = array();
		foreach($files as $f){
			if($f[0] != "."){
				$path = $dir . "/" . $f;
				$relPath = $this->normalize($this->args->dir ."/" . $f);
				$res[$f] = array("name" => $f, "path" => $relPath, "type" => filetype($path), "size" => filesize($path),
					"time" => filemtime($path), "hasDirs" => $this->dirsHasDirs($path));
				$this->thumbnailCreate($relPath);
			}
		}
		print json_encode($res);
	}

	function upload(){
		// debug_log("FileBrowser::upload");
		
		$this->filesUploaded = array();
		
		// Set and validate path
		$dir = $this->post["dir"];
		$path = $this->normalize($this->topDir . "/" . $dir);
		// debug_log("Upload: Path: " . $path);
		
		if(strncmp($this->topDir, $path, strlen($this->topDir))){
			debug_log("FileBrowser: upload: Path Error: {$path} != {$this->topDir}");
			return;
		}
		
		// Create sensible array of upload files
		foreach($this->files["upload"] as $k => $all){
			foreach($all as $i => $v){
		        	$files[$i][$k] = $v;   
			}   
		}
		// debug_log("Files: " . arrayToString($files));
		
		foreach($files as $file){
			// Get file extension
			$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));

			if($this->fileOverwrite){
				$n = pathinfo($file["name"], PATHINFO_FILENAME);
				$e = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
				$name = "{$n}.{$e}";
			}
			else {			
				// Modify filename
				$name = $this->getNewFilename($path, $file["name"]);
			}

			// debug_log("File: " . arrayToString($file) . " Name: " . $name . " Ext: " . $ext);
			
			if(in_array($ext, $this->images)){
				//debug_log("FileBrowser::imageSave: image: " . $file["tmp_name"] . " to: " . $path . "/" . $name);
				$this->imageSave($file["tmp_name"], $path . "/" . $name, $this->sys->pictureWidth);
				$this->thumbnailCreate($dir . "/" . $name, 1);
			}
			else {
				// debug_log("move_uploaded_file: " . $file["tmp_name"] . " to: " . $path . "/" . $name);
				move_uploaded_file($file["tmp_name"], $path . "/" . $name);
			}
			$this->filesUploaded[] = $name;
		}
		print json_encode($this->filesUploaded);
	}
	
	function delete(){
		// debug_log("FileBrowser::delete: " . print_r($_GET, 1));
		$res = array();
		if($this->sys->login->checkLogin("adminDelete")){
			if($this->args){
				$path = $this->normalize($this->args->path);
				// debug_log("FileBrowser::delete: " . $path);
				unlink($this->sys->baseDir . $path);

				$res["error"] = "";
			}
			else {
				$res["error"] = "Error not deleted. Internal error.";
			}
		}
		else {
			$res["error"] = "Error not deleted. You need the adminDelete role in order to delete items";
		}
		print json_encode($res);
	}
	
	function thumbnailCreate($path, $force = 0){
		$pathSource = $this->topDir . "/" . $path;
		$pathThumb = $this->topDir . "/.thumbnails/" . $path;
		$dir = pathinfo($path, PATHINFO_DIRNAME);
		$ext = strtolower(pathinfo($path, PATHINFO_EXTENSION));

		//debug_log("FileBrowser::thumbnailCreate: Forces: {$force} Source: {$pathSource} Dest: {$pathThumb} {$ext}");
		if(!file_exists($this->topDir . "/.thumbnails")){
			//debug_log("thumbnailCreate: mkdir: " . $this->topDir . "/.thumbnails");
			mkdir($this->topDir . "/.thumbnails", 0777, true);
		}

		if($force || !file_exists($pathThumb)){
			if(in_array($ext, $this->images)){
				$dirThumb = pathinfo($pathThumb, PATHINFO_DIRNAME);
				if(!file_exists($dirThumb))
					mkdir($dirThumb, 0777, true);

				$this->imageSave($pathSource, $pathThumb, $this->thumbSize);
			}
		}
	}

	function imageSave($source, $dest, $maxSize, $move=0){
		$e = strtolower(pathinfo($source, PATHINFO_EXTENSION));
		list($iw, $ih, $it) = getimagesize($source);

		// debug_log("FileBrowser::imageSave: $source: $iw x $ih");
		
		if(($iw > $maxSize) || ($ih > $maxSize)){
			//debug_log("FileBrowser::imageSave: resize: $source to $dest");

			// Save out of memory on server
			if(($iw * $ih) >= (4000 * 3000))
				return;

			$aspect = (float)$iw / (float)$ih;
			if($aspect > 1.0){
				$w = $maxSize;
				$h = (int)((float)$w/$aspect);
			}
			else {
				$h = $maxSize;
				$w = (int)((float)$h * $aspect);
			}
			switch($it){
			case IMAGETYPE_GIF:	$image = imagecreatefromgif($source);	break;
			case IMAGETYPE_WBMP:	$image = imagecreatefromwbmp($source);	break;
			case IMAGETYPE_JPEG:	$image = imagecreatefromjpeg($source);	break;
			case IMAGETYPE_JPEG2000:	$image = imagecreatefromjpeg($source);	break;
			case IMAGETYPE_PNG:	$image = imagecreatefrompng($source);	break;
			case IMAGETYPE_XBM:	$image = imagecreatefromxbm($source);	break;
			default:	return;
			}

			$pic = imagecreatetruecolor($w, $h);
			imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $iw, $ih);
			
			switch($it){
			case IMAGETYPE_GIF:	imagegif($pic, $dest);	break;
			case IMAGETYPE_WBMP:	imagewbmp($pic, $dest);	break;
			case IMAGETYPE_JPEG:		imagejpeg($pic, $dest);	break;
			case IMAGETYPE_JPEG2000:	imagejpeg($pic, $dest);	break;
			case IMAGETYPE_PNG:	imagepng($pic, $dest);	break;
			case IMAGETYPE_XBM:	imagexbm($pic, $dest);	break;
			default:	return;
			}
		}
		else {
			//debug_log("FileBrowser::imageSave: move_uploaded_file: $source to $dest");
			if($move)
				move_uploaded_file($source, $dest);
			else
				copy($source, $dest);
		}
	}

	function getNewFilename($dir, $name){
		$n = pathinfo($name, PATHINFO_FILENAME);
		$s = "";
		$e = strtolower(pathinfo($name, PATHINFO_EXTENSION));
		$e = strlen($e) ? ".{$e}" : $e;

		$n = preg_replace('/\s+/', "_", $n);
		$path = "{$dir}/${n}${s}${e}";
		$i = 1;
		while(file_exists($path)){
			$s = "({$i})";
			$path = "{$dir}/${n}${s}${e}";
			$i += 1;
		}

		return "${n}${s}${e}";
	}

	function normalize($path){
		$path = preg_replace('/\/+/s', "/", $path);

		$path = "/$path";
		if (substr($path, -1) != "/")
			$path .= "/";

		$expr = '/\/([^\/]{1}|[^\.\/]{2}|[^\/]{3,})\/\.\.\//s';
		while (preg_match($expr, $path))
			$path = preg_replace($expr, "/", $path);

		$path = substr($path, 0, -1);
		$path = substr($path, 1);
		return $path;
	}

	public	$get;
	public	$post;
	public	$cookie;
	public	$files;
	
	public	$baseDir;
	public	$topDir;
	public	$topUrl;
};

?>
