<?php
/*******************************************************************************
 *	News.php	News Module
 *	Copyright	Terry Barnaby,	BeamWeb,	2021-01-05
 *******************************************************************************
 *
 * Testing
 *
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjItem extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("parentId", "Int", "Number"),
			new BMember("grandparentId", "Int", "Number"),
			new BMember("type", "String"),
			new BMember("objectId", "Int", "Number"),
			new BMember("userId", "Int", "Number"),
			new BMember("userName", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("title", "String"),
			new BMember("description", "StringHtml"),
			new BMember("pictureIds", "String"),
			new BMember("hide", "Bool"),
			new BMember("date", "DateTime"),
			new BMember("dateChildren", "DateTime"),
			new BMember("dateAdded", "DateTime"),
			new BMember("dateExpiry", "DateTime"),
			new BMember("lastUpdate", "DateTime", "Datetime", Null, "timestamp", "default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP"),
		));
	}
};

class News extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "news";
		$this->title = "News";
		$this->info = "News module. This module provides management of all items including news, forum entries, messages etc.";
		//$this->types = categoryGetList("ItemTypes");
		$this->types = array("news", "forSale", "rides", "places");
		$this->categories = categoryGetList("ItemCategories");
		$this->group = "";
		$this->test = $this->sys->test;
		//$this->test = 1;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Items" => "BObjItem"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminNews"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "newsList";
		$this->sys->htmlFuncs[] = "newsListAdd";
		$this->sys->htmlFuncs[] = "newsListGroups";
		$this->sys->htmlFuncs[] = "itemList";
		$this->sys->htmlFuncs[] = "itemListAdd";

		// Create the directory			
		$dir = $this->sys->baseDir . "/files/items";
		if(!file_exists($dir)){
			mkdir($dir);
		}

		// Add help
		$helpfileSource = $this->sys->baseDir . "/modules/{$this->name}/help.html";	
		$helpfile = $this->sys->baseDir . "/files/help/{$this->name}.html";
		if(!file_exists($helpfile)){
			copy($helpfileSource, $helpfile);
			chmod($helpfile, 0664);
		}
		
		if(!in_array("news", $this->types)){
			categoryAdd("ItemTypes", "news");
		}
		if(!in_array("forSale", $this->types)){
			categoryAdd("ItemTypes", "forSale");
		}
		if(!in_array("rides", $this->types)){
			categoryAdd("ItemTypes", "rides");
		}
		
		// Get groups user can use
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("adminEmail", "StringList", "", "Admin Email", "Admin email list (comma separated)");
		return $configs;
	}

	function checkAccess($admin = 0){
		if($admin)
			return $this->sys->login->checkLogin("admin,adminNews") || $this->sys->login->checkLogin("adminGroup" . $this->group);
		else
			return $this->sys->login->checkLogin();
	}
		
	function checkAccessAdminGroup($group){
		return $this->sys->login->checkLogin("admin,adminNews") || $this->sys->login->checkLogin("adminGroup" . $group);
	}
		
	function checkAccessMember(){
		if($this->sys->login->checkLogin("admin,adminNews") || $this->sys->login->checkLogin("adminGroup" . $this->group))
			return 1;
		
		return $this->sys->login->membershipId;
	}
		
	function getMenu(){
		if($this->checkAccess(1)){
			return new BMenu("News", "/{$this->name}", "", array("News List" => "/{$this->name}"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);
		
		//print "Uri: {$this->uri}<br/>";

		if($this->checkAccess(1)){
			switch($this->cmd){
			case "":
			case "viewNews":	$page = $this->viewNews($uri); break;
			case "editList": 	$page = $this->editList(); break;
			case "configEdit":	$page = $this->configEdit($uri); break;
			case "view":		$page = $this->view($uri); break;
			case "add":		$page = $this->edit("0"); break;
			case "edit":		$page = $this->edit($uri); break;
			case "save":		$page = $this->save($uri); break;
			case "delete":		$page = $this->delete($uri); break;
			case "updatePicture":	$page = $this->updatePicture($uri); break;
			case "userSettings":	$page = $this->userSettings($uri); break;
			case "getUpdateNum":	$page = $this->getUpdateNum(); break;
			case "getMessages":	$page = $this->getMessages($uri); break;
			case "help":		$page = $this->help($uri); break;
			
			//case "copyNews":	$page = $this->copyNews($uri); break;

			default: 		$page = $this->viewNews($uri); break;
			}
			return $page;
		}
		else if($this->checkAccess(0)){
			switch($this->cmd){
			case "":
			case "viewNews":	$page = $this->viewNews($uri); break;
			case "view":		$page = $this->view($uri); break;
			case "add":		$page = $this->edit("0"); break;
			case "edit":		$page = $this->edit($uri); break;
			case "save":		$page = $this->save($uri); break;
			case "delete":		$page = $this->delete($uri); break;
			case "updatePicture":	$page = $this->updatePicture($uri); break;
			case "userSettings":	$page = $this->userSettings($uri); break;
			case "getUpdateNum":	$page = $this->getUpdateNum(); break;
			case "getMessages":	$page = $this->getMessages($uri); break;
			case "help":		$page = $this->help($uri); break;
			
			//case "copyNews":	$page = $this->copyNews($uri); break;

			default: 		$page = $this->viewNews($uri); break;
			}
			return $page;
		}
		else {
			switch($this->cmd){
			case "":
			case "viewNews":	$page = $this->viewNews($uri); break;
			case "view":		$page = $this->view($uri); break;
			case "getUpdateNum":	$page = $this->getUpdateNum(); break;
			case "getMessages":	$page = $this->getMessages($uri); break;
			case "help":		$page = $this->help($uri); break;
			default: 		$page = $this->sys->pageDisplayError(new BError(1, "You are not allowed to access this page")); break;
			}
			return $page;
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess(1)){
			$help = $this->sys->baseUrl . "/files/help/news.html";

			$o = new BMenu("News", "", "", array(
				new BMenu("List", "", "href='?cmd=viewNews'"),
				new BMenu("Add", "", "href='?cmd=add'"),
				new BMenu("<b>Management</b>", "", ""),
				new BMenu("EditList", "", "href='?cmd=editList'"),
				new BMenu("Configuration", "",	"href='?cmd=configEdit'"),
				new BMenu("Help", "",		"href='{$help}'"),
				new BMenu("<b>Special</b>", "", ""),
				//new BMenu("Copy News", "",	"href='?cmd=copyNews'"),
				));
		}
		else if($this->checkAccess(0)){
			$help = $this->sys->baseUrl . "/files/help/news.html";

			$o = new BMenu("News", "", "", array(
				new BMenu("List", "", "href='?cmd=viewNews'"),
				new BMenu("Add", "", "href='?cmd=add'"),
				new BMenu("Help", "",		"href='{$help}'"),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}

	function editList(){
		$page = "<h2>News information</h2>\n";
		
		if(!$this->checkAccess(1)){
			$this->sys->pageDisplayError(new BError(1, "You are not allowed to access News"));
		}
		
//		print "Form: "; print_r($_POST); print "<br>\n";
		if(isset($_GET["sel_group"]))
			$this->group = filterString($_GET["sel_group"]);
		else
			$_GET["sel_group"] = $this->group;

				
		$sel = new BSelect("select", "Select", "?cmd=editList", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("period", "date", "Period", "period", "select", array("", "Last Year", "Last 6 Months", "Last 2 Months", "Last Month", "Last Week")),
			new BViewTypeSelect("startDate", "date", "Startdate", "greaterThan", "date"),
			new BViewTypeSelect("endDate", "date", "EndDate", "lessThanEqual", "date"),
			new BViewTypeSelect("type", "", "", "equal", "select", array_merge(array(""), $this->types)),
			new BViewTypeSelect("objectId", "", "", "equal", "text"),
			new BViewTypeSelect("category", "", "", "equal", "select", array_merge(array(""), $this->categories)),
			new BViewTypeSelect("group", "", "", "equal", "select", array_merge(array(""), $this->sys->groups)),
			//new BViewTypeSelect("", "", "", "", "heading"),
			new BViewTypeSelect("order", "", "", "order", "select", array("id", "!id", "date", "!date", "dateAdded", "!dateAdded", "type", "group")),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			));

		$sel->getObj()->period = "Last 6 Months";
		$sel->pagerSet(0, 50, 0);
		$sel->update();
		
		$where = $sel->getSelectString();
		$order = $sel->getOrder();
		if($order == "")
			$order = "!dateAdded";

//		print "Where: {$where}<br>\n";
//		BDatabase::$debug = 1;
		
		$view = new BViewList("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("date", "Date", "datetime"),
			new BViewType("dateAdded", "Date Added", "datetime"),
			new BViewType("type", "Type"),
			new BViewType("objectId", "ObjectId"),
			new BViewType("group", "Group"),
			new BViewType("title", "Title"),
			//new BViewType("description", "Message"),
			));
		
		$list = $this->sys->db->getObjectList("Items", "", $where, $order, $sel->pagerGetLimitString());
		$totalNumber = $this->sys->db->getObjectNumber("Items", $where);
		$sel->pagerSet(0, 50, $totalNumber);

		if($this->group){
			$view->setGroup($this->group);
		}
		$view->setList($list, 1);

		$page = $page . $sel->getHtml();
		$page = $page . $view->getHtml();

		return $page;
	}
	

	function view1($arg){
		$page = "<h2>View Item</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("Items", filterToInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		
		$view = new BViewObj("payment", "Item View");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("parentId"),
			new BViewType("grandparentId"),
			new BViewType("userId"),
			new BViewType("userName"),
			new BViewType("category"),
			new BViewType("group"),
			new BViewType("title", "Title"),
			new BViewType("description", "Description"),
			new BViewType("pictureIds"),
			new BViewType("date"),
			new BViewType("dateChildren"),
			new BViewType("dateAdded", "Date Added"),
			new BViewType("lastUpdate"),
			));

		$view->setObj($obj);
		
		$picture = "";
		if($obj->pictureIds != ""){
			foreach(explode(",", $obj->pictureIds) as $pid){
				$picture .= pictureHtml(intval($pid), "", 0, 1, 0, "align='left' style='padding-right:5px;'");
			}
		}

		$page .= $view->getHtml();
		$page .= $picture;
		return $page;
	}
	
	function view($arg){
		$page = "<h2>View Item</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("Items", filterToInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}

		$html = new BHtmlNode("div", "class='NewsBox'");
		if($obj)
			$this->viewObject($html, $obj);
		else
			$html->appendNode("div", "", "No such news item");
		
		$page .= $html->render();
		
		return $page;
	}
	
	function edit($arg, $group = ""){
		if($group == "")
			$group = $this->group;
		
		if($group == "")
			$group = $this->sys->user->group;
		
		if(isset($_GET["type"])){
			$type = filterString($_GET["type"]);
		}
		else {
			$type = "news";
		}
		if(isset($_GET["objectId"])){
			$objectId = filterString($_GET["objectId"]);
		}
		else {
			$objectId = 0;
		}
			
		$args = "";
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			if(filterToInt($_GET["id"]) == 0)
				return "No payment record id 0.";
			$page = "<h2>Edit Item</h2>";
			$obj = $this->sys->db->getObject("Items", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Items");
			$obj->id = 0;
			$obj->type = $type;
			$obj->objectId = $objectId;
			$obj->group = $group;
			$obj->date = strftime("%Y-%m-%d %H:%M:%S");
			$obj->dateAdded = strftime("%Y-%m-%d %H:%M:%S");
			if($this->sys->user){
				$obj->userId = $this->sys->user->id;
				$obj->userName = $this->sys->user->name;
			}
		}

		$view = new BEditObj("item", "Detailed Form to post message", "/news?cmd=save", "");
		if($this->checkAccess(1)){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("type", "Type", "select", $this->types),
				new BViewType("category", "Category", "select", $this->categories),
				new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
				new BViewType("description", "None", "textareaHtmlSource"),
				new BViewType("pictureIds", "", "hidden"),
				//new BViewType("pictureIds", "", ""),
				//new BViewType("pictures", "", "pictureUploaderLineSmall"),
				new BViewType("pictures", "None", "fileUploader"),
				new BViewType("hide", "Hide item", "checkbox"),
				new BViewType("sendEmail", "Send email to group", "checkbox"),
				//new BViewType("sendFacebook", "Post to Facebook group", "checkbox"),
				new BViewType("dateAdded", "Date Added", "view"),
				new BViewType("date", "", ""),
				new BViewType("dateExpiry", "", ""),
				new BViewType("dateChildren", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("userName", "", "view"),
				new BViewType("objectId", "", "hidden"),
				));
		}
		else if($this->checkAccessMember()){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("type", "Type", "select", $this->types),
				new BViewType("category", "Category", "select", $this->categories),
				new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
				new BViewType("description", "None", "textareaHtml"),
				new BViewType("pictureIds", "", "hidden"),
				//new BViewType("pictureIds", "", ""),
				//new BViewType("pictures", "", "pictureUploaderLineSmall"),
				new BViewType("pictures", "None", "fileUploader"),
				new BViewType("hide", "Hide item", "checkbox"),
				new BViewType("sendEmail", "Send email to group", "checkbox"),
				//new BViewType("sendFacebook", "Post to Facebook group", "checkbox"),
				new BViewType("dateAdded", "Date Added", "view"),
				new BViewType("date", "", "hidden"),
				new BViewType("dateExpiry", "", "hidden"),
				new BViewType("dateChildren", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("userName", "", "hidden"),
				new BViewType("objectId", "", "hidden"),
				));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("type", "Type", "select", $this->types),
				new BViewType("category", "Category", "select", $this->categories),
				new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
				new BViewType("description", "None", "textareaHtml"),
				new BViewType("pictureIds", "", "hidden"),
				//new BViewType("pictureIds", "", ""),
				//new BViewType("pictures", "", "pictureUploaderLineSmall"),
				new BViewType("pictures", "None", "fileUploader"),
				new BViewType("hide", "Hide item", "checkbox"),
				//new BViewType("sendEmail", "Send email to group", "checkbox"),
				//new BViewType("sendFacebook", "Post to Facebook group", "checkbox"),
				new BViewType("dateAdded", "Date Added", "view"),
				new BViewType("date", "", "hidden"),
				new BViewType("dateExpiry", "", "hidden"),
				new BViewType("dateChildren", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("userName", "", "hidden"),
				new BViewType("objectId", "", "hidden"),
				));
		}

		$_SESSION["lastId"] = 0;
		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function save($uri){
		$page = "";

		if(0){
			debug_log(print_r($_POST, 1));
			debug_log(print_r($_FILES, 1));
			debug_log(print_r($_SESSION, 1));
		}

		if($this->sys->login->checkLogin()){
			if(count($_POST)){
				unset($_POST['cmd']);
				$obj = new BObjItem();
				$obj->set($_POST);
				
				// Multiple picture upload support
				if(isset($_POST["fileUploader"]) && (intval($_POST["fileUploader"]) > 1)){
					$obj->id = filterToInt($_SESSION["lastId"]);
					$_SESSION["saveId"] = $obj->id;
					$pobj = $this->sys->db->getObject("Items", $obj->id);
					$obj->pictureIds = $pobj->pictureIds;
					//debug_log("Object id: {$obj->id} '{$obj->pictureIds}'\n");
				}

				// Validate the save
				if(!$this->sys->postValidateCheck($obj->id)){
					$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
					exit();
				}

				if($err = $obj->validate()){
					$items = implode(",", array_keys($err));
					$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
					return $this->edit($obj, $payDo);
				}
				
				$obj->description = htmlRemoveJavascript($obj->description);

				// Upload any pictures present
				if((count($_FILES) == 1) && isset($_FILES["pictures"])){
					$file = $_FILES["pictures"];
					//debug_log("uploadPictures: Do file: {$file["tmp_name"]}\n");

					// Add picture
					$pobj = new BObjPicture();
					$pobj->category = "news";
					$pobj->group = $obj->group;
					$pobj->dateAdded = strftime("%Y-%m-%d");
					$pobj->title = $obj->title;
					$pobj = $this->sys->db->appendObject("Picture", $pobj);
					pictureSave($pobj->id, $file["tmp_name"]);
					if(0){
						if($obj->pictureIds != "")
							$obj->pictureIds .= ",";
						$obj->pictureIds .= $pobj->id;
					}
					else {
						$obj->pictureIds = $pobj->id;
					}
				}

				if($obj->id){
					$this->sys->db->putObject("Items", $obj);
				}
				else {
					$this->sys->db->appendObject("Items", $obj);
				}
				$_SESSION["lastId"] = $obj->id;
				$this->updateNumIncrement();

				if($this->checkAccessMember()){
					// Send Facebook post
					if(isset($_POST['sendFacebook']) && (intVal($_POST['sendFacebook']) == 1)){
						$message = $obj->title;
						if($message != "")
							$message .= "\n";
						$description = htmlToText($obj->description);
						if(strlen($description) >= 200){
							$description = substr($description, 0, 200) . " ... more";
						}
						$message .= $description . "\n";
						$message .= "To the news: {$this->sys->baseUrl}/news?group={$obj->group}\n";

						if(1 && ($facebook = $this->sys->module("facebook"))){
							$facebook->postMessage("TandemClubPage", $message, "{$this->sys->baseUrl}/news?group={$obj->group}");
							$page .= "Posted to facebook group";
						}
					}
					// Send emails
					if(isset($_POST['sendEmail']) && (intval($_POST['sendEmail']) == 1)){
						if($groups = $this->sys->module("groups")){
							$message = $obj->description;
							$message .= "<p>To the News system to view and/or reply: <a href='{$this->sys->baseUrl}/news?group{$obj->group}'>{$this->sys->baseUrl}/news?group{$obj->group}</a>\n";

							$page .= $groups->emailGroup($obj->group, "TandemClub news: " . $obj->title, $message);
						}
					}

				}

				$page .= "<h3>Item saved</h3>";
			}
			else {
				$page .= "<h3>Error: Nothing to save</h3>";
			}

		}
		else {
			return "You need to <a href='/'>Login</a> to post";
		}

		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";

		return $page;
	}
	
	function delete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminNews")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' and 'adminNews' roles can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Items", $id);
			$this->sys->db->deleteObject("Items", $id);
			$page = "<h3>Payment: {$obj->id} Titled: {$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Items", $id);
			$title = $obj->title;
			if($title == "")
				$title = substr($obj->description, 0, 30) . "...";

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete item: {$obj->id}: {$title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}


	//*************************************************************************************************************************************
	// User views for simple messaging
	//*************************************************************************************************************************************

	function viewObject($messages, $obj){
		$m = &$messages->appendNode("div", "class=Message");
		$h = &$m->appendNode("div", "class=MessageHead");
		$pic = userGetPicture($obj->userId);
		if($pic == ""){
			if(file_exists("{$this->sys->baseDir}/themes/{$this->sys->style}/userPicture.jpg")){
				$pic = "{$this->sys->baseUrl}/themes/{$this->sys->style}/userPicture.jpg";
			}
			else {
				$pic = "{$this->sys->baseUrl}/icons/users/cycleHelmit.jpg";
			}
		}
			
		//$pic = "{$this->sys->baseUrl}/modules/message/terry.jpg";
		
		$h->appendNode("a", "href='?cmd=updatePicture'")->appendNode("div", "class=MessageImageUser")->appendNode("img", "src='{$pic}'");
		$t = &$h->appendNode("div", "class=MessageTitles");
		$t->appendNode("span", "class=MessageName", $obj->userName);
		$t->appendNode("span", "class=MessageTime", $obj->date);
		$t->appendNode("span", "class=MessageGroup")->appendNode("a", "href='/groups/{$obj->group}'", $obj->group);
		if($this->checkAccessAdminGroup($obj->group)){
			$d = &$t->appendNode("span");
			$d->appendText("(");
			$d->appendNode("a", "href='/news?cmd=edit&id={$obj->id}' style='padding-left: 0.2em;' title='Edit news item'", "<b>E</b>");
			$d->appendNode("a", "href='/news?cmd=delete&id={$obj->id}' style='padding-left: 0.2em;' title='Delete news item'", "<b>, D</b>");
			$d->appendText(")");
		}
		
		$t->appendNode("div", "class=MessageGroup", $obj->title);
		
		$d = &$m->appendNode("div", "class=MessageContent");
		if($obj->pictureIds != ""){
			$pid = explode(",", $obj->pictureIds)[0];
			$d->appendText(pictureHtml(intval($pid), "", 0, 0, 0, "class='MessagePicture'"));
		}

		//$d->appendNode("p", "", $obj->description);
		$d->appendNode("p", "", htmlClean($obj->description));

		$d1 = &$d->appendNode("div", "style='clear: both;'");
		if($obj->pictureIds != ""){
			$list = explode(",", $obj->pictureIds);
			for($i = 1; $i < count($list); $i++){
				$d1->appendText(pictureHtml(intval($list[$i]), "", 0, 0, 0, "class='MessagePictureThumb'"));
			}
		}
		
	}

	function viewSave($uri){
		$newItem = 0;
		
		//print_r($_POST);
		
		if($this->sys->login->checkLogin()){
			$obj = new BObjItem();
			if($this->sys->user){
				$obj->userId = $this->sys->user->id;
				$obj->userName = $this->sys->user->name;
			}
			$obj->type = filterString($_POST["type"]);
			$obj->objectId = filterInt($_POST["objectId"]);
			$obj->group = filterString($_POST["newPostGroup"]);
			$obj->description  = filterString(htmlRemoveJavascript($_POST["newPostComment"]));
			$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");
			$obj->date = strftime("%Y-%m-%dT%H:%M:%S");
			$obj->dateChildren = strftime("%Y-%m-%dT%H:%M:%S");
			$obj->pictureIds = "";
			
			// Multiple picture upload support
			if(isset($_POST["fileUploader"]) && (intval($_POST["fileUploader"]) > 1)){
				$obj->id = filterToInt($_SESSION["lastId"]);
				$_SESSION["saveId"] = $obj->id;
				$pobj = $this->sys->db->getObject("Items", $obj->id);
				$obj->pictureIds = $pobj->pictureIds;
				//debug_log("Object id: {$obj->id} '{$obj->pictureIds}'\n");
			}

			// Validate the save
			if(!$this->sys->postValidateCheck(0)){
				$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
				exit();
			}

			if($obj->id){
				$this->sys->db->putObject("Items", $obj);
			}
			else {
				$newItem = 1;
				$this->sys->db->appendObject("Items", $obj);
			}

			// Upload any pictures present
			if((count($_FILES) == 1) && isset($_FILES["pictures"]) && ($_FILES["pictures"]["tmp_name"] != "")){
				$file = $_FILES["pictures"];
				$s = getimagesize($file["tmp_name"]);
				$x = $s[0];
				$y = $s[1];
				//debug_log("uploadPictures: Do file: {$file["tmp_name"]}, {$x} x {$y}\n");

				// Add picture
				$pobj = new BObjPicture();
				$pobj->source = "items";
				$pobj->sourceId = $obj->id;
				$pobj->group = $obj->group;
				$pobj->dateAdded = strftime("%Y-%m-%d");
				$pobj->title = $obj->title;
				$pobj = $this->sys->db->appendObject("Picture", $pobj);
				pictureSave($pobj->id, $file["tmp_name"]);
				if($obj->pictureIds != "")
					$obj->pictureIds .= ",";
				$obj->pictureIds .= $pobj->id;

				$this->sys->db->putObject("Items", $obj);
			}
			$this->updateNumIncrement();
			
			if($newItem)
				$this->sendNotifications($obj);

			return "";
		}
		else {
			return "You need to <a href='/'>Login</a> to post";
		}
	}
	
	function getUpdateNum(){
		echo $this->updateNum();
		return Null;
	}
	
	function getMessagesContent($m, $where, $num = 20){
		//debug_log("News::getMessagesContent: {$where}");
		//BDatabase::$debug = 1;
		//$list = $this->sys->db->getObjectList("Items", "", $where, "date", $num);
		$table = $this->sys->databasePrefix . "Items";;
		$list = $this->sys->db->queryObjs("Items", "(select * from {$table} where {$where} order by id desc limit {$num}) order by date");
		
		foreach($list as $obj){
			$this->viewObject($m, $obj);
		}
	}

	function getMessages($uri){
		//debug_log("News::getMessages: " . print_r($_GET, 1));
		
		$where = "`hide`='0'";
		if(isset($_GET["type"])){
			$type = filterString($_GET["type"]);
			if($where != "")
				$where .= " and ";
			$where .= "`type`='{$type}'";
		}
		if(isset($_GET["objectId"])){
			$objectId = filterString($_GET["objectId"]);
			if($where != "")
				$where .= " and ";
			$where .= "`objectId`='{$objectId}'";
		}
		
		if(isset($_GET["group"]) && ($_GET["group"] != "") && ($_GET["group"] != "All")){
			$group = filterString($_GET["group"]);
			if($where != "")
				$where .= " and ";
			$where .= "`group` = '{$group}'";
		}
		$m = new BHtmlNode("div", "style='height: 100%'");
		$this->getMessagesContent($m, $where);
		echo($m->render());
		return NULL;
	}
	
	function viewNews($uri){
		//print_r($_POST);
		$type = "news";

		if(isset($_POST["newPostGroup"]))
			$group = filterString($_POST["newPostGroup"]);
		else
			$group = "";
			
		$objectId = 0;
		if(($group == "") && isset($_GET['group']))
			$group = filterString($_GET['group']);

		if($this->checkAccess() && isset($_GET["notifications"])){
			$this->sys->user->newsNotifications = 1;
			$this->sys->db->putObject("User", $this->sys->user);
		}
		
		if(isset($_POST["advanced"])){
			header("Location: /news?cmd=edit&type={$type}&objectId={$objectId}&group={$group}");
			//return $this->edit($uri, filterString($_POST["newPostGroup"]));
			return null;
		}
		
		if($this->sys->isMobile)
			$this->sys->page->setTemplate("page-messages.template");

		$this->sys->page->addStyleSheet("fixed-height.css");
		
		if((isset($_POST["newPostComment"]) && ($_POST["newPostComment"] != "")) || (isset($_POST["comment"]) && ($_POST["comment"] != ""))){
			if(!$this->sys->login->checkLogin()){
				return "You need to <a href='/user?cmd=login&url=/news'>Login</a> to post";
			}
			
			//$_SESSION["newPostGroup"] = $_POST["newPostGroup"];
			$r = $this->viewSave($uri);
			$url = "?newPostGroup=" . $_POST["newPostGroup"];
			if(isset($_GET["group"]))
				$url .= "&group=" . $_GET["group"];

			header("Location: {$url}");
			if($r != ""){
				print $r;
				die();
			}
		}

		$where = "`type` = 'news' and `hide` = '0'";
		
		$group = "All";
		if(isset($_GET["group"]))
			$group = $_GET["group"];
		
		if($group != "All")
			$where .= " and `group` = '" . $group . "'";

		$page = new BHtmlNode("div", "class='MessagePage' id='messagePage'");

		$mArea = &$page->appendNode("div", "class=MessageSection");
		$f1 = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' action='' method='get'" );
		
		//$f1->appendNode("div", "style='color: red;'", "Experimental System only for ideas");

		$m = &$f1->appendNode("div");
		$s = &$m->appendNode("span");
		$s = &$s->appendNode("select", "class='MessageGroupSelect' name='group' onchange='this.form.submit()' title='Select messaging group to view. All shows messages from all groups'");

		$groups = array_merge(array("All"), $this->sys->groups);
		foreach($groups as $item){
			if($item == $group)
				$s->appendNode("option", "value='{$item}' selected='selected'", "$item");
			else
				$s->appendNode("option", "value='{$item}'", "$item");
		}

		if($this->sys->user)
			$username = $this->sys->user->name;
		else
			$username = "<a href='/user?cmd=login&url=/news'>Login to post</a>";

		$m->appendNode("span", "class=MessageName", $username);
		$s = &$m->appendNode("span", "class=MessageName style='float: right;'");
		if($this->sys->notifications && $this->checkAccess()){
			if($this->sys->isMobile)
				$s->appendNode("script", "", "wsNotifications.enableButtonCreate(0, 'NE', {$this->sys->user->newsNotifications}, '{$group}');");
			else
				$s->appendNode("script", "", "wsNotifications.enableButtonCreate(0, 'Enable Notifications', {$this->sys->user->newsNotifications}, '{$group}');");
		}

		$s->appendNode("span", "class=MessageName")->appendNode("a", "href='/news?cmd=help' title='Help on using this messaging system'", "Help");
		$s->appendNode("span", "class=MessageName")->appendNode("button", "class='ButtonImageMenu' title='Preferences menu' onclick='location.href = \"/news?cmd=userSettings\"; return false;'", "");
		
		$m = &$page->appendNode("div", "id='messages' class='Messages'");

		$this->getMessagesContent($m, $where);

		$mArea = &$page->appendNode("div", "class=MessageSection");
		
		if($this->sys->user){
			$f2 = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'" );
			$f2->appendNode("input", "type='hidden' name='type' value='{$type}'");
			$f2->appendNode("input", "type='hidden' name='objectId' value='{$objectId}'");
			$f2->appendNode("input", "type='hidden' name='newPostGroup' value='{$group}'");

			// Add save validation
			$postValidate = $this->sys->postValidateSave(0);
			$f2->appendNode("input", "type='hidden' name='postValidate' value='{$postValidate}'");

			$groups = $this->sys->groups;
			if(isset($_GET["newPostGroup"]))
				$postGroup = $_GET["newPostGroup"];
			else if(isset($_GET["group"]) && ($_GET["group"] != "All"))
				$postGroup = $_GET["group"];
			else {
				//$postGroup = "TandemClub";
				$postGroup = $this->sys->user->group;
			}

			$e = &$f2->appendNode("div", "class=MessageComment");
			$s = &$e->appendNode("div", "class=MessageCommentButtons title='Add camera or file picture'");
			$w = $s->appendNode("select", "class='MessageGroupSelect' name='newPostGroup' title='Select group to post message to'");
			foreach($groups as $item){
				if($item == $postGroup)
					$w->appendNode("option", "value='{$item}' selected='selected'", "$item");
				else
					$w->appendNode("option", "value='{$item}'", "$item");
			}
			$d = &$s->appendNode("div", "class='MessageButtons'");
			$d->appendNode("script", "", "new FormInputPicture('pictures', 2);");
			//$d->appendNode("button", "class='ButtonImageSmiley' ttype=submit name='smily' value='' title='Add smily'");
			$d->appendNode("button", "class='ButtonImageForm' type=submit name='advanced' value='' title='Advanced form entry'");
			
			
			$e = &$f2->appendNode("div", "class=MessageComment");
			$s = &$e->appendNode("div", "class=MessageCommentEntry title='Add text to post'");
			$s->appendNode("input", "type='text' id='newPostComment' name='newPostComment' placeholder='Enter text to post' value='' autocomplete='off' onkeypress='if(event.which == 10 || event.which == 13){ event.preventDefault(); this.form.submit();}'");
			$e->appendNode("button", "class='ButtonImageSend' type=submit name='newPost' value='' title='Send the post'");
		}
		else {
			$mArea->appendNode("div", "", "<a href='/user?cmd=login&url={$uri}'>Login to post</a>");
		}

		// Update page on changes, really need a notification system rather than polling
		$page->appendNode("script", "", "window.addEventListener('load', (event) => { var m = document.getElementById('messages'); m.scrollTop = m.scrollHeight;});");
		$page->appendNode("script", "", "itemsFetchDataAndSetLoop('/news?type=news&group={$group}', 10000);");

		return $page->render();
	}

	function updatePicture($uri){
		if(count($_POST)){
			//print "Posted";
			if((count($_FILES) == 1) && isset($_FILES["picture"])){
				//debug_log("uploadPictures: Do file: {$file["tmp_name"]}\n");
				$this->sys->modules["user"]->addPicture($this->sys->user->id, $_FILES["picture"]);
			}
			//header("Cache-Control: no-store, no-cache, must-revalidate");
			//header('Cache-Control: post-check=0, pre-check=0', false);
			//header('Pragma: no-cache');
			//header("Expires: 0");
			//header("Location: ?cmd=viewNews");
			header("Pragma: no-cache");
			header("Location: ?cmd=updatePicture");
			return null;
		}
		else {
			// Note 2 mechanisms used for reloading the image ignoring browser cache:
			// 1. Read URK with "Pragma: no-cahe" for Android Chrome.
			// 2. Call location.reload(true); ones for Firefox.

			//$pic = "/files/user/picture-{$this->sys->user->id}-thumb.jpg?" . time();
			$pic = "/files/user/picture-{$this->sys->user->id}-thumb.jpg";
			$page = new BHtmlNode("div", "class='MessagePage'");
			$page->appendNode("script", "", "reloadUrl('$pic');");

			$mArea = &$page->appendNode("div", "class=MessageSection");
			$f = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'" );
			$t = &$f->appendNode("table");
			$r = &$t->appendNode("tr");
			$r->appendNode("td")->appendNode("div", "class=MessageImageUser")->appendNode("img", "src='$pic'");
			$r->appendNode("td", "", "<b>User picture</b>");
			$r->appendNode("td")->appendNode("script", "", "new FormInputPicture('picture', 1, 160, 120);");
			$f->appendNode("input", "class='ButtonImageSend' type=submit name='send' value='Send' onclick='sessionStorage.setItem(\"reload\", \"true\");'");
			$f->appendNode("button", "class='ButtonImageReturn' onclick='location.href = \"?cmd=viewNews\"; return false;'", "Return");

			$page->appendNode("script", "", "var reload = sessionStorage.getItem(\"reload\"); if(reload){ sessionStorage.removeItem(\"reload\"); location.reload(true);}");

			header("Pragma: no-cache");
			
			return $page->render();
		}
	}
	
	function userSettings($uri){
		if(count($_POST)){
			//print "Posted";
			//debug_log(print_r($_POST, 1));
			//debug_log(print_r($_FILES, 1));
			
			if($this->sys->notifications && $this->checkAccess()){
				if(isset($_POST["notifications"]))
					$this->sys->user->newsNotifications = 1;
				else
					$this->sys->user->newsNotifications = 0;

				$this->sys->db->putObject("User", $this->sys->user);
			}
			
			if((count($_FILES) == 1) && isset($_FILES["picture"]) && ($_FILES["picture"]["name"] != "")){
				//debug_log("uploadPictures: Do file: {$_FILES['picture']['tmp_name']}\n");
				$this->sys->modules["user"]->addPicture($this->sys->user->id, $_FILES["picture"]);
			}
			
			//header("Cache-Control: no-store, no-cache, must-revalidate");
			//header('Cache-Control: post-check=0, pre-check=0', false);
			//header('Pragma: no-cache');
			//header("Expires: 0");
			//header("Location: ?cmd=viewNews");
			header("Pragma: no-cache");
			header("Location: ?cmd=userSettings");
			return null;
		}
		else {
			$page = new BHtmlNode("div", "class='MessagePage'");

			$mArea = &$page->appendNode("div", "class=MessageSection");
			$f = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'" );
			$f->appendNode("h2", "", "Messaging user Preferences");
			$t = &$f->appendNode("table");

			// Notifications
			if($this->sys->notifications){
				$r = $t->appendNode("tr");
				$r->appendNode("th", "style='text-align: left; padding-right: 2em;'", "Enable News Notifications");
				$c = $r->appendNode("td");
				$c->appendNode("script", "", "wsNotifications.enableCheckboxCreate(0, {$this->sys->user->newsNotifications});");
			}
			
			// User picture
			// Note 2 mechanisms used for reloading the image ignoring browser cache:
			// 1. Read URK with "Pragma: no-cahe" for Android Chrome.
			// 2. Call location.reload(true); ones for Firefox.
			//$pic = "/files/user/picture-{$this->sys->user->id}-thumb.jpg?" . time();
			$pic = "/files/user/picture-{$this->sys->user->id}-thumb.jpg";
			$page->appendNode("script", "", "reloadUrl('$pic');");

			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("th", "style='text-align: left;'");
			$c->appendNode("span", "", "User picture");
			$c->appendNode("span", "class=ImageUser")->appendNode("img", "src='$pic' style='vertical-align:middle; padding-left: 1em;'");
			//$r->appendNode("td")->appendNode("div", "class=MessageImageUser")->appendNode("img", "src='$pic'");
			$r->appendNode("td")->appendNode("script", "", "new FormInputPicture('picture', 1, 160, 120);");

			$f->appendNode("p");
			$f->appendNode("input", "class='Button' type=submit name='send' value='Save' onclick='sessionStorage.setItem(\"reload\", \"true\");'");
			$f->appendNode("button", "class='ButtonImageReturn' onclick='location.href = \"?cmd=viewNews\"; return false;'", "Return");

			$page->appendNode("script", "", "var reload = sessionStorage.getItem(\"reload\"); if(reload){ sessionStorage.removeItem(\"reload\"); location.reload(true);}");
			header("Pragma: no-cache");
			
			return $page->render();
		}
	}
	
	function help($uri){
		header("Location: /files/help/news.html");
		return Null;
	}

	function copyNews(){
		$list = $this->sys->db->getObjectList("News", "", "", "id", "");
		
		foreach($list as $newsObj){
			$obj = new BObjItem();
			
			$obj->type = "news";
			$obj->category = $newsObj->category;
			$obj->group = $newsObj->group;
			$obj->title = $newsObj->title;
			$obj->description = $newsObj->description;
			if($newsObj->pictureId)
				$obj->pictureIds = strval($newsObj->pictureId);
			$obj->dateAdded = $newsObj->dateAdded;
			$obj->date = $newsObj->dateAdded;
			$obj->dateChildren = $newsObj->dateAdded;
			$this->sys->db->appendObject("Items", $obj);
		}
		return "All news objects coped to items";
	}
	
	function updateNum(){		
		if(($data = file_get_contents("{$this->sys->baseDir}/files/items/updateNum")) == False){
			return 0;
		}
		
		return intval($data);
	}

	function updateNumIncrement(){
		$updateNum = $this->updateNum();
		$updateNum++;
		file_put_contents("{$this->sys->baseDir}/files/items/updateNum", $updateNum);
	}
/*
	function itemVersion(){		
		if(($data = file_get_contents("{$this->sys->baseDir}/files/items/settings")) == False){
			return 0;
		}

		if($settings = json_decode($data))
			return $settings.version;
		else
			return 0;
	}

	function itemVersionIncrement(){
		$settings = array("version" => 0);
		if($data = file_get_contents("{$this->sys->baseDir}/files/items/settings")){
			if($v = json_decode($data))
				$settings = $v;
		}

		$settings->version += 1;
		file_put_contents("{$this->sys->baseDir}/files/items/settings", json_encode($settings));
	}
 */

	function itemList($num=3, $type="news", $group="" ){
		$title = "News";
		$where = "`type` = 'news' and `hide` = '0'";

		if(($group == "") && isset($_GET['group']))
			$group = filterString($_GET['group']);
			
		if($group != ""){
			if($group[0] == "!"){
				$group = substr($group, 1);
				$where .= "and `group`!='$group'";
				$group = "";
			}
			else {
				$where .= "and `group`='$group'";
			}
			$title = $group . " " . $title;
		}
		
		$html = new BHtmlNode("div", "class='NewsBox'");
		$h = &$html->appendNode("div", "class='NewsHeadingBox'");
		$h->appendNode("div", "class='NewsHeadingEntry'")->appendNode("a", "href='{$this->sys->baseUrl}/news?group={$group}'", $title);

		if(!$this->sys->checkAccessView(AccessMode::Anyone, $group)){
			$html->appendText("Sorry this group is private");
			return $html->render();
		}

		$objList = $this->sys->db->getObjectList("Items", "", $where, "!date", "{$num}");

		foreach($objList as $obj){
			$description = "";
			if($obj->pictureIds != ""){
				foreach(explode(",", $obj->pictureIds) as $pid){
					$description .= pictureHtml(intval($pid), "", 0, 1, 0, "align='left' style='padding-right:5px; max-width: 25%;'");
				}
			}

			$description .= htmlClean($obj->description);
			//$description = substr($description, 0, 400) . " ... <a href='/news?cmd=view&id={$obj->id}'>more</a>";
			$description .= "<script>textOverflowAddMore(\"<a href='/news?cmd=view&id={$obj->id}'>more ...</a>\");</script>";

			$d1 = $html->appendNode("div", "class='NewsItem'");
			$t = $d1->appendNode("a", "href='{$this->sys->baseUrl}/news?cmd=view&id={$obj->id}'")->appendNode("div", "class='NewsSummary'");
			$t->appendNode("span", "class='NewsSummaryPostdate'", dateString($obj->date));
			$t->appendNode("span", "class='NewsSummaryAuthor'", $obj->userName);
			$t->appendNode("span", "class='NewsSummaryLink'", $obj->title);

			if($this->checkAccessAdminGroup($obj->group)){
				$d = &$t->appendNode("span");
				$d->appendText("(");
				$d->appendNode("a", "href='/news?cmd=edit&id={$obj->id}' title='Edit news item'", "<b>E</b>");
				$d->appendNode("a", "href='/news?cmd=delete&id={$obj->id}' title='Delete news item'", "<b>, D</b>");
				$d->appendText(")");
			}

			// $d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
			// $d1->appendNode("div", "class='NewsSummaryAuthor'", $obj->userName);
			$d1->appendNode("div", "class='NewsItemContent'", $description);
		}

		return $html->render();
	}

	function itemListAdd($height, $type, $objectId, $group){
		$uri = $_SERVER['REQUEST_URI'];

		//print_r($_POST);
		
		if(($group == "") && isset($_GET['group']))
			$group = filterString($_GET['group']);

		if(isset($_POST["advanced"])){
			header("Location: /news?cmd=edit&type={$type}&objectId={$objectId}&group={$group}");
			//return $this->edit($uri, filterString($_POST["newPostGroup"]));
			return null;
		}
		
		if((isset($_POST["newPostComment"]) && ($_POST["newPostComment"] != "")) || (isset($_POST["comment"]) && ($_POST["comment"] != ""))){
			if(!$this->sys->login->checkLogin()){
				return "You need to <a href='/user?cmd=login&url=/news'>Login</a> to post";
			}
			
			$r = $this->viewSave($uri);

			header("Location: {$uri}");
			die();
		}


		$where = "`type` = '{$type}' and `hide` = '0'";
		if($objectId)
			$where .= " and `objectId` = '{$objectId}'";
		
		if($group != "")
			$where .= " and `group` = '{$group}'";

		$html = new BHtmlNode("div", "class='MessagePage' id='messagePage' style='max-width: 100%;'");
		$html->appendNode("a", "name='News'");
		$mArea = &$html->appendNode("div", "class=MessageSection");
		$f1 = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' action='' method='get'" );
		
		$m = &$f1->appendNode("div");
		//$s = &$m->appendNode("span")->appendNode("a", "href='{$this->sys->baseUrl}/news?group={$group}'", $group);
		$m->appendNode("span")->appendNode("b", "", "News:");
		$m->appendNode("span", "class='NewsHeadingEntry' title='Messaging group'")->appendNode("a", "href='{$this->sys->baseUrl}/news?group={$group}'", $group);

		if($this->sys->user)
			$username = $this->sys->user->name;
		else
			$username = "<a href='/user?cmd=login&uri={$uri}'>Login to post</a>";

		$m->appendNode("span", "class=MessageName", $username);
		$s = &$m->appendNode("span", "class=MessageName style='float: right;'");
		if($this->sys->notifications && $this->checkAccess()){
			if($this->sys->isMobile)
				$s->appendNode("script", "", "wsNotifications.enableButtonCreate(0, 'NE', {$this->sys->user->newsNotifications}, '{$group}');");
			else
				$s->appendNode("script", "", "wsNotifications.enableButtonCreate(0, 'Enable Notifications', {$this->sys->user->newsNotifications}, '{$group}');");
		}
		$s->appendNode("span", "class=MessageName")->appendNode("a", "href='/news?cmd=help' title='Help on using this messaging system'", "Help");
		$s->appendNode("span", "class=MessageName")->appendNode("button", "class='ButtonImageMenu' title='Preferences menu' onclick='location.href = \"/news?cmd=userSettings\"; return false;'", "");

		
		$m = &$html->appendNode("div", "id='messages' class='Messages' style='max-height: {$height}px;'");

		if(!$this->sys->checkAccessView(AccessMode::Anyone, $group)){
			$m->appendText("Sorry this group is private");
			return $html->render();
		}

		$this->getMessagesContent($m, $where, 20);

		$mArea = &$html->appendNode("div", "class=MessageSection");
		if($this->sys->user){
			$f2 = &$mArea->appendNode("form", "id='Message_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'" );
			$f2->appendNode("input", "type='hidden' name='type' value='{$type}'");
			$f2->appendNode("input", "type='hidden' name='objectId' value='{$objectId}'");
			$f2->appendNode("input", "type='hidden' name='newPostGroup' value='{$group}'");

			// Add save validation
			$postValidate = $this->sys->postValidateSave(0);
			$f2->appendNode("input", "type='hidden' name='postValidate' value='{$postValidate}'");

			$e = &$f2->appendNode("div", "class=MessageComment");
			$s = &$e->appendNode("div", "class=MessageCommentEntry title='Add text to post'");
			$s->appendNode("input", "type='text' id='newPostComment' name='newPostComment' placeholder='Enter text to post' value='' autocomplete='off' onkeypress='if(event.which == 10 || event.which == 13){ event.preventDefault(); this.form.submit();}'");

			$s = &$e->appendNode("div", "class=MessageCommentButtons title='Add camera or file picture'");
			$s->appendNode("script", "", "new FormInputPicture('pictures', 2);");

			//$s->appendNode("button", "class='ButtonImageSmiley' ttype=submit name='smily' value='' title='Add smily'");
			$s->appendNode("button", "class='ButtonImageSend' type=submit name='newPost' value='' title='Send the post'");
			$s->appendNode("button", "class='ButtonImageForm' type=submit name='advanced' value='' title='Advanced form entry'");
		}
		else {
			$mArea->appendNode("div", "", "<a href='/user?cmd=login&url={$uri}'>Login to post</a>");
		}

		// Update page on changes, really need a notification system rather than polling
		$html->appendNode("script", "", "window.addEventListener('load', (event) => { var m = document.getElementById('messages'); m.scrollTop = m.scrollHeight;});");
		$html->appendNode("script", "", "itemsFetchDataAndSetLoop('/news?type={$type}&objectId={$objectId}&group={$group}', 10000);");

		return $html->render();
	}

	function newsListGroups($num=3, $notGroup=""){
		$num = intval($num);
		$notGroup = filterString($notGroup);

		$where = "where `type`='news' and `hide` = '0' and (`dateExpiry` is NULL or `dateExpiry` = '' or `dateExpiry` > CURRENT_DATE())";

		if($notGroup != ""){
			if(substr($notGroup, 0, 1) == "!"){
				$notGroup = substr($notGroup, 1);
			}
			$where .= " and `group`!='$notGroup'";
		}	

		$html = new BHtmlNode("div", "class='NewsBox'");
		$h = &$html->appendNode("div", "class='NewsHeadingBox'");
		$h->appendNode("div", "class='NewsHeadingEntry'")->appendNode("a", "href='{$this->sys->baseUrl}/news'", "Group News");

		$limit = $num;
		$l = $this->sys->db->queryObjs("Items", "select `id`,`group` from {$this->sys->databasePrefix}Items {$where} order by `date` desc limit {$limit}");
		if($num > count($l))
			$num = count($l);
		
		// Try and make sure most groups have at least one news item
		
		// Randomise and pic the top
		//shuffle($l);
		//$l = array_slice($l, 0, $num);

		foreach($l as $item){
			$obj = $this->sys->db->getObject("Items", $item->id);
			if(strlen($obj->description) >= 300){
				$description = substr(htmlToText($obj->description), 0, 300);
				$description .= " <a href='{$this->sys->baseUrl}/news?cmd=view&id={$obj->id}'>more ...</a>";
			}
			else {
				$description = htmlToText($obj->description);
			}

			if($obj->pictureIds != ""){
				$pid = explode(",", $obj->pictureIds)[0];
				//$description = pictureHtml(intval($pid), "", 0, 0, 0, "class='MessagePicture'") . $obj->description;
				$description = pictureHtml(intval($pid), "", 0, 0, 0, "class='MessagePicture'") . $description;
			}
			$description .= "<script>textOverflowAddMore('<a href=\'/news?cmd=view&id={$obj->id}\'>more ...</a>');</script>";

			$d1 = $html->appendNode("div", "class='NewsItem'");
			$t = $d1->appendNode("a", "href='{$this->sys->baseUrl}/news?cmd=view&id={$obj->id}'")->appendNode("div", "class='NewsSummary'");
			$t->appendNode("div", "class='NewsSummaryCategory'")->appendNode("a", "href='{$this->sys->baseUrl}/groups/{$obj->group}/index.html'", $obj->group);
			$t->appendNode("span", "class='NewsSummaryPostdate'", dateString($obj->date));
			//$t->appendNode("span", "class='NewsSummaryAuthor'", $obj->userName);
			$t->appendNode("span", "class='NewsSummaryLink'", $obj->title);

			if($this->checkAccessAdminGroup($obj->group)){
				$d = &$t->appendNode("span");
				$d->appendText("(");
				$d->appendNode("a", "href='/news?cmd=edit&id={$obj->id}' title='Edit news item'", "<b>E</b>");
				$d->appendNode("a", "href='/news?cmd=delete&id={$obj->id}' title='Delete news item'", "<b>, D</b>");
				$d->appendText(")");
			}
			$d1->appendNode("div", "class='NewsItemContent'", $description);
		}

		return $html->render();
	}

	function sendNotifications($obj){
		if($obj->type == "forSale"){
			// Send an email to the ForSale person
			if($module = $this->sys->module("forSale")){
				$module->notify($obj);
			}
		}
		else if(($obj->type == "rides") || ($obj->type == "places")){
			// Send an email to the rides administrator
			if($module = $this->sys->module("rides")){
				$module->notify($obj);
			}
		}
		else if($obj->type == "news"){
			// Only when group set
			if(($obj->group == "") || !($groupObj = groupGetObj($obj->group)))
				return;

			if($this->sys->notifications){
				if($obj->title != ""){
					$title = "News: " . $obj->title;
				}
				else {
					$title = "News";
				}

				if($notificationModule = $this->sys->module("notification")){
					$description = htmlToText($obj->description);
					if(strlen($description) >= 100){
						$description = substr($description, 0, 100) . " ... more";
					}

					$icon = "{$this->sys->baseUrl}/icons/users/cycleHelmit.jpg";
					$url = "{$this->sys->baseUrl}/news?group={$obj->group}";

					$notificationModule->send($obj->group, $title, $url, $description, $icon);
				}
			}
			$title = "News: ";
			if($obj->group != ""){
				$title .= $obj->group . ": ";
			}

			$title .= $obj->title;

			// Send an email to everyone interested in receiving news emails
			$subject = "{$this->sys->siteTitle}: {$title}";
			$message = "<p>{$obj->description}</p>\n\n";
			$message .= "<p>To the News to view and/or reply: <a href='{$this->sys->baseUrl}/news?group={$obj->group}'>{$this->sys->baseUrl}/news?group={$obj->group}</a></p>\n";
			$message .= "\n";
			$message .= "<p>You can disable receiving these emails in your website accounts User Preferences</p>\n";

			if($this->test){
				$emailAddressList = array();
				$emailAddressList[] = $this->sys->user->emailAddress;
			}
			else {
				$where = "`newsEmails`='1'";
				$where .= " and `id`!= '{$this->sys->user->id}'";
				if($obj->group != ""){
					$where .= " and (`group`='{$obj->group}' or find_in_set('{$obj->group}', `groups`))";
				}
				$objList = $this->sys->db->getObjectList("User", "emailAddress", $where, "");

				$emailAddressList = array();
				foreach($objList as $o){
					$emailAddressList[] = $o->emailAddress;
				}
			}

			//debug_log("SendEmails: " . print_r($emailAddressList, 1) . "<br>");

			$from = preg_replace("/^.*@/", "news@", $this->sys->siteEmailAddress);
			$noreply = preg_replace("/^.*@/", "noreply@", $this->sys->siteEmailAddress);
			emailListFromReply($from, $noreply, $emailAddressList, $subject, $message, 1);
		}
	}
}


function newsList($num=3, $group=""){
	global	$sys;
	
	if($news = $sys->module("news"))
		return $news->itemList($num, "news", $group);
	else
		return "No news module.";
}

function newsListAdd($height="100px", $type="news", $objectId=0, $group=""){
	global	$sys;
	
	if($news = $sys->module("news"))
		return $news->itemListAdd($height, $type, $objectId, $group);
	else
		return "No news module.";
}

function newsListGroups($num=3, $notGroup=""){
	global	$sys;
	
	if($news = $sys->module("news"))
		return $news->newsListGroups($num, $notGroup);
	else
		return "No news module.";
}

// Compatibility functions
function itemListAdd($height="100px", $type="news", $objectId=0, $group=""){
	global	$sys;
	
	if($news = $sys->module("news"))
		return $news->itemListAdd($height, $type, $objectId, $group);
	else
		return "No news module.";
}

function itemList($num=3, $type="news", $group="" ){
	global	$sys;
	
	if($news = $sys->module("news"))
		return $news->itemList($num, $type, $group);
	else
		return "No news module.";
}
?>
