<?php
/*******************************************************************************
 *	User.php	User Module
 *	Copyright	Terry Barnaby,	BeamWeb,	2021-01-05
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";
require_once "BLogin.php";

function makeRandomPassword() {
	$pass = "";
	$salt = "abchefghjkmnpqrstuvwxyz0123456789";
	srand((double)microtime()*1000000);
	$i = 0;
	while($i <= 7){
		$num = rand() % 33;
		$tmp = substr($salt, $num, 1);
		$pass = $pass . $tmp;
		$i++;
	}
	return $pass;
}

function passwordGenerate($syllables = 3, $use_prefix = false){
    // Define function unless it is already exists
    if (!function_exists('ae_arr'))
    {
        // This function returns random array element
        function ae_arr(&$arr)
        {
            return $arr[rand(0, sizeof($arr)-1)];
        }
    }

    // 20 prefixes
    $prefix = array('aero', 'anti', 'auto', 'bi', 'bio',
                    'cine', 'deca', 'demo', 'dyna', 'eco',
                    'ergo', 'geo', 'gyno', 'hypo', 'kilo',
                    'mega', 'tera', 'mini', 'nano', 'duo');

    // 10 random suffixes
    $suffix = array('dom', 'ity', 'ment', 'sion', 'ness',
                    'ence', 'er', 'ist', 'tion', 'or'); 

    // 8 vowel sounds 
    $vowels = array('a', 'o', 'e', 'i', 'y', 'u', 'ou', 'oo'); 

    // 20 random consonants 
    $consonants = array('w', 'r', 't', 'p', 's', 'd', 'f', 'g', 'h', 'j', 
                        'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm', 'qu');

    $password = $use_prefix?ae_arr($prefix):'';
    $password_suffix = ae_arr($suffix);

    for($i=0; $i<$syllables; $i++)
    {
        // selecting random consonant
        $doubles = array('n', 'm', 't', 's');
        $c = ae_arr($consonants);
        if (in_array($c, $doubles)&&($i!=0)) { // maybe double it
            if (rand(0, 2) == 1) // 33% probability
                $c .= $c;
        }
        $password .= $c;
        //

        // selecting random vowel
        $password .= ae_arr($vowels);

        if ($i == $syllables - 1) // if suffix begin with vovel
            if (in_array($password_suffix[0], $vowels)) // add one more consonant 
                $password .= ae_arr($consonants);

    }

    // selecting random suffix
    $password .= $password_suffix;

    return $password;
}


global	$sys;
$sys->db->addObjectClasses(array("User" => "BObjUser"));

class BObjUser extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("enabled", "Bool"),
			new BMember("fixed", "Bool"),
			new BMember("name", "String"),
			new BMember("userName", "String"),
			new BMember("userPassword", "String"),
			new BMember("postalAddress", "String"),
			new BMember("city", "String"),
			new BMember("postcode", "String"),
			new BMember("country", "String"),
			new BMember("emailAddress", "String", "emailOrNone"),
			new BMember("telephoneHome", "String"),
			new BMember("telephoneWork", "String"),
			new BMember("telephoneMobile", "String"),
			new BMember("roles", "String"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("emailSend", "Bool"),
			new BMember("forumEmailSend", "Bool"),
			new BMember("forsaleEmailSend", "Bool"),
			new BMember("newsEmails", "Bool"),
			new BMember("newsNotifications", "Bool"),
			new BMember("registered", "Bool"),
			new BMember("pending", "Bool"),
			new BMember("registerPassword", "String"),
			new BMember("group", "String"),
			new BMember("groups", "String"),
			new BMember("membershipId", "Int"),
			new BMember("modifiedDate", "Date"),
			new BMember("adminValidate", "Bool"),
			new BMember("gdprAcceptance", "Bool"),
			new BMember("picturePresent", "Bool"),
			new BMember("lastAccessTime", "DateTime"),
			new BMember("rejectText", "String"),
			));
	}

	function dbTableSetup($table){
		global	$sys;

		$obj = new BObjUser();
		$obj->enabled = 1;
		$obj->name = "root";
		$obj->userName = "root";
		$obj->userPassword = password_hash("beam00", PASSWORD_DEFAULT);
		$obj->roles = "admin,adminUsers,adminPages,adminDelete";
		$obj->dateAdded = strftime("%Y-%m-%d");
		$obj->emailSend = 0;
		$obj->forumEmailSend = 0;
		$obj->forsaleEmailSend = 0;
		$sys->db->appendObject($table, $obj);

		return "";
	}

};

class BLoginForm {
	function __construct($uri){
		$this->type = "BLoginForm";
		$this->title = "Account";
		$this->uri = $uri;
	}
	function rebaseMenu(){
	}
	function render(){
		global $sys;

		$page = new BHtmlNode("div", "class='loginLeft'");
		$f = &$page->appendNode("form", "method='post' action='{$sys->baseUrl}/user?cmd=loginFromForm'");
		$f->appendNode("input", "type='hidden' name='uri' value='{$this->uri}'");

		if($sys->siteName == "TandemClub")
			$f->appendNode("a", "href='/files/members.html'", "Members website access info");

		$t = &$f->appendNode("table");
		//$r = &$t->appendNode("tr"); $c = &$r->appendNode("td"); $c->appendText("Username");
		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td");
		$c->appendNode("input", "class='loginEntry' type='text' name='username' placeholder='Username'");

		//$r = &$t->appendNode("tr"); $c = &$r->appendNode("td");	$c->appendText("Password");
		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td");
		$c->appendNode("input", "class='loginEntry' type='password' name='password' placeholder='Password'");

		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td");
		$c->appendNode("input", "class='loginButton' type='submit' value='Login Now'");

		if(!$sys->loginRequired and $sys->loginRegister){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td");
			$c->appendNode("a", "href='{$sys->baseUrl}/user?cmd=register'", "Create New Account");
		}
		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td");
		$c->appendNode("a", "href='{$sys->baseUrl}/user?cmd=forgottenPassword'", "Forgotten password");
 
		return $page->render();
//		return "Test";
	}
}

class BLogoutForm {
	function __construct(&$sys){
		$this->sys = $sys;
		$this->type = "BLogoutForm";
		$this->title = "Account";
	}
	function rebaseMenu(){
	}
	function render(){
		global $sys;
		
		$page = new BHtmlNode("div", "class='loginLeft'");
		if($this->sys->user && !$this->sys->user->fixed){
			$page->appendNode("div")->appendNode("a", "href='{$sys->baseUrl}/user?cmd=userPreferences'", "User Preferences");
			$page->appendNode("div")->appendNode("a", "href='{$sys->baseUrl}/user?cmd=deleteAccount'", "Delete My Account");
		}
		$page->appendNode("div")->appendNode("a", "href='{$sys->baseUrl}/user?cmd=logout'", "Logout");
 
		return $page->render();
	}
}

function pageLogin($message = ""){
	global $sys;

	$uri = $_SERVER['REQUEST_URI'];

	$content = new BHtmlNode("div", "class='login'");
	if($message != ""){
		$content->appendNode("p", "", $message);
	}
	
	$content->appendNode("div", "class='error'", "This site area is only accessable to authorised users. Please login to access.<br>");
	$content->appendNode("h2", "", "Login to system");
	$f = &$content->appendNode("form", "method='post' action='{$sys->baseUrl}/user?cmd=loginFromForm'");
	$t = &$f->appendNode("table");
	$r = &$t->appendNode("tr");
	$c = &$r->appendNode("td");
	$c->appendText("Username");
	$c = &$r->appendNode("td");
	$c->appendNode("input", "type='text' name='username'");
	
	$r = &$t->appendNode("tr");
	$c = &$r->appendNode("td");
	$c->appendText("Password");
	$c = &$r->appendNode("td");
	$c->appendNode("input", "type='password' name='password'");
	
	$r = &$t->appendNode("tr");
	$c = &$r->appendNode("td");
	$c->appendNode("input", "type='hidden' name='uri' value='{$uri}'");
	$c->appendNode("input", "type='submit' value='Login'");
	
	$sys->pageDisplay("{$sys->siteTitle} - Login", $content->render());
}

function pageLogin1(){
	global $sys;
	
	$content = "<div class='error'>";
	$content .= "This site area is only accessable to authorised users.<br>";
	$content .= "</div>";
	$content .= "<H3>Please Login to the System</H3>";
	
	$sys->pageDisplay("{$sys->siteTitle} - Login", $content);
}


class User extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "user";
		$this->title = "User";
		$this->info = "User module. This module allows the management of the sites users.";
		$this->test = $this->sys->test;
		$this->testEmails = $this->sys->test;
		//$this->test = 1;
		//$this->testEmails = 1;
		
		// Create the directory			
		$dir = $this->sys->baseDir . "/files/user";
		if(!file_exists($dir)){
			mkdir($dir);
		}

		// Add HTML functions
		$this->sys->htmlFuncs[] = "userGetLoginBlock";
	}

	function setRoles($roles){
		$this->rolesAll = explode(",", $roles);
	}

	function appendRoles($roles){
		$this->rolesAll = array_merge($this->rolesAll, $roles);
	}
	
	function checkAccess(){
		if($this->sys->login->checkLogin("adminUsers")){
			return 1;
		}
		else {
			return 0;
		}		
	}
		
	function getMenu(){
		if($this->sys->login->checkLogin("adminUsers")){
			return new BMenu("User", "/{$this->name}", "", array("User List" => "/{$this->name}", "User Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);
		
		$page = "";
		
		//if($this->cmd == "fixPictures"){
		//	return $this->fixPictures();
		//}

		switch($this->cmd){
		case "login":
			if(isset($_GET["url"]))
				$html = new BLoginForm($_GET["url"]);
			else
				$html = new BLoginForm($uri);

			return $html->render();
			break;

		case "loginFromForm":
			if(!$this->sys->login->checkLogin()){
				if($this->sys->login->id && !$this->sys->login->enabled){
					$this->sys->pageDisplayError(new BError(1, "Account is disabled"));
				}
				else {
					$message = "<p>Incorrrect login information</p>";
					$message .= "<p> if you have forgotten your password please request a new one <a href='/user?cmd=forgottenPassword'>here.</a></p>";
					if($this->sys->loginRegister)
						$message .= "<p>If you don't have a login account you can request one <a href='/user?cmd=register'>here,</a>";
					$message .= "</p>";

					//sleep(3);
					$this->sys->pageDisplayError(new BError(1, $message));
				}
				return "";
			}

			// Clear style
			unset($_SESSION["style"]);

			if(isset($_POST['uri']) && (strpos($_POST['uri'], "cmd=logoutDone") === false)){
				$uri = filterString($_POST['uri']);
				header("Location: {$uri}");
				die();
			}
			else {
				header("Location: {$this->sys->baseUrl}");
				die();
			}
			break;
		case "logout":
			$this->sys->login->logout();
			header("Location: {$this->sys->baseUrl}/user?cmd=logoutDone");
			exit(0);
			break;
		case "logoutDone":
			return "<h1>You have been logged out</h1>\n";
			break;
		case "register":
			return $this->register();
			break;
		case "registerEmail":
			return $this->registerEmail();
			break;
		case "registerActivate":
			return $this->registerActivate();
			break;
		case "userPreferences":
			return $this->userPreferences();
			break;
		case "userPreferencesSave":
			return $this->userPreferencesSave();
			break;
		case "deleteAccount":
			return $this->deleteAccount();
			break;
		case "deleteAccountDo":
			return $this->deleteAccountDo();
			break;
		case "forgottenPassword":
			return $this->forgottenPassword();
			break;
		case "forgottenPasswordEmail":
			return $this->forgottenPasswordEmail();
			break;
		case "forgottenPasswordChange":
			return $this->forgottenPasswordChange();
			break;
		}
		
		if(!$this->sys->login->checkLogin("adminUsers")){
			$this->sys->pageDisplayError(new BError(1, "You do not have admin privilages needed to access this page"));
			return Null;
		}

		switch($this->cmd){
		case "view":			return $this->view(filterToInt($_GET["id"]));	break;
		case "add":			return $this->edit("0");		break;
		case "edit":			return $this->edit(filterToInt($_GET["id"]));	break;
		case "save":			return $this->save();			break;
		case "delete":			return $this->delete(filterToInt($_GET["id"]));	break;
		case "deleteDo":		return $this->deleteDo(filterToInt($_GET["id"]));	break;
		case "statistics":		return $this->statistics();		break;
		default: 			return $this->viewList();		break;
		}
		
		return $page;
	}

	function getPageBlock($uri){
		if($this->sys->login->checkLogin("adminUsers")){
			return new BMenu("Users", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("", "", ""),
				new BMenu("Add", "",	"onmouseover='this.href = \"?cmd=add\"'"),
				new BMenu("Statistics", "", "href='?cmd=statistics'"),
				));
		}
		else {
			return Null;
		}
	}

	function getLoginBlock($uri){
		if($this->sys->login->checkLogin()){
			return new BLogoutForm($this->sys);
		}
		else {
			return new BLoginForm(filterString($_SERVER["REQUEST_URI"]));
		}
	}
	

	function viewList(){
		$listSize = 30;

		if($this->test || $this->testEmails){
			$this->sys->status = "<h1><font color='red'>System is in test mode: {$this->test}</font></h1>\n";
		}

		$page = "<h2>Users</h2>\n";

		$sel = new BSelect("select", "Select", "", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("enabled", "", "", "equal", "tristate"),
			new BViewTypeSelect("pending", "", "", "equal", "tristate"),
			new BViewTypeSelect("emailSend", "", "", "equal", "tristate"),
			new BViewTypeSelect("forumEmailSend", "", "", "equal", "tristate"),
			new BViewTypeSelect("forsaleEmailSend", "", "", "equal", "tristate"),
			new BViewTypeSelect("name", "", "Name", "like"),
			new BViewTypeSelect("userName", "", "", "like"),
			new BViewTypeSelect("emailAddress", "", "Email", "like"),
			new BViewTypeSelect("postcode", "", "", "like"),
			new BViewTypeSelect("group", "", "Group", "like", "select", array_merge(array(""), $this->groupsList())),
			new BViewTypeSelect("roles", "", "Role", "inList", "select", array_merge(array("","ANY","admin%","view%"), $this->rolesAll)),
			new BViewTypeSelect("dateAdded", "", "", "period", "select", array("", "Last 2 Months", "Last Month", "To Now", "To Next Month", "Next Month")),
			new BViewTypeSelect("order", "", "", "order", "select", array("id", "!id", "name", "userName", "email", "postcode", "group", "roles", "!modifiedDate")),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			new BViewTypeSelect("membershipId", "", "Member", "isset", "tristate"),
			));

		$sel->getObj()->roles = "";
		$sel->getObj()->order = "id";
		$sel->pagerSet(0, $listSize, 0);	
		$sel->postUpdate();
		$where = $sel->getSelectString();
		$order = $sel->getOrder();

		//debug_log("SelString: {$where} Order: {$order}\n");

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("pending"),
			new BViewType("name"),
			new BViewType("enabled"),
			new BViewType("fixed"),
			new BViewType("emailSend", "Emails"),
			new BViewType("forumEmailSend", "Forum Emails"),
			new BViewType("forsaleEmailSend", "ForSale Emails"),
			new BViewType("emailAddress", "Email Address"),
			new BViewType("telephoneHome", "Home Telephone"),
			new BViewType("telephoneMobile", "Mobile Telephone"),
			new BViewType("membershipId"),
			new BViewType("group"),
			new BViewType("lastAccessTime"),
			new BViewType("roles", "Roles"),
			));
		
		$view->setSelection($where);
		$view->setOrder($order);
		$view->setLimit($sel->pagerGetLimitString());
		$view->setTable("User");
		$totalNumber = $this->sys->db->getObjectNumber("User", $where);
		$sel->pagerSet(0, $listSize, $totalNumber);
		
		$page .= $sel->getHtml();
		$page .= $view->getHtml();

		$page .= "<div class='help'>";
		$page .= "This lists all of the users of the system. ";
		$page .= "In the left hand bar there are links to the functions you can perform.";
		$page .= "</div>";

		return $page;
	}

	function view($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$obj = $this->sys->db->getObject("User", $id);
		}

		$page = "<h2>User View</h2>";

		$view = new BViewObj("user");
		$view->setView(array(
			new BViewType("name"),
			new BViewType("pending"),
			new BViewType("enabled"),
			new BViewType("fixed"),
			new BViewType("userName"),
			new BViewType("emailSend", "Send Emails"),
			new BViewType("forumEmailSend", "Send Forum Emails"),
			new BViewType("forsaleEmailSend", "Send Forsale Emails"),
			new BViewType("newsEmails"),
			new BViewType("newsNotifications"),
			new BViewType("emailAddress", "Email Address"),
			new BViewType("postalAddress", "Postal Address", "textarea", "", 200, 80),
			new BViewType("city", "Town/City"),
			new BViewType("postcode"),
			new BViewType("country"),
			new BViewType("telephoneHome"),
			new BViewType("telephoneWork"),
			new BViewType("telephoneMobile"),
			new BViewType("roles"),
			new BViewType("group"),
			new BViewType("groups"),
			new BViewType("membershipId"),
			new BViewType("dateAdded"),
			new BViewType("modifiedDate"),
			new BViewType("adminValidate"),
			new BViewType("gdprAcceptance"),
			new BViewType("lastAccessTime"),
			));
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function edit($arg){
		$page = "<h2>Edit User</h2>";

		if($arg == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else if(is_object($arg)){
			$obj = $arg;
			$obj->userPassword = "";
		}
		else if($arg == 0){
			$page .= "<p>A random password has been created.</p>";
			$obj = $this->sys->db->getNewObject("User");
			$obj->enabled = 1;
			$obj->emailSend = 1;
			$obj->forumEmailSend = 0;
			$obj->forsaleEmailSend = 0;
			$obj->userPassword = makeRandomPassword();
			$obj->dateAdded = strftime("%Y-%m-%d");
			$obj->sendRegisterEmail = 1;
		}
		else {
			$obj = $this->sys->db->getObject("User", $arg);
			$obj->userPassword = "";
		}

		$view = new BEditObj("user", "User Information", "?cmd=save", "");
		$view->setView(array(new BViewType("id", "", "hidden"),
			new BViewType("name", "Full name"),
			new BViewType("pending", "", "checkbox"),
			new BViewType("enabled", "", "checkbox"),
			new BViewType("sendRegisterEmail", "Send register email", "checkbox"),
			new BViewType("fixed", "", "checkbox"),
			new BViewType("userName", "", "username-new"),
			new BViewType("userPassword", "", "password-new"),
			new BViewType("emailSend", "Emails (Admin, groups)", "checkbox"),
			new BViewType("forumEmailSend", "Forum Emails", "checkbox"),
			new BViewType("forsaleEmailSend", "Forsale Emails", "checkbox"),
			new BViewType("newsEmails", "News Emails", "checkbox"),
			new BViewType("newsNotifications", "News Notifications", "checkbox"),
			new BViewType("emailAddress", "Email Address"),
			new BViewType("postalAddress", "Postal Address", "textarea", "", 300, 80),
			new BViewType("city", "Town/City"),
			new BViewType("postcode"),
			new BViewType("country", "", "select", $this->countryList()),
			new BViewType("telephoneHome"),
			new BViewType("telephoneWork"),
			new BViewType("telephoneMobile"),
			new BViewType("roles", "Roles", "selectMultiCheckbox", $this->rolesAll),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("groups", "Groups of Interest", "selectMultiCheckbox", $this->sys->groups),
			new BViewType("membershipId"),
			new BViewType("adminValidate", "", "hidden"),
			new BViewType("gdprAcceptance", "I have read the data privacy note above and I consent to it.", "checkbox"),
			new BViewType("rejectText", "Reject text", "textarea", "", 300, 80),
			new BViewType("lastAccessTime", "LastAccessTime", "view"),
			new BViewType("dateAdded", "", "view"),
			new BViewType("modifiedDate", "", "view"),
			));
		
		$view->addExtraButton("reject", "Reject");

		$view->setHelp("name", "Enter the users full name.");
		$view->setHelp("enabled", "Used to enable the user. Only enabled users can login.");
		$view->setHelp("fixed", "Used to disallow user perference configuration by the user. A fixed website login.");
		$view->setHelp("userName", "Enter the users login name. Their email address is best used here.");
		$view->setHelp("userPassword", "Enter the users password to override the default one.");
		$view->setHelp("emailSend", "Enable emails to be sent to the user for admin, news, events etc.");
		$view->setHelp("forumEmailSend", "Enable emails to be sent to the user on forum posts.");
		$view->setHelp("forsaleEmailSend", "Enable emails to be sent to the user on forsale posts.");
		$view->setHelp("emailAddress", "Enter the users email address.");
		$view->setHelp("postalAddress", "Enter the users postal address. Optional");
		$view->setHelp("city", "Enter the users local city/town. Optional");
		$view->setHelp("telephoneWork", "Enter the users work telephone number. Optional");
		$view->setHelp("telephoneHome", "Enter the users home telephome number. Optional");
		$view->setHelp("telephoneMobile", "Enter the users mobile number. Optional");
		$view->setHelp("roles", "Enter the system roles the user is allowed to perform.");
		$view->setHelp("group", "Enter the primary group this user belongs to.");
		$view->setHelp("groups", "Enter the other groups the user is interested in.");
		$view->setHelp("sendRegisterEmail", "Select to send the user an email with password and a link to register.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save(){
//		print "Save: "; print_r($_POST);
		$membershipInfoChanged = 0;

		$page = "";
		if(count($_POST)){
			unset($_POST['cmd']);
			if(isset($_POST["sendRegisterEmail"]) && ($_POST["sendRegisterEmail"] == 1))
				$register = 1;
			else
				$register = 0;

			$obj = new BObjUser();
			$obj->set($_POST);
			$obj->modifiedDate = strftime("%Y-%m-%d");

			// Validate the save
			if(!$this->sys->postValidateCheck($obj->id)){
				$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
				exit();
			}

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if(!$obj->gdprAcceptance){
				$this->sys->status = "<span class='editError'>Error you need to accept the data privacy agreement.</span>";
				return $this->edit($obj);
			}

			if(isset($_POST["reject"])){
				// Send rejected email to user
				$message = "Dear {$obj->name},\n\n";
				$message .= "Your website account has not been enabled at our website, {$this->sys->siteName}.\n";
				$message .= htmlToText($obj->rejectText) . "\n";
				$message .= "This is an automated response, please do not reply by email.\n";

				emailList($obj->emailAddress, "Account request at {$this->sys->siteName}", $message, 0);
				
				$page .= "Sent rejected email to {$obj->emailAddress}<br>";
			}

			// Check Username is not already used
			if($obj->id == 0){
				$userName = dbEscapeString($obj->userName);
				$objList = $this->sys->db->getObjectList("User", "", "userName = '{$userName}'", "");
				if(count($objList)){
					$this->sys->status = "<span class='editError'>That user id is already in use, please try another. Your email address is a suitable userName.</span>";
					return $this->edit($obj);
				}
//				$page .= "The password set is: {$obj->userPassword}.</br>";
			}
			else {
				$objOld = $this->sys->db->getObject("User", $obj->id);

				// Modify some settings
				if($obj->userPassword == "")
					unset($obj->userPassword);
				
				if(($obj->name != $objOld->name) ||
					($obj->postalAddress != $objOld->postalAddress) ||
					($obj->city != $objOld->city) ||
					($obj->postcode != $objOld->postcode) ||
					($obj->country != $objOld->country) ||
					($obj->emailAddress != $objOld->emailAddress) ||
					($obj->telephoneHome != $objOld->telephoneHome) ||
					($obj->telephoneWork != $objOld->telephoneWork) ||
					($obj->telephoneMobile != $objOld->telephoneMobile)){
					$membershipInfoChanged = 1;
				}
				
				if(!$objOld->enabled && $obj->enabled){
					// Send enabled email to user
					$message = "Dear {$obj->name},\n\n";
					$message .= "Your website account has now been enabled at our website, {$this->sys->siteName}.\n";
					$message .= "Your Username is: {$obj->userName}.\n";
					$message .= "This is an automated response, please do not reply by email.\n";

					emailList($obj->emailAddress, "Account request at {$this->sys->siteName}", $message, 0);
				}
			}

			$page .= $this->addUser($obj, $register);

			if(!isset($_POST["reject"]) && $this->test == 0){
				// Update membership record if needed			
				if($membershipInfoChanged && array_key_exists("membership", $this->sys->modules))
					$this->sys->modules["membership"]->userUpdate($obj, "Admin changed");
			}

			$page .= "User details saved.";
		}
		return $page;
	}

	function addUser(&$obj, $register = 0){
//		print "Save: "; print_r($_POST);

		if($err = $obj->validate()){
			$items = implode(",", array_keys($err));
			return "Error in user data: $items";
		}

		// Check Username is no already used
		if($obj->id == 0){
			$obj->dateAdded = strftime("%Y-%m-%d");
			$userName = dbEscapeString($obj->userName);
			$objList = $this->sys->db->getObjectList("User", "", "userName = '{$userName}'", "");
			if(count($objList)){
				return "That user id is already in use, please try another";
			}
		}

		// Modify some settings
		if(isset($obj->userPassword)){
			if($obj->userPassword != ""){
				$password = $obj->userPassword;
			}
			else {
				$password = makeRandomPassword();
			}
			$obj->userPassword = password_hash($password, PASSWORD_DEFAULT);
		}

		if($register){
			//$obj->enabled = 0;
			$obj->registered = 0;
			$obj->registerPassword = makeRandomPassword();
			$registerPassword = password_hash($obj->registerPassword, PASSWORD_DEFAULT);
		}
		
		if($obj->id)
			$this->sys->db->putObject("User", $obj);
		else
			$this->sys->db->appendObject("User", $obj);

		if($register){
			// Send register email to user
			$message = "Dear {$obj->name},\n\n";
			$message .= "You are now registered at our website, {$this->sys->siteName}.\n";
			$message .= "To activate your account, please click on the following link:\n";
			$message .= "  {$this->sys->baseUrl}/user?cmd=registerActivate&id={$obj->id}&registerPassword={$registerPassword}\n\n";
			$message .= "Once your account is activated you will be able to login. Your Username is: {$obj->userName} and your password is: {$password}.\n";
			$message .= "Please change the password once logged in using the \"User Preferences\" link in the menu's.\n";
			$message .= "This is an automated response, please do not reply by email.\n";

			emailList($obj->emailAddress, "Account request at {$this->sys->siteName}", $message);
		}
		
		return "";
	}
	
	function deleteUser($userName){
		$list = $this->sys->db->getObjectList("User", "", "`userName` = '{$userName}'");
		foreach($list as $user){
			$this->sys->db->deleteObject("User", $user->id);
		}
	}

	function delete($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("User", $id);

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteDo&id={$id}' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete user {$obj->name} ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page = $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function deleteDo($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("User", $id);

		// Now delete the item itself
		$this->sys->db->deleteObject("User", $id);

		$page = "";
		$page .= "<h3>User {$obj->name} has been Deleted</h3>";	
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function register($obj = Null){
		$this->sys->title = "{$this->sys->siteTitle} - Register";

		if($obj == Null)
			$obj = $this->sys->db->getNewObject("User");

		$obj->enabled = 0;
		$obj->registered = 0;
		$obj->roles = "";
		$obj->dateAdded = strftime("%Y-%m-%d");
		$obj->modifiedDate = strftime("%Y-%m-%d");
		$obj->emailSend = 1;
		$obj->forumEmailSend = 1;
		$obj->forsaleEmailSend = 1;
		$obj->adminValidate = $this->sys->loginRegisterValidate;

		$page = "<h2>Enter User Details</h2>";
		$page .= "<div class='help'>";
		$page .= "Please fill in the following information and click on the \"Save\" button when complete. ";
		$page .= "We recommend using your email address as a user name.</br>";
		$page .= "We require you to provide your genuine name and postal address or a valid phone number for access to this site.";
		$page .= " We will check new login account requests.</br>";
		if(!$this->sys->loginNoGroups)
			$page .= " Your group can be set to a particular group you are affiliated with.";
		$page .= " If you provide an address this will be used to ease form filling in on the website.";
		$page .= " Items marked with a \"*\" are required information. ";
		if($this->sys->smsSend)
			$page .= "If you enter a mobile telephone number, you will receive text message alerts from the website when sent. ";
		$page .= "</br>The 'Send Forum Emails' and 'Send Forsale Emails' options allow you to receive emails when new discussions are posted or for sale entries are added.";
		$page .= "</br>Data privacy: The information you provide on this form will be added to the database to be used for purposes of managing the website and sending you information from the club. Our data privacy pages describe this. Please indicate on the form below that you accept this.";
		$page .= "</div>";

		$view = new BEditObj("User", "User Information", "?cmd=registerEmail");
		if($this->sys->loginUsernameEmail){
			$emailType = "hidden";
		}
		else {
			$emailType = "";
		}
		if($this->sys->login->checkLogin()){
			$adminEditCheckbox = "checkbox";
		}
		else {
			$adminEditCheckbox = "hidden";
		}
		
		if($this->sys->loginNoGroups){
			$group1Type = "hidden";
			$group2Type = "hidden";
		}
		else {
			$group1Type = "select";
			$group2Type = "selectMultiCheckbox";
		}

		$view->setView(array(new BViewType("id", "", "hidden"),
			new BViewType("name", "Full name *", "username-new"),
			new BViewType("userName", "Username (use your email address) *", "username-new"),
			new BViewType("userPassword", "Password *", "password-new"),
			new BViewType("userPassword1", "Password again *", "password-new"),
			new BViewType("emailAddress", "Email Address *", $emailType),
			new BViewType("postalAddress", "Postal Address", "textarea", "", 300, 80),
			new BViewType("city", "Town/City"),
			new BViewType("postcode"),
			new BViewType("country", "", "select", $this->countryList()),
			new BViewType("telephoneHome", "Telephone Home *"),
			new BViewType("telephoneWork"),
			new BViewType("telephoneMobile"),
			new BViewType("emailSend", "Send Emails (Admin, groups)", $adminEditCheckbox),
			new BViewType("forumEmailSend", "Send Forum Emails", "checkbox"),
			new BViewType("forsaleEmailSend", "Send Forsale Emails", "checkbox"),
			new BViewType("group", "TC Group", $group1Type, array_merge(array(""), $this->sys->groups)),
			new BViewType("groups", "TC Groups (interested in)", $group2Type, $this->sys->groups),
			new BViewType("dateAdded", "", "hidden"),
			new BViewType("modifiedDate", "", "hidden"),
			new BViewType("adminValidate", "", "hidden"),
			new BViewType("gdprAcceptance", "I have read the data privacy note above and I consent to it.", "checkbox"),
			));

		$view->setHelp("name", "Enter the users full name.");
		$view->setHelp("enabled", "Used to enable the user. Only enabled users can login.");
		$view->setHelp("fixed", "Used to disallow user perference configuration by the user. A fixed website login.");
		$view->setHelp("userName", "Enter the users login name. Their email address is best used here.");
		$view->setHelp("userPassword", "Enter the users password to override the default one.");
		$view->setHelp("emailSend", "Enable emails to be sent to the user for admin, news, events etc.");
		$view->setHelp("forumEmailSend", "Enable emails to be sent to the user on forum posts.");
		$view->setHelp("forsaleEmailSend", "Enable emails to be sent to the user on forsale posts.");
		$view->setHelp("emailAddress", "Enter the users email address.");
		$view->setHelp("postalAddress", "Enter the users postal address. Optional");
		$view->setHelp("city", "Enter the users local city/town. Optional");
		$view->setHelp("telephoneWork", "Enter the users work telephone number. Optional");
		$view->setHelp("telephoneHome", "Enter the users home telephome number. Optional");
		$view->setHelp("telephoneMobile", "Enter the users mobile number. Optional");
		$view->setHelp("roles", "Enter the system roles the user is allowed to perform.");
		$view->setHelp("group", "Enter the primary group this user belongs to.");
		$view->setHelp("groups", "Enter the other groups the user is interested in.");
		$view->setHelp("sendRegisterEmail", "Select to send the user an email with password and a link to register.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function registerEmail(){
		$obj = new BObjUser();
		$obj->set($_POST);

		$obj->id = 0;
		$obj->enabled = 0;
		$obj->registered = 0;
		$obj->roles = "";
		$obj->dateAdded = strftime("%Y-%m-%d");

		// Validate the save
		if(!$this->sys->postValidateCheck($obj->id)){
			$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
			exit();
		}
		
		if($obj->emailAddress == ""){
			$obj->emailAddress = $obj->userName;
		}

		if($err = $obj->validate()){
			$items = implode(",", array_keys($err));
			$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		if(($_POST["telephoneHome"] == "") && ($_POST["postalAddress"] == "")){
			$this->sys->status = "<span class='editError'>Error you need to provide a valid home telephone number and/or valid postal address.</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		if(!$obj->gdprAcceptance){
			$this->sys->status = "<span class='editError'>Error you need to accept the data privacy agreement.</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		if(!isset($_POST["userPassword1"]) || ($_POST["userPassword1"] != $_POST["userPassword"])){
			$this->sys->status = "<span class='editError'>Error passwords are not the same</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		// Check Username is not already used
		$userName = dbEscapeString($obj->userName);
		$objList = $this->sys->db->getObjectList("User", "", "userName = '{$userName}'", "");
		if(count($objList)){
			$this->sys->status = "<span class='editError'>That user id is already in use, please try another. Your email address is a suitable userName.</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		// Check password
		if(strlen($_POST["userPassword1"]) < 6){
			$this->sys->status = "<span class='editError'>Error passwords need to be at least 6 characters long</span>";
			if($this->sys->loginUsernameEmail)
				$obj->emailAddress = "";
			return $this->register($obj);
		}

		// Modify some settings
		if($obj->userPassword != ""){
			$obj->userPassword = password_hash($obj->userPassword, PASSWORD_DEFAULT);
		}
		else {
			unset($obj->userPassword);
		}

		$obj->registerPassword = makeRandomPassword();
		$registerPassword = password_hash($obj->registerPassword, PASSWORD_DEFAULT);

		$this->sys->db->appendObject("User", $obj);

		// Check if a member of the organisation
		if(array_key_exists("membership", $this->sys->modules)){
			$obj->membershipId = $this->sys->modules["membership"]->userUpdateMembership($obj);
			$this->sys->db->putObject("User", $obj);
		}
		
		// Send register email to user
		$message = "Dear {$obj->name},\n\n";
		$message .= "You are now registered at our website, {$this->sys->siteName}.\n";
		$message .= "To activate your account, please login here:\n";
		$message .= "  {$this->sys->baseUrl}/user?cmd=registerActivate&id={$obj->id}&registerPassword={$registerPassword}\n\n";

		if($this->sys->loginRegisterValidate && $obj->adminValidate){
			$message .= "Your registration request will need to be accepted by an administrator. This will done as soon as we can.\n";
		}
		
		$message .= "This is an automated response, please do not reply by email.\n";

		emailList($obj->emailAddress, "Account request at {$this->sys->siteName}", $message, 0);

		return "<h1>New Account Registration</h1>Thank you, your account information has been emailed to your email address. Please check your email's and follow the directions on the email from us to complete registration.";
	}

	function registerActivate(){
		if(!isset($_GET["id"]) || !isset($_GET["registerPassword"])){
			$this->sys->pageDisplayError(new BError(1, "Incorrect parameters passed"));
			return "";
		}

		$obj = $this->sys->db->getObject("User", filterToInt($_GET["id"]));
		$pw1 = $_GET["registerPassword"];

		if($obj->enabled){
			return "You are already registered";
		}
		else if(password_verify($obj->registerPassword, $pw1)){
			$obj->registered = 1;

			$page = "<h1>New Account Registration</h1>";
			if($this->sys->loginRegisterValidate && $obj->adminValidate){
				$page .= "Thank you, your request for a login on the site has been sent to the site administrators for approval. We will attend to this as soon as possible.";
				$page .= " You will receive an email when we enable this account.<br>";
				$obj->pending = 1;
				if($this->test)
					$this->sys->siteEmailAdmin = $this->sys->testEmailAddress;

				if($this->sys->siteEmailAdmin){
					$subject = "{$this->sys->siteTitle}: Admin: New account registration";
					$message = "A new website account has been applied for. Please enable or delete using the following link.\n";
					$message .= "Name: {$obj->name}\n";
					$message .= "Edit: {$this->sys->baseUrl}/user?cmd=edit&id={$obj->id}\n";
					emailList($this->sys->siteEmailAdmin, $subject, $message, 0);
				}
			}
			else {
				$page .= "Thank you, you are now registered for a login on the site. Please Login using your username and password.";
				$obj->enabled = 1;
			}
			$page .= " You can now receive News/Forums items by email if you enabled this feature.<br>";
			if($this->sys->smsSend)
				$page .= "You will also receive mobile phone text alerts if you entered a mobile telephone number.";

			$this->sys->db->putObject("User", $obj);

			return $page;
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Your registration password is incorrect"));
			return "";
		}
	}

	function userPreferences(){
		if(!$this->sys->login->checkLogin("")){
			$this->sys->login = new BLogin($this->sys, "pageLogin", true);

			if(!$this->sys->login->performLogin()){
				exit();
			}

			if(!$this->sys->login->checkLogin("")){
				$this->sys->pageDisplayError(new BError(1, "You do not have login privilages needed to access this page"));
				exit();
			}
		}

		$page = "<h2>Edit User Preferences</h2>";

		$obj = $this->sys->db->getObject("User", $this->sys->login->id);
		if($this->sys->loginNoGroups && !$obj->membershipId){
			$group1Type = "hidden";
			$group2Type = "hidden";
		}
		else {
			$group1Type = "select";
			$group2Type = "selectMultiCheckbox";
		}

		if($this->checkAccess()){
			$adminEditCheckbox = "checkbox";
		}
		else {
			$adminEditCheckbox = "hidden";
		}

		$view = new BEditObj("user", "User Information", "?cmd=userPreferencesSave");
		if($this->sys->loginUsernameEmail){
			$view->setView(array(new BViewType("id", "", "hidden"),
				new BViewType("userPassword1", "New Password1", "password-new"),
				new BViewType("userPassword2", "New Password2", "password-new"),
				new BViewType("emailAddress", "Username/Email Address"),
				new BViewType("postalAddress", "Postal Address", "textarea", "", 300, 80),
				new BViewType("city", "Town/City"),
				new BViewType("postcode"),
				new BViewType("country", "", "select", $this->countryList()),
				new BViewType("telephoneWork"),
				new BViewType("telephoneHome"),
				new BViewType("telephoneMobile"),
				new BViewType("emailSend", "Admin emails (Admin, groups)", $adminEditCheckbox),
				new BViewType("forumEmailSend", "Forum Emails", "checkbox"),
				new BViewType("forsaleEmailSend", "Forsale Emails", "checkbox"),
				new BViewType("newsEmails", "News Emails", "checkbox"),
				new BViewType("newsNotifications", "News Notifications", "checkboxNotification"),
				new BViewType("group", "Group", $group1Type, array_merge(array(""), $this->sys->groups)),
				new BViewType("groups", "Groups of Interest", $group2Type, $this->sys->groups),
				//new BViewType("picture", "Picture", "pictureUploaderLineVerySmall"),
				new BViewType("picture", "Picture (for news items etc.)", "pictureUpload1_VerySmall"),
				));
		}
		else {
			$view->setView(array(new BViewType("id", "", "hidden"),
				new BViewType("userPassword1", "New Password1", "password-new"),
				new BViewType("userPassword2", "New Password2", "password-new"),
				new BViewType("emailAddress", "Email Address"),
				new BViewType("postalAddress", "Postal Address", "textarea", "", 300, 80),
				new BViewType("city", "Town/City"),
				new BViewType("postcode"),
				new BViewType("country", "", "select", $this->countryList()),
				new BViewType("telephoneWork"),
				new BViewType("telephoneHome"),
				new BViewType("telephoneMobile"),
				new BViewType("emailSend", "Send Emails (Admin, groups)", $adminEditCheckbox),
				new BViewType("forumEmailSend", "Send Forum Emails", "checkbox"),
				new BViewType("forsaleEmailSend", "Send Forsale Emails", "checkbox"),
				new BViewType("newsEmails", "News Emails", "checkbox"),
				new BViewType("newsNotifications", "News Notifications", "checkboxNotification"),
				new BViewType("group", "Group", $group1Type, array_merge(array(""), $this->sys->groups)),
				new BViewType("groups", "Groups of Interest", $group2Type, $this->sys->groups),
				//new BViewType("picture", "Picture", "pictureUploaderLineVerySmall"),
				new BViewType("picture", "Picture (for news items etc.)", "pictureUpload1_VerySmall"),
				));
		}

		$view->setHelp("userPassword1", "Enter your new password to override the default one.");
		$view->setHelp("userPassword2", "Enter your new password again to override the default one.");
		$view->setHelp("emailAddress", "Enter your new email address.");
		$view->setHelp("postalAddress", "Enter your postal address. Optional");
		$view->setHelp("city", "Enter your local city/town. Optional");
		$view->setHelp("postcode", "Enter your local city/town. Optional");
		$view->setHelp("telephoneWork", "Enter your work telephone number. Optional");
		$view->setHelp("telephoneHome", "Enter your home telephome number. Optional");
		$view->setHelp("telephoneMobile", "Enter your mobile number. Optional");
		$view->setHelp("emailSend", "Enable emails to be sent for admin, groups, news, events etc.");
		$view->setHelp("forumEmailSend", "Enable emails to be sent to the user on forum posts.");
		$view->setHelp("forsaleEmailSend", "Enable emails to be sent to the user on forsale posts.");
		$view->setHelp("roles", "Enter the system roles the user is allowed to perform.");
		$view->setHelp("group", "Enter the primary group this user belongs to.");
		$view->setHelp("groups", "Enter the other groups the user is interested in.");
		$view->setHelp("picture", "Upload a picture to be show on news posts etc.");

		$page .= "<div class='help'>";
		$page .= "Please edit any fields that need changing and click on the Save button to save the updated information.<br> ";
		$page .= "To change your password enter the new password in the New Password 1 and New Password 2 fields. ";
		if($this->sys->smsSend)
			$page .= "Note that if you enter a mobile number then you will receive text message alerts when sent.";
		$page .= "</div>";


		$obj->userPassword = "";
		$obj->userPassword1 = "";
		$obj->userPassword2 = "";
		$view->setObj($obj);

		$page = $page . $view->getHtml();

		return $page;
	}

	function pictureSave($filename, $type, $id){
		// Saves a user picture as a square thumb picture
		$dirname = "/files/user";
		$thumbSize = 40;

		//debug_log("pictureSave: {$fileName} {$type} {$id}\n");

		$maxsize = 1024;
		$s = getimagesize($filename);
		$aspect = (float)$s[0] / (float)$s[1];

		//debug_log("pictureSave: original: {$s[0]} x {$s[1]}\n");
		if($aspect > 1.0){
			$w = $s[0];
			if($w > $maxsize)
				$w = $maxsize;
			$h = (int)((float)$w/$aspect);
			$th = $thumbSize;
			$tw = (int)round((float)$th * $aspect);
			$tx = ($s[0] - $s[1]) / 2;
			$ty = 0;
		}
		else {
			$h = $s[1];
			if($h > $maxsize)
				$h = $maxsize;
			$w = (int)((float)$h * $aspect);
			$tw = $thumbSize;
			$th = (int)round((float)$tw/$aspect);
			$tx = 0;
			$ty = ($s[1] - $s[0]) / 2;
		}

		//debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
		//print "Stype: '{$type}'<br>";

		if(($type == "jpg") || ($type == "jpeg"))
			$image = imagecreatefromjpeg($filename);
		else if($type == "png")
			$image = imagecreatefrompng($filename);
		else if($type == "gif")
			$image = imagecreatefromgif($filename);

		if($image){
			$pic = imagecreatetruecolor($w, $h);
			$thumb = imagecreatetruecolor($thumbSize, $thumbSize);
			imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
			imagejpeg($pic, $this->sys->baseDir . "/" . $dirname . "/picture-{$id}.jpg");

			imagecopyresized($thumb, $image, 0, 0, $tx, $ty, $tw, $th, $s[0], $s[1]);
			imagejpeg($thumb, $this->sys->baseDir . "/" . $dirname . "/picture-{$id}-thumb.jpg");
		}
	}

	function fixPictures(){
		$thumbSize = 40;
		$type = "jpg";

		$dirname = $this->sys->baseDir . "/files/user";
		if($dir = opendir($dirname)){
		        while(($filename = readdir($dir)) !== false){
				if(fnmatch("picture*", $filename) && (fnmatch("*thumb*", $filename)) == 0){
					$id = intval(substr($filename, 8));
					echo "filename: $filename : id: {$id}, filetype: " . filetype($dirname . "/" . $filename) . "</br>\n";
					
					$filename = $dirname . "/" . $filename;
					print("Do {$filename}<br>");
					
					$maxsize = 1024;
					$s = getimagesize($filename);
					$aspect = (float)$s[0] / (float)$s[1];

					//debug_log("pictureSave: original: {$s[0]} x {$s[1]}\n");
					if($aspect > 1.0){
						$w = $s[0];
						if($w > $maxsize)
							$w = $maxsize;
						$h = (int)((float)$w/$aspect);
						$th = $thumbSize;
						$tw = (int)round((float)$th * $aspect);
						$tx = ($s[0] - $s[1]) / 2;
						$ty = 0;
					}
					else {
						$h = $s[1];
						if($h > $maxsize)
							$h = $maxsize;
						$w = (int)((float)$h * $aspect);
						$tw = $thumbSize;
						$th = (int)round((float)$tw/$aspect);
						$tx = 0;
						$ty = ($s[1] - $s[0]) / 2;
					}

					//debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
					//print "Stype: '{$type}'<br>";

					if(($type == "jpg") || ($type == "jpeg"))
						$image = imagecreatefromjpeg($filename);
					else if($type == "png")
						$image = imagecreatefrompng($filename);
					else if($type == "gif")
						$image = imagecreatefromgif($filename);
					
					print("Image: {$image}"); print("<br>\n");
					if($image){
						#$pic = imagecreatetruecolor($w, $h);
						#imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
						#imagejpeg($pic, $this->sys->baseDir . "/" . $dirname . "/picture-{$id}.jpg");

						#print("Create thumb at: " . 
						$thumb = imagecreatetruecolor($thumbSize, $thumbSize);
						imagecopyresized($thumb, $image, 0, 0, $tx, $ty, $tw, $th, $s[0], $s[1]);
						unlink($dirname . "/picture-{$id}-thumb.jpg");
						imagejpeg($thumb, $dirname . "/picture-{$id}-thumb.jpg");
					}
				}
			}
			closedir($dir);
		}
	}

	function addPicture($id, $file){
		//debug_log("addPicture: Do file: {$file["tmp_name"]}\n");
		if($obj = $this->sys->db->getObject("User", $id)){
			// Add picture
			$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
			$this->pictureSave($file["tmp_name"], $ext, $obj->id);

			$obj->picturePresent = 1;
			$this->sys->db->putObject("User", $obj);
		}
	}
	
	function userPreferencesSave(){
		//print_r($_GET);	print_r($_POST); print_r($_FILES);

		if(!$this->sys->login->checkLogin("")){
			$this->sys->pageDisplayError(new BError(1, "You do not have login privilages needed to access this page"));
			exit();
		}

		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = $this->sys->db->getObject("User", $this->sys->login->id);
			$objOld = clone $obj;
			$objNew = $this->sys->db->getObject("User", $this->sys->login->id);
			$objNew->set($_POST);

			if(($this->sys->loginUsernameEmail) && ($objNew->emailAddress != $obj->emailAddress)){
				if($this->checkIfUserExists($objNew->emailAddress)){
					$this->sys->status = "<span class='editError'>There is already a user with the Username: {$objNew->emailAddress}</span>";
					return $this->userPreferences($objNew);
				}
			}

			// Validate the save
			if(!$this->sys->postValidateCheck($objNew->id)){
				$this->sys->pageDisplayError(new BError(1, "Save validation error. You may have accessed other sites pages in the background."));
				exit();
			}

			if($err = $objNew->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->userPreferences($objNew);
			}

			if($objNew->id != $this->sys->login->id){
				$this->sys->pageDisplayError(new BError(1, "Your login id is incorrect"));
				exit();
			}
			
			if($this->sys->loginUsernameEmail){
				$obj->userName = $objNew->emailAddress;
			}
			$obj->emailAddress = $objNew->emailAddress;
			$obj->telephoneHome = $objNew->telephoneHome;
			$obj->telephoneMobile = $objNew->telephoneMobile;
			$obj->telephoneWork = $objNew->telephoneWork;
			$obj->postalAddress = $objNew->postalAddress;
			$obj->city = $objNew->city;
			$obj->postcode = $objNew->postcode;
			$obj->country = $objNew->country;
			$obj->emailSend = $objNew->emailSend;
			$obj->forumEmailSend = $objNew->forumEmailSend;
			$obj->forsaleEmailSend = $objNew->forsaleEmailSend;
			$obj->newsEmails = $objNew->newsEmails;
			$obj->newsNotifications = $objNew->newsNotifications;
			$obj->group = $objNew->group;
			$obj->groups = $objNew->groups;

			// Modify some settings
			if(isset($_POST['userPassword1']) && (filterString($_POST['userPassword1']) != "")){
				$userPassword1 = filterString($_POST['userPassword1']);
				$userPassword2 = filterString($_POST['userPassword2']);
				if($userPassword1 != $userPassword2){
					$this->sys->pageDisplayError(new BError(1, "Passwords are not the same."));
					exit();
				}
				else {
					$obj->userPassword = password_hash($userPassword1, PASSWORD_DEFAULT);
				}
			}
			else {
				unset($obj->userPassword);
			}

			if($obj->id)
				$this->sys->db->putObject("User", $obj);

			// Upload any pictures present
			if((count($_FILES) == 1) && isset($_FILES["picture"]) && ($_FILES["picture"]["tmp_name"] != "")){
				$file = $_FILES["picture"];
				//debug_log("uploadPictures: Do file: {$file['tmp_name']}\n");

				// Add picture
				$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
				//pictureSaveDir($file["tmp_name"], $ext, "/files/user", $obj->id, 40);
				
				$this->pictureSave($file["tmp_name"], $ext, $obj->id);

				$obj->picturePresent = 1;
				$this->sys->db->putObject("User", $obj);
			}

			// Update membership record if needed
			$changedReason = "";
			if($obj->emailAddress != $objOld->emailAddress) 
				$changedReason .= "Email address changed, ";
			if($obj->telephoneHome != $objOld->telephoneHome) 
				$changedReason .= "Home telephone changed, ";
			if($obj->telephoneMobile != $objOld->telephoneMobile) 
				$changedReason .= "Mobile telephone changed, ";
			if($obj->telephoneWork != $objOld->telephoneWork) 
				$changedReason .= "Work telephone changed, ";
			if($obj->postalAddress != $objOld->postalAddress) 
				$changedReason .= "Postal address changed, ";
			if($obj->city != $objOld->city) 
				$changedReason .= "City address changed, ";
			if($obj->postcode != $objOld->postcode) 
				$changedReason .= "Postcode address changed, ";

			//debug_log("userPreferencesSave: reason: '{$changedReason}'");
			if(strlen($changedReason) && (array_key_exists("membership", $this->sys->modules)))
				$this->sys->modules["membership"]->userUpdate($obj, $changedReason);

			return "Your preferences have been updated.";
		}
	}
	
	function deleteAccount(){
		if(!$this->sys->login->checkLogin("")){
			$this->sys->pageDisplayError(new BError(1, "You do not have login privilages needed to access this page"));
			exit();
		}

		$page = "<h2>Delete My Account</h2>";

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteAccountDo' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete your account ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete Account'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page .= $f->render();

		return $page;
	}

	function deleteAccountDo(){
		if(!$this->sys->login->checkLogin("")){
			$this->sys->pageDisplayError(new BError(1, "You do not have login privilages needed to access this page"));
			exit();
		}

		$page = "";
		$this->sys->db->deleteObject("User", $this->sys->login->id);

		$this->sys->login->logout();
		header("Location: ?cmd=logoutDone");
		return $page;
	}

	function forgottenPassword(){
		$page = "<h2>Forgotten My Username and/or Password</h2>";
		
		$page .= "If you have forgotten your username or password please enter ";
		$page .= "your email address below and click on the save button.<br>";
		$page .= "An email will be sent to you with your Username and a link that will allow ";
		$page .= "you to change your password.";
		$page .= "<br/>You will need to keep this browser window open and click on the link in the email when it arrives.";
		$page .= "This is for increased security.";

		$view = new BEditObj("Email", "Email Address", "?cmd=forgottenPasswordEmail");
		$view->setView(array(new BViewType("emailAddress", "Email Address")));
		$view->setObj(new BObjUser());
		$view->setSaveButtonTitle("Send");

		$page = $page . $view->getHtml();

		return $page;
	}

	function forgottenPasswordEmail(){
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjUser();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->forgottenPassword();
			}

			// Find user with email address
			$obj->emailAddress = dbEscapeString($obj->emailAddress);
			$objList = $this->sys->db->getObjectList("User", "", "emailAddress = '{$obj->emailAddress}'", "");
			if(count($objList)){
				$user = $objList[0];
				$seed = rand();
				$_SESSION["userid"] = $user->id;
				$_SESSION["seed"] = $seed;
				$message = "Someone, hopefully you, have requested information on your login details.\n";
				$message .= "If you have not requested this, please ignore this message.\n\n";
				$message .= "Your user name for the website: {$this->sys->siteName} is: {$user->userName}\n";
				$message .= "To receive a new password please click on or enter the following link into your web browser\n\n";
				$message .= "<a href='{$this->sys->baseUrl}/user?cmd=forgottenPasswordChange&seed={$seed}'>{$this->sys->baseUrl}/user?cmd=forgottenPasswordChange&seed={$seed}</a>\n\n";
				$message .= "Note you need to do this while your WEB browser still has the TC website open. from the request\n";

				emailList($obj->emailAddress, "{$this->sys->siteName}: User Details", $message, 1);
		
				$page = "An email has been sent to that address with your username and a link to allow you to obtain a new password";		
				$page .= "<br/>You will need to keep this browser window open and click on the link in the email when it arrives.";
				$page .= "This is for increased security.";

				return $page;
			}
			else {
				return "No user with that email address is known";
			}
		}
		return "";
	}
	
	function forgottenPasswordChange(){
		$page = "<h2>Password change</h2>";
		
		if(isset($_GET["seed"]) && ($_GET["seed"] == $_SESSION["seed"])){
			// Create a new password and change it
			$password = passwordGenerate();
			$user = $this->sys->db->getObject("User", $_SESSION['userid']);
			$user->userPassword = password_hash($password, PASSWORD_DEFAULT);
			$this->sys->db->putObject("User", $user);
			
			$page .= "Your password has been changed to: {$password} for Username: {$user->userName}<br>\n";
			$page .= "Please login using your username and password and you can then change this password ";
			$page .= "if required.";
			unset($_SESSION["seed"]);

			$html = new BLoginForm("/");

			$page .= $html->render();
		}
		else {
			$page .= "Password cannot be changed as different browser session is being used from that ";
			$page .= "which sent the password change email.";
		}
		return $page;
	}

	function getUserInfo($id){
		return $this->sys->db->getObject("User", $id);
	}

	function updateAccessTime($id){
		$date = new BDate("now");
		$obj = new StdClass();
		$obj->id = $id;
		$obj->lastAccessTime = $date->getDateTime();
		
		$this->sys->db->putObjectNoError("User", $obj);
	}

	function checkIfUserExists($userName){
		// Check Username is already used
		$userName = dbEscapeString($userName);
		$objList = $this->sys->db->getObjectList("User", "", "userName = '{$userName}'", "");
		if(count($objList)){
			return 1;
		}
		else {
			return 0;
		}
	}

	function countryList(){
		$l = array("");
		if($maps = $this->sys->module("maps")){
			$l = array_merge($l, $maps->getCountryList());
		}
		return $l;
	}
	
	function groupsList(){
		$groups = array();
		if($this->sys->db->tableExists("Groups")){
			$list = $this->sys->db->getObjectList("Groups", "", "", "name", "");

			foreach($list as $g){
	//			$groups[$g->name] = $g->title;
				$groups[$g->name] = $g->name;
			}
		}
		return $groups;
	}
	
	function getUserList($name){
		if($name != "")
			$where = "`name` like '%{$name}%";
		else
			$where = "";

		$list = $this->sys->db->getObjectList("User", "id,name,emailAddress,postcode,telephoneHome", $where, "name", "");
		return $list;
	}
	
	function getPicture($id){
		if($obj = $this->sys->db->getObject("User", $id)){
			if(file_exists("{$this->sys->baseDir}/files/user/picture-${id}-thumb.jpg"))
				return "{$this->sys->baseUrl}/files/user/picture-${id}-thumb.jpg";
		}
		return "";
	}
	
	function statistics(){
		$date = new BDate("now");
		$date->addSeconds(-6*3600);	// 6 hours


		$f = new BHtmlNode("div");
		$f->appendNode("h3", "", "User Statistics");
		$t = &$f->appendNode("table", "class='BView'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Item");
		$r->appendNode("th", "align='left'", "Value");
		
		$value = $this->sys->db->getObjectNumber("User", "");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total users");
		$r->appendNode("td", "", $value);
		
		$value = $this->sys->db->getObjectNumber("User", "`membershipId`='0'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total non membership users");
		$r->appendNode("td", "", $value);

		$where = "`lastAccessTime` > '{$date->getDateTime()}'";
		$value = $this->sys->db->getObjectNumber("User", $where);
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Logged in users");
		$r->appendNode("td", "", $value);
		
		$value = $this->sys->db->getObjectNumber("User", "`emailSend`='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total emailSend users");
		$r->appendNode("td", "", $value);

		$value = $this->sys->db->getObjectNumber("User", "`forumEmailSend`='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total forumEmailSend users");
		$r->appendNode("td", "", $value);

		$value = $this->sys->db->getObjectNumber("User", "`forsaleEmailSend`='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total forsaleEmailSend users");
		$r->appendNode("td", "", $value);

		$value = $this->sys->db->getObjectNumber("User", "`newsEmails`='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total newsEmails users");
		$r->appendNode("td", "", $value);

		$value = $this->sys->db->getObjectNumber("User", "`newsNotifications`='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total newsNotifications users");
		$r->appendNode("td", "", $value);

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("name"),
			new BViewType("emailSend", "Emails"),
			new BViewType("forumEmailSend", "Forum Emails"),
			new BViewType("forsaleEmailSend", "ForSale Emails"),
			new BViewType("emailAddress", "Email Address"),
			new BViewType("telephoneHome", "Home Telephone"),
			new BViewType("telephoneMobile", "Mobile Telephone"),
			new BViewType("membershipId"),
			new BViewType("group"),
			new BViewType("roles", "Roles"),
			));
		
		$view->setSelection($where);
		$view->setTable("User");
		//$view->setOrder($order);
		//$view->setLimit($sel->pagerGetLimitString());
		//$totalNumber = $this->sys->db->getObjectNumber("User", $where);
		//$sel->pagerSet(0, $listSize, $totalNumber);
		
		$page = $f->render();
		$page .= "<h3>Users logged in</h3>";
		$page .= $view->getHtml();

		return $page;
	}
	
	var $rolesAll = array("admin","adminUsers","adminDelete");
};

function userGetName($id){
	global $sys;
	
	return $sys->modules["user"]->getUserInfo($id)->name;
}

function userGetPicture($id){
	global $sys;
	
	return $sys->modules["user"]->getPicture($id);
}

function userGetLoginBlock(){
	global $sys;
	
	return $sys->modules["user"]->getLoginBlock("/")->render();
}
?>
